Package: reactRouter
Type: Package
Title: 'React Router' for 'shiny' Apps and 'Quarto'
Version: 0.2.0
Authors@R: c(
    person(given = "Felix", family = "Luginbuhl", role = c("aut", "cre", "cph"), email = "felix.luginbuhl@protonmail.ch", comment = c(ORCID = "0009-0008-6625-2899")),
    person(given = "Andryas", family = "Waurzenczak", role = c("ctb"))
    )
Maintainer: Felix Luginbuhl <felix.luginbuhl@protonmail.ch>
Description: Provides a wrapper around the 'react-router-dom' 'React' library for use in
  'Shiny' applications and 'Quarto' documents. Enables client-side routing with hash,
  memory, and browser history strategies, nested routes, dynamic segments, data loaders,
  actions, and navigation hooks.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.0)
Imports: htmltools, shiny, shiny.react, checkmate, jsonlite
Suggests: testthat (>= 3.0.0), chromote (>= 0.1.1), shinytest2, dplyr,
        muiMaterial
RoxygenNote: 7.3.3
URL: https://felixluginbuhl.com/reactRouter/
BugReports: https://github.com/lgnbhl/reactRouter/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-10 10:03:04 UTC; felix
Author: Felix Luginbuhl [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0008-6625-2899>),
  Andryas Waurzenczak [ctb]
Repository: CRAN
Date/Publication: 2026-05-10 10:20:02 UTC
Built: R 4.5.2; ; 2026-05-10 12:40:29 UTC; unix

Package: reactRouter
Type: Package
Title: 'React Router' for 'shiny' Apps and 'Quarto'
Version: 0.2.1
Authors@R: c(
    person(given = "Felix", family = "Luginbuhl", role = c("aut", "cre", "cph"), email = "felix.luginbuhl@protonmail.ch", comment = c(ORCID = "0009-0008-6625-2899")),
    person(given = "Andryas", family = "Waurzenczak", role = c("ctb"))
    )
Maintainer: Felix Luginbuhl <felix.luginbuhl@protonmail.ch>
Description: Provides a wrapper around the 'react-router-dom' 'React' library for use in
  'Shiny' applications and 'Quarto' documents. Enables client-side routing with hash,
  memory, and browser history strategies, nested routes, dynamic segments, data loaders,
  actions, and navigation hooks.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.0)
Imports: htmltools, shiny, shiny.react, checkmate, jsonlite
Suggests: testthat (>= 3.0.0), chromote (>= 0.1.1), shinytest2, dplyr,
        muiMaterial
RoxygenNote: 7.3.3
URL: https://felixluginbuhl.com/reactRouter/
BugReports: https://github.com/lgnbhl/reactRouter/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-09-01 08:00:31 UTC; felix
Author: Felix Luginbuhl [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0008-6625-2899>),
  Andryas Waurzenczak [ctb]
Repository: CRAN
Date/Publication: 2026-09-01 08:20:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(print,reactRouter)
export(Await)
export(BrowserRouter)
export(Form)
export(HashRouter)
export(JS)
export(Link)
export(Link.shinyInput)
export(MemoryRouter)
export(NavLink)
export(NavLink.shinyInput)
export(Navigate)
export(Outlet)
export(Route)
export(RouterProvider)
export(Routes)
export(ScrollRestoration)
export(createBrowserRouter)
export(createHashRouter)
export(createMemoryRouter)
export(createRoutesFromElements)
export(dataResponse)
export(isRouteErrorResponse)
export(reactRouterDependency)
export(reactRouterExample)
export(redirect)
export(redirectDocument)
export(replaceResponse)
export(updateLink.shinyInput)
export(updateNavLink.shinyInput)
export(useActionData)
export(useAsyncError)
export(useAsyncValue)
export(useBlocker)
export(useFetcher)
export(useFetchers)
export(useHref)
export(useInRouterContext)
export(useLinkClickHandler)
export(useLoaderData)
export(useLocation)
export(useMatch)
export(useMatches)
export(useNavigate)
export(useNavigation)
export(useNavigationType)
export(useOutlet)
export(useOutletContext)
export(useParams)
export(useResolvedPath)
export(useRevalidator)
export(useRouteError)
export(useRouteLoaderData)
export(useRoutes)
export(useSearchParams)
export(useSubmit)
export(useViewTransitionState)
importFrom(shiny.react,JS)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# reactRouter <img src="man/figures/logo.png" align="right" height="138" />

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/reactRouter)](https://CRAN.R-project.org/package=reactRouter)
[![Grand
total](https://cranlogs.r-pkg.org/badges/grand-total/reactRouter)](https://cran.r-project.org/package=reactRouter)
[![R-CMD-check](https://github.com/lgnbhl/reactRouter/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/lgnbhl/reactRouter/actions/workflows/R-CMD-check.yaml)
[![](https://img.shields.io/badge/react--router--dom-7.15.0-blue.svg)](https://reactrouter.com)
[![LinkedIn](https://img.shields.io/badge/LinkedIn-Follow-E4405F?style=social&logo=linkedin)](https://www.linkedin.com/in/FelixLuginbuhl)
<!-- badges: end -->

`reactRouter` brings [React Router](https://reactrouter.com), the
world’s most popular React routing library, to R and Shiny.

### Why reactRouter?

`reactRouter` works with Shiny apps and Quarto documents — and also on
its own. It extends whatever you are already building with new features:

- **URL-based navigation** — your application behaves like a real
  website, with a home page and multiple sub-pages (e.g. one per report
  or analysis), each with its own shareable URL.
- **Reactivity without a server** — data loaders fetch and display data
  as users navigate, entirely in the browser, without requiring a
  running R server.
- **Static hosting** — the output is plain HTML/JS, deployable to GitHub
  Pages, Netlify, Posit Connect, or any static file host (if you don’t
  use it with Shiny).
- **AI-friendly** — React Router is one of the most widely used
  JavaScript libraries, so AI tools like Claude, ChatGPT, or Copilot
  know it well. Ask an AI to generate React Router code, adapt it to R,
  and you’re done — no React knowledge needed.

Used alone, `reactRouter` brings the best of both worlds: the
**reactivity of Shiny** (data that updates as users interact) and the
**simplicity of Quarto** (multiple pages as static files, hostable
anywhere, no server to maintain or pay for).

### Usage

Create **URL pages** easily:

``` r
library(reactRouter)
library(htmltools)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        NavLink(to = "/", "Main"), " | ",
        NavLink(to = "/analysis", "Analysis"), hr(),
        Outlet()
      ),
      Route(index = TRUE, element = "Main content"),
      Route(path = "analysis", element = "Analysis content")
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)
```

Or use **data loaders** to fetch and display data client-side — no R
server needed:

``` r
library(reactRouter)
library(htmltools)

# local or fetch an API
people_json <- jsonlite::toJSON(dplyr::starwars, dataframe = "rows", auto_unbox = TRUE)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        NavLink(to = "/", "Home"), " | ",
        NavLink(to = "/people/1", "Luke"),
        hr(), Outlet()
      ),
      Route(index = TRUE, element = p("Try #/people/4 for Darth Vader.")),
      Route(
        path = "people/:id",
        loader = JS(sprintf("({ params }) => (%s)[params.id - 1]", people_json)),
        element = div(
          useLoaderData(tags$h3(), selector = "name"),
          useLoaderData(tags$p(),  selector = "homeworld")
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)
```

### Install

``` r
install.packages("reactRouter")
# remotes::install_github("lgnbhl/reactRouter") # development version
```

### Resources

- [Package documentation](https://felixluginbuhl.com/reactRouter/)
- [Getting Started
  vignette](https://felixluginbuhl.com/reactRouter/articles/introduction.html)
- [All R
  examples](https://github.com/lgnbhl/reactRouter/tree/main/inst/examples)

### Acknowledgements

`reactRouter` is built on top of
[shiny.react](https://github.com/Appsilon/shiny.react), the R package by
[Appsilon](https://www.appsilon.com/) that makes it possible to use
React components in Shiny and Quarto.

Data loaders work seamlessly with any R package that uses `shiny.react`
under the hood. This includes the [MUI](https://mui.com) component
ecosystem for R:

- [muiMaterial](https://github.com/lgnbhl/muiMaterial) — Material UI
  components (buttons, dialogs, tabs, and more)
- [muiDataGrid](https://github.com/lgnbhl/muiDataGrid) — interactive
  data grids
- [muiCharts](https://github.com/lgnbhl/muiCharts) — charts and
  visualisations
- [muiTreeView](https://github.com/lgnbhl/muiTreeView) — tree view
  components

Learn more about how to use data loaders with these R packages
[here](https://felixluginbuhl.com/reactRouter/articles/useLoaderData.html).

### Contribute

Found a bug or have a feature request? [Open an
issue](https://github.com/lgnbhl/reactRouter/issues). Pull requests are
welcome.

Follow [Felix Luginbuhl](https://linkedin.com/in/FelixLuginbuhl) on
LinkedIn for updates.

<!-- README.md is generated from README.Rmd. Please edit that file -->

# reactRouter <img src="man/figures/logo.png" align="right" height="138" />

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/reactRouter)](https://CRAN.R-project.org/package=reactRouter)
[![Grand
total](https://cranlogs.r-pkg.org/badges/grand-total/reactRouter)](https://cran.r-project.org/package=reactRouter)
[![R-CMD-check](https://github.com/lgnbhl/reactRouter/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/lgnbhl/reactRouter/actions/workflows/R-CMD-check.yaml)
[![](https://img.shields.io/badge/react--router--dom-7.18.2-blue.svg)](https://reactrouter.com)
[![LinkedIn](https://img.shields.io/badge/LinkedIn-Follow-E4405F?style=social&logo=linkedin)](https://www.linkedin.com/in/FelixLuginbuhl)
<!-- badges: end -->

`reactRouter` brings [React Router](https://reactrouter.com), the
world’s most popular React routing library, to R and Shiny.

### Why reactRouter?

`reactRouter` works with Shiny apps and Quarto documents — and also on
its own. It extends whatever you are already building with new features:

- **URL-based navigation** — your application behaves like a real
  website, with a home page and multiple sub-pages (e.g. one per report
  or analysis), each with its own shareable URL.
- **Reactivity without a server** — data loaders fetch and display data
  as users navigate, entirely in the browser, without requiring a
  running R server.
- **Static hosting** — the output is plain HTML/JS, deployable to GitHub
  Pages, Netlify, Posit Connect, or any static file host (if you don’t
  use it with Shiny).
- **AI-friendly** — React Router is one of the most widely used
  JavaScript libraries, so AI tools like Claude, ChatGPT, or Copilot
  know it well. Ask an AI to generate React Router code, adapt it to R,
  and you’re done — no React knowledge needed.

Used alone, `reactRouter` brings the best of both worlds: the
**reactivity of Shiny** (data that updates as users interact) and the
**simplicity of Quarto** (multiple pages as static files, hostable
anywhere, no server to maintain or pay for).

### Usage

Create **URL pages** easily:

``` r
library(reactRouter)
library(htmltools)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        NavLink(to = "/", "Main"), " | ",
        NavLink(to = "/analysis", "Analysis"), hr(),
        Outlet()
      ),
      Route(index = TRUE, element = "Main content"),
      Route(path = "analysis", element = "Analysis content")
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)
```

Or use **data loaders** to fetch and display data client-side — no R
server needed:

``` r
library(reactRouter)
library(htmltools)

# local or fetch an API
people_json <- jsonlite::toJSON(dplyr::starwars, dataframe = "rows", auto_unbox = TRUE)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        NavLink(to = "/", "Home"), " | ",
        NavLink(to = "/people/1", "Luke"),
        hr(), Outlet()
      ),
      Route(index = TRUE, element = p("Try #/people/4 for Darth Vader.")),
      Route(
        path = "people/:id",
        loader = JS(sprintf("({ params }) => (%s)[params.id - 1]", people_json)),
        element = div(
          useLoaderData(tags$h3(), selector = "name"),
          useLoaderData(tags$p(),  selector = "homeworld")
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)
```

### Install

``` r
install.packages("reactRouter")
# remotes::install_github("lgnbhl/reactRouter") # development version
```

### Resources

- [Package documentation](https://felixluginbuhl.com/reactRouter/)
- [Getting Started
  vignette](https://felixluginbuhl.com/reactRouter/articles/introduction.html)
- [All R
  examples](https://github.com/lgnbhl/reactRouter/tree/main/inst/examples)

### Acknowledgements

`reactRouter` is built on top of
[shiny.react](https://github.com/Appsilon/shiny.react), the R package by
[Appsilon](https://www.appsilon.com/) that makes it possible to use
React components in Shiny and Quarto.

Data loaders work seamlessly with any R package that uses `shiny.react`
under the hood. This includes the [MUI](https://mui.com) component
ecosystem for R:

- [muiMaterial](https://github.com/lgnbhl/muiMaterial) — Material UI
  components (buttons, dialogs, tabs, and more)
- [muiDataGrid](https://github.com/lgnbhl/muiDataGrid) — interactive
  data grids
- [muiCharts](https://github.com/lgnbhl/muiCharts) — charts and
  visualisations
- [muiTreeView](https://github.com/lgnbhl/muiTreeView) — tree view
  components

Learn more about how to use data loaders with these R packages
[here](https://felixluginbuhl.com/reactRouter/articles/useLoaderData.html).

### Contribute

Found a bug or have a feature request? [Open an
issue](https://github.com/lgnbhl/reactRouter/issues). Pull requests are
welcome.

Follow [Felix Luginbuhl](https://linkedin.com/in/FelixLuginbuhl) on
LinkedIn for updates.

# reactRouter examples

Each subfolder is a runnable example. Launch any of them with:

```r
reactRouter::reactRouterExample("<name>")
# or
shiny::runApp(system.file("examples", "<name>", package = "reactRouter"))
```

The recommended entry point is `RouterProvider()` paired with one of the
`create*Router()` functions. See
`vignette("routers", package = "reactRouter")` for guidance on choosing a
router and `vignette("introduction", package = "reactRouter")` for a tour.

## Showcase

| Example | What it shows |
|---|---|
| [`basic`](basic/) | Layout route, nested routes, `Outlet()`, `Link()`, splat `*` route |
| [`bslib`](bslib/) | reactRouter inside a `bslib::page_navbar()` |
| [`star-wars-explorer`](star-wars-explorer/) | Larger app: dynamic segments, loaders, `errorElement`, MUI Material/Charts/DataGrid integration |
| [`shiny.fluent`](shiny.fluent/) | Multi-page `rhino` + `shiny.fluent` dashboard (see folder README) |

## Routers

| Example | What it shows |
|---|---|
| [`RouterProvider`](RouterProvider/) | The recommended entry point |
| [`createHashRouter`](createHashRouter/) | Hash-based data router (default choice) |
| [`createMemoryRouter`](createMemoryRouter/) | In-memory data router (wizards, embedded widgets) |
| [`MemoryRouter`](MemoryRouter/) | Legacy component-based memory router |

## Routes & navigation

| Example | What it shows |
|---|---|
| [`dynamic-segment`](dynamic-segment/) | Minimal `Route(path = ":id")` + `useParams()` |
| [`useNavigate`](useNavigate/) | Programmatic navigation |
| [`useNavigation`](useNavigation/) | Reading navigation state |
| [`useLocation`](useLocation/) | Reading the current location |
| [`useParams`](useParams/) | Reading route parameters |
| [`useSearchParams`](useSearchParams/) / [`setSearchParams`](setSearchParams/) | Reading and updating search params |
| [`useOutlet`](useOutlet/) / [`useOutletContext`](useOutletContext/) | Outlet inspection and parent → child context |
| [`useLinkClickHandler`](useLinkClickHandler/) | Custom link components |
| [`useViewTransitionState`](useViewTransitionState/) | View transitions |
| [`useRoutes`](useRoutes/) | Routes-as-data (legacy `Routes()` API) |
| [`useInRouterContext`](useInRouterContext/) | Detect whether you are inside a router |
| [`ScrollRestoration`](ScrollRestoration/) | Restore scroll position on navigation |

## Data loading & actions

| Example | What it shows |
|---|---|
| [`useLoaderData`](useLoaderData/) | The basic `loader` + `useLoaderData()` pattern |
| [`useLoaderData-api`](useLoaderData-api/) | Loader fetching from a remote API |
| [`Form`](Form/) | `Form()` GET + loader, POST + action with `useActionData()` |
| [`useFetcher`](useFetcher/) / [`useFetchers`](useFetchers/) | Background fetches without navigation |
| [`useSubmit`](useSubmit/) | Programmatic form submission |
| [`useRevalidator`](useRevalidator/) | Re-run the current loader on demand |
| [`shouldRevalidate`](shouldRevalidate/) | Skip loader re-runs when data is unchanged |
| [`Await`](Await/) / [`useAsyncValue`](useAsyncValue/) / [`useAsyncError`](useAsyncError/) | Deferred / streaming loader data |
| [`useBlocker`](useBlocker/) | Intercept navigation (e.g. unsaved-changes prompt) |

## Loader/action helpers

| Example | What it shows |
|---|---|
| [`redirect`](redirect/) | `redirect()` from a loader |
| [`redirectDocument`](redirectDocument/) | Full-document redirect |
| [`replaceResponse`](replaceResponse/) | Replace history entry from a loader |
| [`dataResponse`](dataResponse/) | Loader returning a typed `Response` |
| [`useRouteError`](useRouteError/) | Catching loader/action errors |

## Notes

- Most examples use the *data router* API (`createHashRouter()` +
  `RouterProvider()`), which supports `loader`, `action`, `useLoaderData()`,
  `useFetcher()`, etc. The [`MemoryRouter`](MemoryRouter/) example uses the
  older *component-style* API (`MemoryRouter()` + `Routes()` + `Route()`) —
  still supported in React Router v7, but new code should prefer the data
  router style.
- `RouterProvider()` examples render as static HTML and can also be embedded
  in Quarto / R Markdown documents — many use `htmltools::browsable()`
  rather than `shinyApp()`.

# Minimal example showing useSearchParams() which exposes useSearchParams()
# useSearchParams reads URL query parameters (read-only)

library(reactRouter)
library(htmltools)

Layout <- div(
  tags$h2("useSearchParams Example"),
  tags$nav(tags$ul(
    tags$li(NavLink(to = "/?color=red", "Red")),
    tags$li(NavLink(to = "/?color=blue&size=large", "Blue Large")),
    tags$li(NavLink(to = "/?tag=news&tag=sports&tag=tech", "Multiple tags")),
    tags$li(NavLink(to = "/", "Clear"))
  )),
  tags$hr(),
  tags$h3("color param (single value):"),
  useSearchParams(tags$span(), param = "color"),
  tags$h3("size param (often absent):"),
  useSearchParams(tags$span(style = "color: gray"), param = "size"),
  tags$h3("tag param (can repeat):"),
  useSearchParams(tags$span(), param = "tag"),
  tags$h3("tag param, rendered as a list:"),
  useSearchParams(
    param = "tag",
    render = JS(
      "(tags) => tags.length
        ? React.createElement('ul', null, tags.map((t, i) =>
            React.createElement('li', { key: i }, t)))
        : React.createElement('em', null, '(none)')"
    )
  ),
  Outlet()
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE, element = div(tags$p("Home page")))
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# Minimal example showing useParams() which exposes useParams()
# useParams extracts dynamic segments from the URL (e.g. :id)

library(reactRouter)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        tags$h2("useParams Example"),
        tags$nav(tags$ul(
          tags$li(NavLink(to = "/", "Home")),
          tags$li(NavLink(to = "/user/42", "User 42")),
          tags$li(NavLink(to = "/user/99", "User 99"))
        )),
        tags$hr(),
        Outlet()
      ),
      Route(
        index = TRUE,
        element = div(tags$p("Select a user above."))
      ),
      Route(
        path = "user/:id",
        element = div(
          tags$h3("User Detail"),
          tags$p(
            "User ID: ",
            useParams(tags$span(), selector = "id")
          )
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# useRevalidator() shows whether the current route's loader is being re-run
# ("idle" or "loading") — without a page-level navigation.
#
# To trigger revalidation, inject the hook's `revalidate` function as the
# `onClick` handler of any element using the generic hook-wrapper pattern:
#   useRevalidator(tags$button(...), as = "onClick", selector = "revalidate")
#
# A 1-second delay is added to the loader so the "loading" state is visible.

library(reactRouter)
library(htmltools)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      loader = JS(
        "async () => {
        await new Promise(r => setTimeout(r, 1000));
        return { time: new Date().toLocaleTimeString() };
      }"
      ),
      element = div(
        tags$h2("useRevalidator Example"),
        tags$p(
          "Click the button to re-run the loader and refresh the timestamp. ",
          "The URL and scroll position do not change.",
          style = "color: #555;"
        ),
        tags$p(
          tags$strong("Loaded at: "),
          useLoaderData(tags$span(), selector = "time")
        ),
        tags$p(
          tags$strong("Revalidation state: "),
          useRevalidator(tags$span(), selector = "state")
        ),
        useRevalidator(
          tags$button(
            "Refresh (revalidate loader)",
            style = "padding: 6px 14px; cursor: pointer;"
          ),
          as = "onClick",
          selector = "revalidate"
        ),
        tags$hr(),
        Outlet()
      ),
      Route(index = TRUE, element = div(tags$p("Home page.")))
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# https://github.com/remix-run/react-router/tree/dev/examples/basic

library(shiny)
library(reactRouter)

Layout <- div(
  # A "layout route" is a good place to put markup you want to
  # share across all the pages on your site, like navigation.
  tags$nav(
    tags$ul(
      tags$li(
        reactRouter::Link(to = "/", "Home")
      ),
      tags$li(
        reactRouter::Link(to = "/about", "About")
      ),
      tags$li(
        reactRouter::Link(to = "/dashboard", "Dashboard")
      ),
      tags$li(
        reactRouter::Link(to = "/nothing-here", "Nothing Here")
      )
    )
  ),
  tags$hr(),
  # An <Outlet> renders whatever child route is currently active,
  # so you can think about this <Outlet> as a placeholder for
  # the child routes we defined above.
  reactRouter::Outlet()
)

Home <- div(
  tags$h2("Home")
)

About <- div(
  tags$h2("About")
)

Dashboard <- div(
  tags$h2("Dashboard")
)

NoMatch <- div(
  tags$h2("Nothing to see here!"),
  tags$p(
    Link(to = "/", "Go to the home page")
  )
)

Header <- div(
  h1("Basic Example"),
  tags$p(
    paste0(
      'This example demonstrates some of the core features of React Router
        including reactRouter::createHashRouter(), reactRouter::RouterProvider(),
        nested reactRouter::Route(), reactRouter::Outlet(),
        reactRouter::Link(), and using a "*" route (aka "splat route")
        to render a "not found" page when someone visits an unrecognized URL.'
    )
  ),
  Layout
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = Header,
      Route(
        index = TRUE,
        element = Home
      ),
      Route(
        path = "about",
        element = About
      ),
      Route(
        path = "dashboard",
        element = Dashboard
      ),
      # Using path="*" means "match anything", so this route
      # acts like a catch-all for URLs that we don't have explicit
      # routes for.
      Route(
        path = "*",
        element = NoMatch
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

box::use(
    app / view / header,
    app / logic / data[hero_stats]
)

#' @export
ui <- function(id) {
    ns <- shiny::NS(id)
    shiny::tagList(
        header$ui(ns("header")),
        shiny.fluent::Stack(
            horizontal = TRUE,
            horizontalAlign = "center",
            styles = list(
                root = list(width = "100%")
            ),
            shiny.fluent::Stack(
                styles = list(root = list(width = "50%")),
                echarts4r::echarts4rOutput(ns("rank_plot"))
            ),
            shiny.fluent::Stack(
                styles = list(root = list(width = "50%")),
                echarts4r::echarts4rOutput(ns("rank_prop_plot"))
            )
        )
    )
}

#' @export
server <- function(id, hero_selected) {
    shiny::moduleServer(id, function(input, output, session) {
        ns <- session$ns

        hero_details <- shiny::reactive({
            shiny::req(hero_selected())

            x <- purrr::keep(
                hero_stats,
                ~ .x$id == as.integer(hero_selected())
            ) |>
                purrr::pluck(1)

            tibble::tibble(
                win = unlist(x[["pub_win_trend"]]),
                pick = unlist(x[["pub_pick_trend"]]),
                rank = c(
                    "Herald",
                    "Guardian",
                    "Crusader",
                    "Archon",
                    "Legend",
                    "Ancient",
                    "Divine"
                )
            )
        })

        output$rank_prop_plot <- echarts4r::renderEcharts4r({
            shiny::req(hero_details())

            hero <- hero_details()

            hero |>
                dplyr::mutate(win_prop = win / pick) |>
                echarts4r::e_chart(rank) |>
                echarts4r::e_bar(win_prop, legend = FALSE)
        })

        output$rank_plot <- echarts4r::renderEcharts4r({
            shiny::req(hero_details())

            hero <- hero_details()

            hero |>
                tidyr::gather(variable, value, -rank) |>
                dplyr::group_by(variable) |>
                echarts4r::e_chart(rank) |>
                echarts4r::e_area(value) |>
                echarts4r::e_tooltip(trigger = "axis")
        })

        header$server("header", hero_selected = hero_selected)
    })
}

# dataResponse() — wraps a loader/action payload with HTTP status, statusText
# and headers, while still exposing the value via useLoaderData() /
# useActionData(). Useful when you want to attach metadata to the
# response without changing how the route's element consumes it.
#
# Two routes:
#   /profile  — static loader built with the R helper dataResponse()
#   /custom   — custom loader using window.jsmodule['@/reactRouter'].helpers.data() to
#               attach a 201 status to a computed payload.

library(reactRouter)
library(htmltools)

custom_loader <- JS(
  "() => {
     const payload = { greeting: 'hello', generatedAt: new Date().toISOString() };
     return window.jsmodule['@/reactRouter'].helpers.data(payload, {
       status: 201,
       headers: { 'X-Built-With': 'reactRouter R' }
     });
   }"
)

Layout <- div(
  style = "max-width: 540px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("dataResponse() Example"),
  tags$nav(tags$ul(
    tags$li(NavLink(to = "/", "Home")),
    tags$li(NavLink(to = "/profile", "/profile (static R object)")),
    tags$li(NavLink(to = "/custom", "/custom (computed in JS)"))
  )),
  tags$hr(),
  Outlet()
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE, element = tags$p("Pick a link above.")),
      Route(
        path = "profile",
        loader = dataResponse(
          list(name = "Ada Lovelace", role = "Engineer", years = 12),
          init = list(status = 200)
        ),
        element = div(
          tags$h3("Profile"),
          useLoaderData(tags$pre(style = "background:#f8f9fa;padding:10px;"))
        )
      ),
      Route(
        path = "custom",
        loader = custom_loader,
        element = div(
          tags$h3("Custom"),
          tags$p(tags$strong("Greeting: "), useLoaderData(tags$span(), selector = "greeting")),
          tags$p(tags$strong("Generated at: "), useLoaderData(tags$span(), selector = "generatedAt"))
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

box::use(
    app / logic / data[hero_stats]
)

#' @export
ui <- function(id) {
    ns <- shiny::NS(id)

    shiny::uiOutput(ns("header"))
}

#' @export
server <- function(id, hero_selected) {
    shiny::moduleServer(id, function(input, output, session) {
        ns <- session$ns

        output$header <- shiny::renderUI({
            shiny::req(hero_selected())

            hero <- purrr::keep(
                hero_stats,
                ~ .x$id == as.integer(hero_selected())
            ) |>
                purrr::pluck(1)

            shiny::tagList(
                shiny.fluent::Stack(
                    tokens = list(childrenGap = 20),
                    horizontalAlign = "center",
                    styles = list(
                        root = list(
                            padding = 20,
                            margin = "0 auto"
                        )
                    ),
                    shiny.fluent::Stack(
                        horizontal = TRUE,
                        maxWidth = "100%",
                        styles = list(root = list(width = "100%")),
                        horizontalAlign = "space-evenly",
                        tokens = list(childrenGap = 20),
                        shiny.fluent::Image(
                            src = paste0(
                                "https://cdn.cloudflare.steamstatic.com",
                                hero$img
                            ),
                            width = 256,
                            height = 144
                        ),
                        shiny.fluent::Stack(
                            tokens = list(childrenGap = 8),
                            shiny.fluent::Text(
                                variant = "xxLarge",
                                hero$localized_name
                            ),
                            shiny.fluent::Text(
                                variant = "large",
                                paste(
                                    toupper(hero$attack_type),
                                    "-",
                                    paste(unlist(hero$roles), collapse = ", ")
                                )
                            ),
                            shiny.fluent::Stack(
                                horizontal = TRUE,
                                tokens = list(childrenGap = 8),
                                lapply(hero$roles, function(role) {
                                    shiny.fluent::Text(
                                        role,
                                        styles = list(
                                            root = list(
                                                background = "#e54d2e",
                                                color = "white",
                                                padding = "4px 10px",
                                                borderRadius = 12,
                                                fontSize = 12,
                                                fontWeight = 500
                                            )
                                        )
                                    )
                                })
                            )
                        )
                    ),

                    shiny.fluent::Separator()
                )
            )
        })
    })
}

# useSubmit() triggers a form submission programmatically — it invokes
# the route's action() without requiring a <Form> element or even a
# user-visible form. Useful for "quick actions" like delete/confirm
# buttons that still go through React Router's data flow.
#
# Requires a data router (createHashRouter / createBrowserRouter).

library(reactRouter)
library(htmltools)

# Route action: receives the submitted data and returns a message.
# In a real app this would mutate a database / call an API.
action <- JS(
  "async ({ request }) => {
     const form = await request.formData();
     const intent = form.get('intent');
     return { message: `action received intent = ${intent} at ${new Date().toLocaleTimeString()}` };
   }"
)

# render receives the submit function:
#   submit(targetOrFormData, options?)
# Here we submit a plain object as form data with method 'post'.
submit_buttons <- JS(
  "submit => React.createElement('div', null,
    React.createElement('button',
      { onClick: () => submit({ intent: 'like' },   { method: 'post' }) },
      'Like'
    ),
    ' ',
    React.createElement('button',
      { onClick: () => submit({ intent: 'delete' }, { method: 'post' }) },
      'Delete'
    )
  )"
)

Layout <- div(
  style = "max-width: 520px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("useSubmit Example"),
  tags$p(
    style = "color: #555; font-size: 0.9em;",
    "These buttons call ", tags$code("submit()"), " directly. No ",
    tags$code("<Form>"), " element is rendered — the route's ",
    tags$code("action()"), " still runs."
  ),
  useSubmit(render = submit_buttons),
  tags$hr(),
  tags$h3("Last action response:"),
  useActionData(
    tags$pre(style = "background: #f4f4f4; padding: 8px;"),
    selector = "message"
  )
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      action = action,
      element = Layout
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# useBlocker() intercepts navigation when a condition is true.
# It exposes a state field: "unblocked" | "blocked" | "proceeding".
#
# The render function receives the full blocker object, including
# b.proceed() and b.reset() methods. Wire these to onClick handlers
# to build a real confirm/cancel dialog — otherwise the blocker stays
# stuck in "blocked" and the user can never navigate or reset.

library(reactRouter)
library(htmltools)

# shouldBlock fires when moving between different pathnames
should_block <- JS(
  "({ currentLocation, nextLocation }) =>
    currentLocation.pathname !== nextLocation.pathname"
)

# render receives the full blocker object b:
#   b.state      — "unblocked" | "blocked" | "proceeding"
#   b.proceed()  — confirm navigation
#   b.reset()    — cancel and return to "unblocked"
blocker_ui <- JS(
  "b => {
    const state = React.createElement('code', null, b.state);
    if (b.state !== 'blocked') return state;
    return React.createElement(React.Fragment, null,
      state, ' — ',
      React.createElement('button', { onClick: () => b.proceed() }, 'Leave'),
      ' ',
      React.createElement('button', { onClick: () => b.reset() }, 'Stay')
    );
  }"
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        style = "max-width: 480px; margin: 0 auto; padding: 20px; font-family: system-ui;",
        tags$h2("useBlocker Example"),
        tags$nav(tags$ul(
          tags$li(NavLink(to = "/", "Home (blocks navigation)")),
          tags$li(NavLink(to = "/other", "Other page"))
        )),
        tags$hr(),
        Outlet()
      ),
      Route(
        index = TRUE,
        element = div(
          tags$h3("Home — with blocker"),
          tags$p(
            tags$strong("Blocker state: "),
            useBlocker(shouldBlock = should_block, render = blocker_ui)
          ),
          tags$p(
            style = "color: #555; font-size: 0.9em;",
            "Click 'Other page'. The state changes to ",
            tags$code('"blocked"'), " and two buttons appear.",
            tags$br(),
            tags$em("Leave"), " calls ", tags$code("b.proceed()"),
            " to confirm navigation; ",
            tags$em("Stay"), " calls ", tags$code("b.reset()"),
            " to cancel and return to ", tags$code('"unblocked"'), "."
          )
        )
      ),
      Route(
        path = "other",
        element = div(
          tags$h3("Other page"),
          tags$p("No blocker on this route. Navigate freely.")
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# RouterProvider example
# Mirrors the React Router v7 composition pattern:
#   RouterProvider(router = createHashRouter(routes), fallbackElement = ...)
# Swap createHashRouter() for createBrowserRouter() or createMemoryRouter()
# to change the routing strategy.
#
# `fallbackElement` is shown while the initial route's loader resolves.

library(reactRouter)
library(htmltools)

Home <- div(
  tags$h3("Home"),
  useLoaderData(tags$p(), selector = "message"),
  tags$p("Built with ", tags$code("createMemoryRouter()"), "."),
  tags$p(
    "Swap it for ",
    tags$code("createHashRouter()"),
    " or ",
    tags$code("createBrowserRouter()"),
    " to change the routing strategy."
  )
)

About <- div(
  tags$h3("About"),
  tags$p(
    "This app composes ",
    tags$code("RouterProvider()"),
    " with ",
    tags$code("createMemoryRouter()"),
    ", matching the official React Router v7 API."
  )
)

Layout <- div(
  style = "max-width: 480px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("RouterProvider Example"),
  tags$nav(tags$ul(
    tags$li(NavLink(to = "/", "Home")),
    tags$li(NavLink(to = "/about", "About"))
  )),
  tags$hr(),
  Outlet()
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = Layout,
      Route(
        index = TRUE,
        loader = JS(
          "async () => {
          await new Promise(r => setTimeout(r, 1500));
          return { message: 'Data loaded after 1.5 s!' };
        }"
        ),
        element = Home
      ),
      Route(path = "about", element = About)
    )
  ),
  fallbackElement = tags$div(
    style = "max-width: 480px; margin: 0 auto; padding: 40px; font-family: system-ui; color: gray;",
    tags$h2("Loading application...")
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

box::use(
    app / logic / data[hero_stats],
    app / logic / utils
)

#' @export
ui <- function(id) {
    ns <- shiny::NS(id)

    shiny::tagList(
        shiny.fluent::Stack(
            horizontal = FALSE,
            horizontalAlign = "center",
            shiny.fluent::Text(
                "reactRouter + Rhino + shiny.fluent + opendota",
                variant = "xLarge"
            ),
            shiny.fluent::Text(
                "This is a simple example of a Shiny app using reactRouter, Rhino, ",
                "and shiny.fluent to display a list of heroes from the OpenDota API exploring dynamic routing."
            ),
            shiny.fluent::Text(
                "Click on a hero to view their details. ",
                "The hero's ID will be included in the URL, allowing for dynamic routing, e.g, /#/{hero_id}/details."
            )
        ),
        shiny.fluent::Stack(
            horizontal = TRUE,
            horizontalAlign = "center",
            styles = list(root = list(marginTop = 32)), # Add distance from the element on top
            shiny.fluent::SearchBox.shinyInput(
                ns("search"),
                placeholder = "Search for a Dota 2 hero...",
                underlined = TRUE,
                iconProps = list(iconName = "Search"),
                styles = list(root = list(width = 300))
            )
        ),
        shiny::uiOutput(ns("hero_list"))
    )
}

#' @export
server <- function(id) {
    shiny::moduleServer(id, function(input, output, session) {
        ns <- session$ns

        hero_stats_filter <- shiny::reactive({
            if (nchar(input$search) > 2) {
                heros_name <- unlist(purrr::map(
                    hero_stats,
                    ~ .x[["localized_name"]]
                ))

                # Find the hero name with the highest match to input$search using stringdist
                distances <- stringdist::stringdist(
                    tolower(input$search),
                    tolower(heros_name),
                    method = "jw"
                )
                if (any(distances == 0)) {
                    return(hero_stats[which(distances == 0)])
                }

                index <- which(distances >= 0 & distances <= 0.4)
                if (length(index) > 0) {
                    return(hero_stats[index])
                } else {
                    return(NULL)
                }
            } else {
                return(hero_stats)
            }
        })

        output$hero_list <- shiny::renderUI({
            if (is.null(hero_stats_filter())) {
                shiny::div(
                    style = "
                        position: absolute;
                        top: 50%;
                        left: 50%;
                        transform: translate(-50%, -50%);
                        font-size: 1.2em;
                        text-align: center;
                    ",
                    "No heroes found matching your search criteria."
                )
            } else {
                shiny::req(hero_stats_filter())

                hero_list <- purrr::map(hero_stats_filter(), function(.x) {
                    shiny::div(
                        shiny.fluent::DocumentCard(
                            styles = list(
                                root = list(
                                    "max-width" = "100%",
                                    width = "250px"
                                ) # Card is 250px wide
                            ),
                            onClickHref = paste0("#/", .x$id, "/details"),
                            shiny.fluent::DocumentCardPreview(
                                previewImages = list(
                                    list(
                                        name = "",
                                        previewImageSrc = paste0(
                                            "https://cdn.cloudflare.steamstatic.com",
                                            .x$img
                                        ),
                                        width = 250, # Match the card width
                                        height = 140, # Adjust proportionally (e.g., 16:9 = 250 x 140)
                                        imageFit = "cover"
                                    )
                                )
                            ),
                            shiny.fluent::Stack(
                                horizontal = FALSE,
                                horizontalAlign = "center", # Center icons horizontally
                                tokens = list(childrenGap = 5),
                                styles = list(root = list(marginBottom = 10)), # Space below the Stack
                                children = list(
                                    shiny::span(.x$localized_name),
                                    shiny::span(
                                        toupper(.x$primary_attr),
                                        style = paste0(
                                            "color: ",
                                            utils$primary_attr_color(
                                                .x$primary_attr
                                            ),
                                            "; font-weight: bold;"
                                        )
                                    )
                                )
                            )
                        )
                    )
                })

                shiny::div(
                    style = "display: grid; grid-template-columns: repeat(auto-fill, minmax(240px, 1fr)); gap: 32px; max-width: 1080px; margin: 0 auto; padding: 40px 20px;",
                    shiny::tagList(hero_list)
                )
            }
        })

        return(shiny::reactive({
            current_route <- sub("#/", "", session$clientData$url_hash)
            # print(current_route)
            return(stringr::str_extract(current_route, "\\d+"))
        }))
    })
}

box::use(
    app / view / header
)

#' @export
ui <- function(id) {
    ns <- shiny::NS(id)

    shiny::tagList(
        header$ui(ns("header")),
        shiny::uiOutput(ns("benchmark_ui"))
    )
}

#' @export
server <- function(id, hero_selected) {
    shiny::moduleServer(id, function(input, output, session) {
        ns <- session$ns

        benchmark_data <- shiny::reactive({
            shiny::req(hero_selected())

            data <- httr::GET(paste0(
                "https://api.opendota.com/api/benchmarks?hero_id=",
                hero_selected()
            ))
            data <- httr::content(data)

            data$result |>
                purrr::map(function(.x) {
                    purrr::map(.x, as.data.frame) |>
                        dplyr::bind_rows()
                }) |>
                purrr::imap_dfr(~ dplyr::mutate(.x, metric = .y))
        })

        output$benchmark_ui <- shiny::renderUI({
            shiny::req(benchmark_data())

            benchmark_data <- benchmark_data()

            purrr::walk2(
                unique(benchmark_data$metric),  
                seq_along(unique(benchmark_data$metric)),
                function(.x, i) {
                    output[[paste0(
                        "chart_",
                        .x
                    )]] <- echarts4r::renderEcharts4r({
                        plot <- benchmark_data |>
                            dplyr::filter(metric == .x) |>
                            echarts4r::e_charts(percentile) |>
                            echarts4r::e_area(value, legend = FALSE) |>
                            echarts4r::e_tooltip(trigger = "axis") |>
                            echarts4r::e_group("charts")

                        # Call e_connect_group only on the last chart
                        if (i == length(unique(benchmark_data$metric))) {
                            plot <- plot |>
                                echarts4r::e_connect_group("charts")
                        }

                        plot
                    })
                }
            )

            shiny::div(
                style = "display: grid; grid-template-columns: repeat(auto-fill, minmax(240px, 1fr)); gap: 32px; max-width: 1080px; margin: 0 auto; padding: 40px 20px;",
                shiny::tagList(
                    purrr::map(unique(benchmark_data$metric), function(.x) {
                        shiny::div(
                            shiny::h4(.x),
                            echarts4r::echarts4rOutput(
                                outputId = ns(paste0("chart_", .x)),
                                height = "300px"
                            )
                        )
                    })
                )
            )
        })

        header$server("header", hero_selected = hero_selected)
    })
}

# redirectDocument() returns a JS loader that performs a full-page reload
# to the given URL — as opposed to redirect()'s client-side navigation.
# Use it when leaving the SPA entirely, e.g. to a server-rendered page
# or an external site, so the browser fully unloads the router.
#
# Compare:
#   /spa  --redirect-->          /home    (in-router; URL bar updates, no reload)
#   /docs --redirectDocument--> /static/  (browser-level navigation; reload)

library(reactRouter)
library(htmltools)

Layout <- div(
  style = "max-width: 540px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("redirectDocument() Example"),
  tags$nav(tags$ul(
    tags$li(NavLink(to = "/", "Home")),
    tags$li(NavLink(to = "/external", "/external (full-page redirect to https://example.com)"))
  )),
  tags$p(
    style = "color: gray; font-size: 0.9em;",
    "Click /external to leave the SPA via a document-level redirect."
  ),
  tags$hr(),
  Outlet()
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE, element = tags$h3("Home")),
      Route(
        path = "external",
        loader = redirectDocument("https://example.com"),
        element = NULL
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

box::use(
  app / view / home,
  app / view / menu,
  app / view / details,
  app / view / benchmark,
  app / view / rank
)

#' @export
ui <- function(id) {
  ns <- shiny::NS(id)
  shiny::tagList(
    shiny.fluent::fluentPage(
      waiter::use_waiter(),
      waiter::waiterOnBusy(
        color = "#C7C6C6",
        html = shiny::tagList(
          waiter::spin_fading_circles(),
          "Fetching OpenDota API data and creating page..."
        ),
        fadeout = TRUE
      ),
      reactRouter::HashRouter(
        reactRouter::Routes(
          reactRouter::Route(
            path = "/",
            element = home$ui(ns("home"))
          ),
          reactRouter::Route(
            path = "/:projectId/*",
            element = menu$ui(ns("menu")),
            children = list(
              reactRouter::Route(
                path = "details",
                element = details$ui(ns("details"))
              ),
              reactRouter::Route(
                path = "benchmark",
                element = benchmark$ui(ns("benchmark"))
              ),
              reactRouter::Route(
                path = "rank",
                element = rank$ui(ns("rank"))
              )
            )
          ),
          reactRouter::Route(path = "*", element = "Custom error 404")
        )
      )
    )
  )
}

#' @export
server <- function(id) {
  shiny::moduleServer(id, function(input, output, session) {
    hero_selected <- home$server("home")

    shiny::observe({
      shiny::req(hero_selected())

      print(paste0("hero_id selected: ", hero_selected()))
    })

    menu$server("menu", hero_selected = hero_selected)
    details$server("details", hero_selected = hero_selected)
    benchmark$server("benchmark", hero_selected = hero_selected)
    rank$server("rank", hero_selected = hero_selected)
  })
}

# Dashboard Form Example
# Demonstrates Form component in a dashboard context:
#   - GET form to search KPIs by name or category (updates URL search params, triggers loader)
#   - POST form to generate a report by metric and date range (triggers action, result via useActionData)
#   - useNavigation to show "submitting" / "loading" state

library(reactRouter)
library(htmltools)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        style = "max-width: 640px; margin: 0 auto; padding: 20px; font-family: system-ui;",
        tags$h2("Form Example"),
        tags$p(
          style = "color: gray;",
          "Navigation state: ",
          useNavigation(tags$strong(), selector = "state")
        ),
        tags$nav(tags$ul(
          tags$li(NavLink(to = "/", "Home")),
          tags$li(NavLink(to = "/search", "Search Metrics (GET)")),
          tags$li(NavLink(to = "/report", "Report (POST)"))
        )),
        tags$hr(),
        Outlet()
      ),

      # -- Index route ---
      Route(
        index = TRUE,
        element = div(
          tags$p("This example demonstrates the Form component."),
          tags$ul(
            tags$li(
              tags$strong("Search Metrics (GET)"),
              " — search dashboard KPIs by name or category; updates URL search params and triggers a loader."
            ),
            tags$li(
              tags$strong("Report (POST)"),
              " — generate a dashboard report by metric and date range; triggers a route action, result shown via useActionData."
            )
          )
        )
      ),

      # -- GET form: search metrics with loader ---
      Route(
        path = "search",
        loader = JS(
          "({ request }) => {
            const metrics = [
              { name: 'Revenue',         category: 'Finance',   value: '$142,500', trend: '+12.3%' },
              { name: 'Operating Costs',  category: 'Finance',   value: '$87,200',  trend: '+3.1%' },
              { name: 'Net Profit',       category: 'Finance',   value: '$55,300',  trend: '+18.7%' },
              { name: 'Active Users',     category: 'Engagement', value: '8,320',   trend: '+5.1%' },
              { name: 'New Signups',      category: 'Engagement', value: '1,204',   trend: '+22.0%' },
              { name: 'Churn Rate',       category: 'Engagement', value: '2.4%',    trend: '-0.8%' },
              { name: 'Page Views',       category: 'Traffic',   value: '214,700',  trend: '-2.4%' },
              { name: 'Bounce Rate',      category: 'Traffic',   value: '38.2%',    trend: '-1.5%' },
              { name: 'Avg Session Time', category: 'Traffic',   value: '4m 12s',   trend: '+8.6%' }
            ];

            const url   = new URL(request.url);
            const q     = (url.searchParams.get('q') || '').trim();
            if (!q) return { results: '', query: '', count: 0 };

            const lower = q.toLowerCase();
            const hits  = metrics.filter(m =>
              m.name.toLowerCase().includes(lower) ||
              m.category.toLowerCase().includes(lower)
            );

            const summary = hits.map(m => m.name + '  |  ' + m.category + '  |  ' + m.value + '  (' + m.trend + ')').join('\\n');
            return { results: summary, query: q, count: hits.length };
          }"
        ),
        element = div(
          tags$h3("Search Metrics (GET)"),
          tags$p(
            style = "color: gray; font-size: 0.9em;",
            "The Form uses ",
            tags$code("method=\"get\""),
            " so submitting updates the URL to ",
            tags$code("?q=..."),
            " and triggers the route loader."
          ),
          Form(
            method = "get",
            style = "display: flex; gap: 8px; margin-bottom: 16px;",
            tags$input(
              type = "search",
              name = "q",
              placeholder = "e.g. revenue, traffic, engagement...",
              style = "flex: 1; padding: 8px; border: 1px solid #ccc; border-radius: 4px;"
            ),
            tags$button(
              type = "submit",
              style = "padding: 8px 16px; border: none; background: #0d6efd; color: white; border-radius: 4px; cursor: pointer;",
              "Search"
            )
          ),
          tags$div(
            style = "background: #f8f9fa; padding: 12px; border-radius: 6px; margin-bottom: 12px;",
            tags$span(style = "color: gray;", "Query: "),
            useSearchParams(tags$strong(), param = "q"),
            tags$span(style = "color: gray; margin-left: 16px;", "Results: "),
            useLoaderData(tags$strong(), selector = "count")
          ),
          tags$pre(
            style = "background: #f5f5f5; padding: 10px; white-space: pre-wrap;",
            useLoaderData(tags$span(), selector = "results")
          )
        )
      ),

      # -- POST form: dashboard report generator ---
      Route(
        path = "report",
        action = JS(
          "async ({ request }) => {
            const fd     = await request.formData();
            const metric = fd.get('metric');
            const from   = fd.get('from');
            const to     = fd.get('to');
            const grain  = fd.get('granularity') || 'daily';

            await new Promise(r => setTimeout(r, 800));

            const totals = { Revenue: '$142,500', Users: '8,320', 'Page Views': '214,700' };
            const avgs   = { Revenue: '$4,750/day', Users: '277/day', 'Page Views': '7,157/day' };
            const trends = { Revenue: '+12.3%', Users: '+5.1%', 'Page Views': '-2.4%' };

            return {
              metric: metric,
              period: from + ' to ' + to,
              granularity: grain,
              total: totals[metric] || 'N/A',
              average: avgs[metric] || 'N/A',
              trend: trends[metric] || 'N/A'
            };
          }"
        ),
        element = div(
          tags$h3("Generate Report (POST)"),
          tags$p(
            style = "color: gray; font-size: 0.9em;",
            "The Form uses ",
            tags$code("method=\"post\""),
            " so submitting triggers the route ",
            tags$code("action"),
            ". The result is available via useActionData."
          ),
          Form(
            method = "post",
            style = "display: flex; flex-direction: column; gap: 10px; max-width: 400px; margin-bottom: 16px;",
            tags$label(
              "Metric",
              tags$select(
                name = "metric",
                style = "display: block; width: 100%; padding: 8px; border: 1px solid #ccc; border-radius: 4px; margin-top: 4px;",
                tags$option(value = "Revenue", "Revenue"),
                tags$option(value = "Users", "Active Users"),
                tags$option(value = "Page Views", "Page Views")
              )
            ),
            tags$label(
              "From",
              tags$input(
                type = "date",
                name = "from",
                value = "2026-01-01",
                required = NA,
                style = "display: block; width: 100%; padding: 8px; border: 1px solid #ccc; border-radius: 4px; margin-top: 4px;"
              )
            ),
            tags$label(
              "To",
              tags$input(
                type = "date",
                name = "to",
                value = "2026-03-31",
                required = NA,
                style = "display: block; width: 100%; padding: 8px; border: 1px solid #ccc; border-radius: 4px; margin-top: 4px;"
              )
            ),
            tags$fieldset(
              style = "border: 1px solid #ccc; border-radius: 4px; padding: 8px;",
              tags$legend("Granularity"),
              tags$label(
                style = "margin-right: 12px;",
                tags$input(
                  type = "radio",
                  name = "granularity",
                  value = "daily",
                  checked = NA
                ),
                " Daily"
              ),
              tags$label(
                style = "margin-right: 12px;",
                tags$input(
                  type = "radio",
                  name = "granularity",
                  value = "weekly"
                ),
                " Weekly"
              ),
              tags$label(
                tags$input(
                  type = "radio",
                  name = "granularity",
                  value = "monthly"
                ),
                " Monthly"
              )
            ),
            tags$button(
              type = "submit",
              style = "padding: 10px 20px; border: none; background: #198754; color: white; border-radius: 4px; cursor: pointer; align-self: flex-start;",
              "Generate Report"
            )
          ),
          tags$h4("Report Summary:"),
          tags$div(
            style = "background: #f8f9fa; padding: 12px; border-radius: 6px; margin-bottom: 8px;",
            tags$p(
              tags$strong("Metric: "),
              useActionData(into = tags$span(), selector = "metric")
            ),
            tags$p(
              tags$strong("Period: "),
              useActionData(into = tags$span(), selector = "period")
            ),
            tags$p(
              tags$strong("Granularity: "),
              useActionData(into = tags$span(), selector = "granularity")
            ),
            tags$p(
              tags$strong("Total: "),
              useActionData(into = tags$span(), selector = "total")
            ),
            tags$p(
              tags$strong("Average: "),
              useActionData(into = tags$span(), selector = "average")
            ),
            tags$p(
              tags$strong("Trend: "),
              useActionData(into = tags$span(), selector = "trend")
            )
          )
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# Await example — classic into= mode
# Demonstrates Await() which renders content once a deferred loader promise
# resolves, enabling non-blocking ("streaming") data loading.
#
# The loader returns a slow promise alongside fast data (no defer() needed in
# React Router v7 — just return the object directly). The fast data renders
# immediately; Await() waits for the slow promise and streams in the result
# without blocking the page transition.
#

library(reactRouter)
library(htmltools)

Layout <- div(
  style = "max-width: 560px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("Await Example"),
  tags$nav(tags$ul(
    tags$li(NavLink(to = "/", "Dashboard")),
    tags$li(NavLink(to = "/report", "Monthly Report (deferred)"))
  )),
  tags$hr(),
  Outlet()
)

ui <- RouterProvider(
  router = createMemoryRouter(
    Route(
      path = "/",
      element = Layout,

      # Index route — instant load
      Route(
        index = TRUE,
        element = div(
          tags$h3("Dashboard"),
          tags$p(
            "Navigate to ",
            tags$strong("Monthly Report"),
            " to see deferred loading."
          ),
          tags$p(
            "The report loader immediately returns fast summary data and ",
            tags$em("defers"),
            " the slower details query (~2 s). The page renders at once;",
            " Await() streams in the deferred metrics when the promise resolves."
          )
        )
      ),

      # Report route — fast summary + slow deferred details
      Route(
        path = "report",
        loader = JS(
          "async () => {
            // Fast data returned immediately
            const summary = { title: 'Q1 2026 Monthly Report', pages: 42 };

            // Slow promise — deferred so the page does not block on it
            const details = new Promise((resolve) =>
              setTimeout(() => resolve({
                revenue:   '$1,240,000',
                users:     '18,430',
                growth:    '+14.2%',
                topRegion: 'EMEA'
              }), 2000)
            );

            return { summary, details };
          }"
        ),
        element = div(
          tags$h3(useLoaderData(tags$span(), selector = "summary.title")),
          tags$p(
            tags$strong("Pages: "),
            useLoaderData(tags$span(), selector = "summary.pages")
          ),
          tags$hr(),

          tags$h4("Detailed metrics (deferred — loads in ~2 s)"),
          tags$p(
            style = "color: gray; font-size: 0.9em;",
            "Below renders once the deferred promise resolves:"
          ),

          # Show each field individually using selector.
          # The `fallback` element is shown while the promise is pending.
          tags$div(
            style = "background: #f8f9fa; padding: 12px; border-radius: 6px;",
            tags$p(
              tags$strong("Revenue: "),
              Await(
                tags$span(),
                resolveKey = "details",
                selector = "revenue",
                fallback = tags$em(style = "color:gray;", "loading…")
              )
            ),
            tags$p(
              tags$strong("Users: "),
              Await(
                tags$span(),
                resolveKey = "details",
                selector = "users",
                fallback = tags$em(style = "color:gray;", "loading…")
              )
            ),
            tags$p(
              tags$strong("Growth: "),
              Await(
                tags$span(),
                resolveKey = "details",
                selector = "growth",
                fallback = tags$em(style = "color:gray;", "loading…")
              )
            ),
            tags$p(
              tags$strong("Top region: "),
              Await(
                tags$span(),
                resolveKey = "details",
                selector = "topRegion",
                fallback = tags$em(style = "color:gray;", "loading…")
              )
            )
          )
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# Minimal example showing useLocation() which exposes useLocation()
# useLocation properties: pathname, search, hash, state, key

library(shiny)
library(reactRouter)

Layout <- div(
  h2("Location Example"),
  tags$nav(
    tags$ul(
      tags$li(NavLink(to = "/", "Home")),
      tags$li(NavLink(to = "/about", "About")),
      tags$li(NavLink(
        to = "/about?q=test#section",
        "About with query & hash"
      ))
    )
  ),
  tags$hr(),
  tags$h3("Pathname only:"),
  useLocation(tags$span(), selector = "pathname"),
  tags$h3("Search:"),
  useLocation(tags$span(), selector = "search"),
  tags$h3("Hash:"),
  useLocation(tags$span(), selector = "hash"),
  tags$hr(),
  Outlet()
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE, element = div(tags$p("Home page"))),
      Route(path = "about", element = div(tags$p("About page")))
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# openDota API example for fetching hero stats

print("Fetching hero stats from OpenDota API...")
hero_stats <- httr::GET("https://api.opendota.com/api/heroStats")
hero_stats <- httr::content(hero_stats)
hero_stats

# useFetchers() returns all currently active fetchers.
# Useful for a global loading indicator: if any fetcher is busy, show a spinner.
#
# This example runs two Form(navigate = FALSE) searches with different keys.
# useFetchers(selector = "state") maps over all active fetchers and returns
# their states as a JSON array.

library(reactRouter)
library(htmltools)
library(jsonlite)

people_json <- jsonlite::toJSON(
  data.frame(name = c("Alice", "Bob", "Charlie", "David", "Eve")),
  dataframe = "rows",
  auto_unbox = TRUE
)

loader_js <- JS(sprintf(
  "async ({ request }) => {
    await new Promise(r => setTimeout(r, 1000));
    const q = new URL(request.url).searchParams.get('q') || '';
    const db = %s;
    return db.filter(r => r.name.toLowerCase().includes(q.toLowerCase()));
  }",
  people_json
))

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        tags$h2("useFetchers Example — Global Loading Indicator"),
        tags$p(
          "Submit one or more search forms. Each uses a different fetcher key. ",
          "useFetchers() tracks all of them.",
          style = "color: #555;"
        ),
        tags$p(
          tags$strong("All fetcher states: "),
          useFetchers(tags$code(), selector = "state")
        ),
        tags$hr(),

        tags$h3("Search A"),
        Form(
          navigate = FALSE,
          fetcherKey = "search-a",
          method = "get",
          action = "/search",
          style = "display: flex; gap: 8px; margin-bottom: 8px;",
          tags$input(
            type = "search",
            name = "q",
            placeholder = "Name...",
            style = "padding: 4px;"
          ),
          tags$button(type = "submit", "Search A", style = "cursor: pointer;")
        ),
        tags$p(
          "State A: ",
          useFetcher(tags$code(), fetcherKey = "search-a", selector = "state")
        ),
        tags$p(
          "Results A: ",
          useFetcher(tags$span(), fetcherKey = "search-a", selector = "data")
        ),

        tags$h3("Search B"),
        Form(
          navigate = FALSE,
          fetcherKey = "search-b",
          method = "get",
          action = "/search",
          style = "display: flex; gap: 8px; margin-bottom: 8px;",
          tags$input(
            type = "search",
            name = "q",
            placeholder = "Name...",
            style = "padding: 4px;"
          ),
          tags$button(type = "submit", "Search B", style = "cursor: pointer;")
        ),
        tags$p(
          "State B: ",
          useFetcher(tags$code(), fetcherKey = "search-b", selector = "state")
        ),
        tags$p(
          "Results B: ",
          useFetcher(tags$span(), fetcherKey = "search-b", selector = "data")
        ),

        tags$hr(),
        Outlet()
      ),
      Route(index = TRUE, element = div()),
      Route(
        path = "search",
        loader = loader_js,
        element = div()
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(reactRouter)

## ----basic--------------------------------------------------------------------
library(reactRouter)

HashRouter(
  NavLink(to = "/", "Main"),
  NavLink(to = "/analysis", "Analysis"),
  Routes(
    Route(path = "/", element = "Main content"),
    Route(path = "/analysis", element = "Analysis content")
  )
)

## -----------------------------------------------------------------------------
# code to run in a Quarto document
# example adapted from: https://github.com/remix-run/react-router/tree/dev/examples/basic
library(reactRouter)
library(htmltools)

Layout <- div(
  # A "layout route" is a good place to put markup you want to
  # share across all the pages on your site, like navigation.
  tags$nav(
    tags$ul(
      tags$li(
        reactRouter::Link(to = "/", "Home")
      ),
      tags$li(
        reactRouter::Link(to = "/dashboard", "Dashboard")
      ),
      tags$li(
        reactRouter::Link(to = "/nothing-here", "Nothing Here")
      )
    )
  ),
  tags$hr(),
  # An <Outlet> renders whatever child route is currently active,
  # so you can think about this <Outlet> as a placeholder for
  # the child routes we defined above.
  reactRouter::Outlet()
)

reactRouter::HashRouter(
  div(
    style = "border:1px solid black;", # add border just for the example
    h1("Basic Example"),
    tags$p(
      paste0('This example demonstrates some of the core features of React Router
          including nested reactRouter::Route(), reactRouter::Outlet(), 
          reactRouter::Link(), and using a "*" route (aka "splat route") 
          to render a "not found" page when someone visits an unrecognized URL.'
      )
    ),
    reactRouter::Routes(
      Route(
        path = "/",
        element = Layout,
        Route(
          index = TRUE,
          element = div(
            tags$h2("Home"),
            tags$p("Home content")
          )
        ),
        Route(
          path = "dashboard",
          element = div(
            tags$h2("Dashboard"),
            tags$p("Dashboard here")
          )
        ),
        # Using path="*"" means "match anything", so this route
        # acts like a catch-all for URLs that we don't have explicit
        # routes for.
        Route(
          path = "*",
          element = div(
            tags$h2("Nothing to see here!"),
            tags$p(
              Link(to = "/", "Go to the home page")
            )
          )
        )
      )
    )
  )
)

# replaceResponse() returns a JS loader that performs a *replace* navigation.
# Same as redirect(), but the target URL replaces the current entry in
# the history stack instead of pushing a new one — ideal for legacy
# aliases that should not appear in the user's back-history.
#
# /legacy --replaceResponse--> /home   (back button skips /legacy)
# /old    --redirect-->         /home  (back button revisits /old, then redirects again)
#
# Renamed from replace() so it doesn't mask base::replace.

library(reactRouter)
library(htmltools)

Layout <- div(
  style = "max-width: 540px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("replaceResponse() Example"),
  tags$nav(tags$ul(
    tags$li(NavLink(to = "/", "Home")),
    tags$li(NavLink(to = "/legacy", "/legacy (replaceResponse -> /home)")),
    tags$li(NavLink(to = "/old", "/old (redirect -> /home)"))
  )),
  tags$p(
    style = "color: gray; font-size: 0.9em;",
    "Open /legacy or /old, then press the browser Back button: ",
    "the replaceResponse() route is gone from history, the redirect() route is not."
  ),
  tags$hr(),
  Outlet()
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE, element = tags$h3("Home")),
      Route(path = "legacy", loader = replaceResponse("/"), element = NULL),
      Route(path = "old",    loader = redirect("/"),         element = NULL)
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

box::use(
    app / view / header,
    app / logic / data[hero_stats]
)


#' @export
ui <- function(id) {
    ns <- shiny::NS(id)

    shiny::uiOutput(ns("details_page"))
}

#' @export
server <- function(id, hero_selected) {
    shiny::moduleServer(id, function(input, output, session) {
        ns <- session$ns

        output$details_page <- shiny::renderUI({
            shiny::req(hero_selected())

            hero <- purrr::keep(
                hero_stats,
                ~ .x$id == as.integer(hero_selected())
            ) |>
                purrr::pluck(1)

            shiny.fluent::fluentPage(
                style = "background-color: #1b2838; color: white;",

                header$ui(ns("header")),

                shiny.fluent::Stack(
                    # Stats Sections
                    shiny.fluent::Stack(
                        horizontal = TRUE,
                        horizontalAlign = "space-evenly",
                        tokens = list(childrenGap = 20),
                        maxWidth = "100%",
                        styles = list(root = list(width = "100%")),

                        # Attributes
                        shiny.fluent::Stack(
                            tokens = list(childrenGap = 10),
                            shiny.fluent::Text(
                                variant = "large",
                                "Attributes"
                            ),
                            shiny.fluent::Text(paste(
                                "STR:",
                                hero$base_str,
                                "+",
                                hero$str_gain
                            )),
                            shiny.fluent::Text(paste(
                                "AGI:",
                                hero$base_agi,
                                "+",
                                hero$agi_gain
                            )),
                            shiny.fluent::Text(paste(
                                "INT:",
                                hero$base_int,
                                "+",
                                hero$int_gain
                            ))
                        ),

                        # Combat
                        shiny.fluent::Stack(
                            tokens = list(childrenGap = 10),
                            shiny.fluent::Text(
                                variant = "large",
                                "Combat Stats"
                            ),
                            shiny.fluent::Text(paste(
                                "Base Attack:",
                                hero$base_attack_min,
                                "-",
                                hero$base_attack_max
                            )),
                            shiny.fluent::Text(paste(
                                "Attack Range:",
                                hero$attack_range
                            )),
                            shiny.fluent::Text(paste(
                                "Attack Speed:",
                                round(1 / hero$attack_rate * 100, 2)
                            )),
                            shiny.fluent::Text(paste(
                                "Projectile Speed:",
                                hero$projectile_speed
                            ))
                        ),

                        # Survivability
                        shiny.fluent::Stack(
                            tokens = list(childrenGap = 10),
                            shiny.fluent::Text(
                                variant = "large",
                                "Survivability"
                            ),
                            shiny.fluent::Text(paste(
                                "Health:",
                                hero$base_health
                            )),
                            shiny.fluent::Text(paste(
                                "Health Regen:",
                                hero$base_health_regen
                            )),
                            shiny.fluent::Text(paste(
                                "Mana:",
                                hero$base_mana
                            )),
                            shiny.fluent::Text(paste(
                                "Mana Regen:",
                                hero$base_mana_regen
                            )),
                            shiny.fluent::Text(paste(
                                "Armor:",
                                hero$base_armor
                            )),
                            shiny.fluent::Text(paste(
                                "Magic Resist:",
                                paste0(hero$base_mr, "%")
                            ))
                        )
                    ),

                    shiny.fluent::Separator(),

                    # Misc Info
                    shiny.fluent::Stack(
                        horizontal = TRUE,
                        horizontalAlign = "center",
                        tokens = list(childrenGap = 40),
                        shiny.fluent::Text(paste(
                            "Move Speed:",
                            hero$move_speed
                        )),
                        shiny.fluent::Text(paste(
                            "Day Vision:",
                            hero$day_vision
                        )),
                        shiny.fluent::Text(paste(
                            "Night Vision:",
                            hero$night_vision
                        )),
                        shiny.fluent::Text(paste("Legs:", hero$legs)),
                        shiny.fluent::Text(paste(
                            "CM Enabled:",
                            ifelse(hero$cm_enabled, "Yes", "No")
                        ))
                    )
                )
            )
        })

        header$server("header", hero_selected = hero_selected)
    })
}

# fmt: skip
links <- tibble::tribble(
  ~ route, ~ name, ~ icon,
  '','Heroes', 'GridViewSmall',
  'details','Details', 'ProductList',
  'benchmark','Benchmark', 'ProductVariant',
  'rank', 'Rank', 'Medal'
)

#' @export
ui <- function(id) {
    ns <- shiny::NS(id)
    shiny::div(
        class = "wrapper",
        shiny::div(
            class = "grid-container",
            shiny::uiOutput(ns("navbar")),
            shiny::div(class = "main", reactRouter::Outlet())
        )
    )
}

#' @export
server <- function(id, hero_selected) {
    shiny::moduleServer(id, function(input, output, session) {
        ns <- session$ns

        output$navbar <- shiny::renderUI({
            shiny::req(hero_selected())

            navigation <- shiny.fluent::Nav(
                groups = list(
                    list(
                        links = (purrr::map(
                            split(links, seq(nrow(links))),
                            function(.x) {
                                list(
                                    name = .x$name,
                                    url = ifelse(
                                        .x$route == "",
                                        "#/",
                                        paste0(
                                            "#/",
                                            hero_selected(),
                                            "/",
                                            .x$route
                                        )
                                    ),
                                    key = .x$route,
                                    icon = .x$icon
                                )
                            }
                        ) |>
                            unname())
                    )
                ),
                initialSelectedKey = '',
                styles = list(
                    root = list(
                        height = '100%',
                        boxSizing = 'border-box',
                        overflowY = 'auto'
                    )
                )
            )

            shiny::div(class = "sidenav", navigation)
        })
    })
}

# useOutletContext() reads a context value passed through Outlet(context = ...).
# The parent route sets context; any child route reads it without prop-drilling.
# Useful for sharing user info, theme settings, or any route-scoped data.

library(reactRouter)
library(htmltools)

# Simulated current user passed from the parent layout to all child routes
current_user <- list(
  name = "Alice",
  role = "admin",
  email = "alice@example.com"
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        style = "max-width: 520px; margin: 0 auto; padding: 20px; font-family: system-ui;",
        tags$h2("useOutletContext Example"),
        tags$nav(tags$ul(
          tags$li(NavLink(to = "/", "Dashboard")),
          tags$li(NavLink(to = "/profile", "Profile")),
          tags$li(NavLink(to = "/settings", "Settings"))
        )),
        tags$hr(),
        # Pass user info to all child routes via context
        Outlet(context = current_user)
      ),
      Route(
        index = TRUE,
        element = div(
          tags$h3("Dashboard"),
          tags$p(
            "Welcome back, ",
            tags$strong(useOutletContext(tags$span(), selector = "name")),
            "!"
          ),
          tags$p(
            "Your role: ",
            useOutletContext(tags$code(), selector = "role")
          )
        )
      ),
      Route(
        path = "profile",
        element = div(
          tags$h3("Profile"),
          tags$p("Name:  ", useOutletContext(tags$span(), selector = "name")),
          tags$p("Email: ", useOutletContext(tags$span(), selector = "email")),
          tags$p("Role:  ", useOutletContext(tags$span(), selector = "role")),
          tags$details(
            tags$summary("Full context (JSON)"),
            tags$pre(useOutletContext(tags$span()))
          )
        )
      ),
      Route(
        path = "settings",
        element = div(
          tags$h3("Settings"),
          tags$p(
            "Editing settings for ",
            tags$strong(useOutletContext(tags$span(), selector = "name"))
          )
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# Minimal example showing useNavigation() which exposes useNavigation()
# useNavigation returns the navigation state: "idle", "loading", or "submitting"

library(reactRouter)
library(htmltools)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        tags$h2("useNavigation Example"),
        tags$nav(tags$ul(
          tags$li(NavLink(to = "/", "Home")),
          tags$li(NavLink(to = "/slow", "Slow Page (2s)"))
        )),
        tags$p(
          tags$strong("Navigation state: "),
          useNavigation(tags$span(), selector = "state")
        ),
        tags$hr(),
        Outlet()
      ),
      Route(
        index = TRUE,
        element = div(tags$p(
          "Home page. Click 'Slow Page' to see loading state."
        ))
      ),
      Route(
        path = "slow",
        loader = JS(
          "async () => {
            await new Promise(r => setTimeout(r, 2000));
            return { message: 'Data loaded after 2 seconds!' };
          }"
        ),
        element = div(
          tags$h3("Slow Page"),
          tags$p(useLoaderData(
            tags$span(),
            selector = "message"
          ))
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# useLinkClickHandler(to) returns a click handler that drives client-side
# navigation. Use it to make non-anchor elements behave like links —
# buttons, custom widgets, list rows — without giving up React Router's
# history-aware navigation.
#
# Because the hook returns a function, the `render` form is the natural
# fit. The handler can also be injected as a prop (e.g. `as = "onClick"`)
# of an existing element.

library(reactRouter)
library(htmltools)

# A "fake link" built out of a <span>: same navigation behaviour as <Link>,
# different markup.
fakeLink <- function(to, label) {
  useLinkClickHandler(
    to = to,
    render = JS(sprintf(
      "h => React.createElement(
         'span',
         { role: 'link', tabIndex: 0, onClick: h,
           style: {
             cursor: 'pointer',
             color: '#0a66c2',
             textDecoration: 'underline',
             marginRight: '12px'
           }
         },
         '%s'
       )",
      label
    ))
  )
}

Layout <- div(
  style = "max-width: 520px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("useLinkClickHandler Example"),
  tags$p(
    style = "color: #555; font-size: 0.9em;",
    "These ", tags$code("<span>"), "s navigate via ",
    tags$code("useLinkClickHandler"), " — they are not ",
    tags$code("<a>"), " or ", tags$code("<Link>"), " elements."
  ),
  tags$nav(
    style = "margin-bottom: 16px;",
    fakeLink("/",        "Home"),
    fakeLink("/about",   "About"),
    fakeLink("/contact", "Contact")
  ),

  tags$h3("Inject the handler as a button's onClick"),
  tags$p(
    style = "color: #555; font-size: 0.9em;",
    "The button below uses ", tags$code("replace = TRUE"),
    " — clicking it replaces the current history entry instead of pushing a new one."
  ),
  useLinkClickHandler(
    to = "/about",
    replace = TRUE,
    into = tags$button("Replace history with /about"),
    as = "onClick"
  ),

  tags$hr(),
  Outlet()
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE,     element = tags$h3("Home")),
      Route(path = "about",   element = tags$h3("About")),
      Route(path = "contact", element = tags$h3("Contact"))
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# This file allows packrat (used by rsconnect during deployment) to pick up dependencies.
library(rhino)
library(treesitter)
library(treesitter.r)
library(httr)
library(shiny.fluent)
library(reactRouter)
library(stringdist)
library(echarts4r)
library(waiter)

#' @export
primary_attr_color <- function(attr) {
    switch(
        attr,
        "str" = "darkred",
        "agi" = "darkgreen",
        "int" = "darkblue",
        "all" = "purple", # or "indigo"/"#6a0dad" if you want darker purple
        "black" # fallback/default
    )
}

# ScrollRestoration example
# Demonstrates ScrollRestoration() which restores the window scroll position
# when navigating between pages, mimicking native browser behaviour.
# Place <ScrollRestoration /> once inside the root layout of a data router app.

library(reactRouter)
library(htmltools)

# Helper to build a long page so scrolling is visible
long_page <- function(title, color) {
  items <- lapply(seq_len(30), function(i) {
    tags$p(
      style = paste0(
        "padding: 6px 10px; background:",
        color,
        "; border-radius: 4px;"
      ),
      paste("Item", i, "—", title)
    )
  })
  div(
    tags$h3(title),
    tags$p(
      style = "color: gray; font-size: 0.9em;",
      "Scroll down, then navigate away and come back — your position is restored."
    ),
    do.call(div, items)
  )
}

Layout <- div(
  style = "max-width: 520px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("ScrollRestoration Example"),
  tags$nav(tags$ul(
    tags$li(NavLink(to = "/", "Home (long)")),
    tags$li(NavLink(to = "/news", "News (long)")),
    tags$li(NavLink(to = "/about", "About (short)"))
  )),
  tags$hr(),
  # ScrollRestoration must be rendered inside the router layout
  ScrollRestoration(),
  Outlet()
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE, element = long_page("Home", "#e8f4fd")),
      Route(path = "news", element = long_page("News", "#fdf6e8")),
      Route(
        path = "about",
        element = div(
          tags$h3("About"),
          tags$p(
            "This page is short — scrolling back to 'Home' or 'News' restores your previous position."
          )
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# useInRouterContext() returns TRUE when the component is rendered inside
# a router (RouterProvider, HashRouter, MemoryRouter, ...) and FALSE
# otherwise. It is the safe gate to use in reusable components that may
# be mounted with or without a surrounding router — call it before any
# other router hook to avoid runtime errors.

library(reactRouter)
library(htmltools)

# A small "router status" panel reused in two contexts. Inside a router
# it should report TRUE; outside, FALSE.
# React renders bare booleans as nothing, so use `render` to coerce
# the value to a readable string before injecting it.
status_panel <- div(
  style = "padding: 8px 12px; border: 1px solid #ddd; border-radius: 4px;",
  tags$strong("Inside router context: "),
  useInRouterContext(render = JS("v => React.createElement('code', null, String(v))"))
)

# Render the same panel both inside and outside a router. The "inside"
# panel is mounted via RouterProvider(createHashRouter(...)) with a single
# catch-all route — createHashRouter() is preferred over the legacy
# HashRouter() because it unlocks loaders, actions, and errorElement.
inside_router <- RouterProvider(
  router = createHashRouter(
    Route(path = "*", element = status_panel)
  )
)

ui <- div(
  style = "max-width: 480px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("useInRouterContext Example"),

  tags$h4("Outside any router"),
  status_panel,

  tags$h4("Inside a router"),
  inside_router
)

htmltools::browsable(ui)

# useRoutes() builds the route tree imperatively from inside a component,
# instead of declaring it as the value of createHashRouter()'s children.
#
# Two styles are shown below — pick whichever you prefer:
#   1. Pass Route() elements as children (R-idiomatic).
#   2. Pass a `routes` JS() expression with plain route objects (matches
#      the official React Router v7 docs).
#
# useRoutes() must be called inside a router. We mount RouterProvider with
# createHashRouter() and a single catch-all route whose element drives
# routing through useRoutes(). createHashRouter() is preferred over the
# legacy HashRouter() because it unlocks loaders, actions, and errorElement.

library(reactRouter)
library(htmltools)

# Style 1 — Route() children
app_with_route_children <- div(
  tags$nav(
    NavLink(to = "/",      "Home"), " | ",
    NavLink(to = "/about", "About")
  ),
  tags$hr(),
  useRoutes(
    Route(path = "/",      element = tags$h3("Home")),
    Route(path = "/about", element = tags$h3("About")),
    Route(path = "*",      element = tags$h3("Not found"))
  )
)

# Style 2 — plain object array via JS()
app_with_plain_routes <- div(
  tags$nav(
    NavLink(to = "/",      "Home"), " | ",
    NavLink(to = "/about", "About")
  ),
  tags$hr(),
  useRoutes(
    routes = JS("[
      { path: '/',      element: React.createElement('h3', null, 'Home (object route)') },
      { path: '/about', element: React.createElement('h3', null, 'About (object route)') },
      { path: '*',      element: React.createElement('h3', null, 'Not found') }
    ]")
  )
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "*",
      element = div(
        style = "max-width: 480px; margin: 0 auto; padding: 20px; font-family: system-ui;",
        tags$h2("useRoutes Example"),
        tags$h4("Style 1 — Route() children"),
        app_with_route_children,
        tags$hr(),
        tags$h4("Style 2 — plain object array"),
        app_with_plain_routes
      )
    )
  )
)

htmltools::browsable(ui)

# Minimal example using createMemoryRouter() and createRoutesFromElements().
# This uses the data router API with in-memory routing: the browser URL is
# never read or modified, so it works on file:// (static HTML, Quarto).
# Demonstrates the `loader` argument with useLoaderData() to fetch and display data.
# Prefer createHashRouter() when you want the URL to reflect the current page.

library(reactRouter)
library(htmltools)

Home <- div(
  h2("Home"),
  p("Welcome to the Home page.")
)

About <- div(
  h2("About"),
  p("This page was loaded with data from a loader:"),
  tags$blockquote(useLoaderData(
    tags$span(),
    selector = "message"
  )),
  p(
    tags$small(
      "Loaded at: ",
      useLoaderData(tags$span(), selector = "timestamp")
    )
  )
)

NoMatch <- div(
  h2("404 - Not Found"),
  p(NavLink(to = "/", "Go back home"))
)

Layout <- div(
  tags$nav(
    tags$ul(
      tags$li(NavLink(to = "/", end = TRUE, "Home")),
      tags$li(NavLink(to = "/about", "About")),
      tags$li(NavLink(to = "/nothing-here", "Nothing Here"))
    )
  ),
  tags$hr(),
  Outlet()
)

ui <- RouterProvider(
  router = createMemoryRouter(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE, element = Home),
      Route(
        path = "about",
        loader = JS(
          "async () => {
            await new Promise(r => setTimeout(r, 500));
            return {
              message: 'Hello from the About loader!',
              timestamp: new Date().toLocaleTimeString()
            };
          }"
        ),
        element = About
      ),
      Route(path = "*", element = NoMatch)
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# shouldRevalidate Example
# Demonstrates: shouldRevalidate on a Route loader
# https://api.reactrouter.com/v7/types/react-router.ShouldRevalidateFunction.html
#
# By default, React Router re-runs ALL loaders on every navigation.
# shouldRevalidate lets you skip re-runs when the data won't have changed,
# avoiding unnecessary work (network requests, DB queries, etc.).
#
# This example has a root loader that tracks how many times it has been called.
# shouldRevalidate tells React Router to only re-run it when navigating to
# Settings — not when switching between Home, Posts, and About.

library(reactRouter)
library(htmltools)

# Root loader: simulates an "expensive" config fetch.
# Uses a JS closure to count invocations across navigations.
root_loader <- JS(
  "
  (() => {
    let calls = 0;
    return () => {
      calls++;
      return { calls, fetchedAt: new Date().toLocaleTimeString() };
    };
  })()
"
)

# shouldRevalidate controls when the root loader re-runs after the initial load.
# Return true  → re-run the loader (revalidate)
# Return false → skip the loader (use cached data)
#
# Here: only revalidate when navigating TO /settings.
should_revalidate_root <- JS(
  "
  function({ nextUrl }) {
    return nextUrl.pathname === '/settings';
  }
"
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      loader = root_loader,
      shouldRevalidate = should_revalidate_root,
      element = div(
        style = "max-width: 600px; margin: 0 auto; padding: 20px; font-family: system-ui;",
        tags$h2("shouldRevalidate Example"),
        tags$p(
          style = "color: #555; font-size: 0.9em;",
          "The root loader runs once on initial load, then only again when navigating",
          " to Settings. Navigating between Home, Posts, and About skips the loader."
        ),
        # Counter badge — shows how many times the root loader has run
        tags$div(
          style = paste(
            "display: inline-flex; align-items: center; gap: 10px;",
            "background: #f0f7ff; border: 1px solid #90caf9;",
            "border-radius: 6px; padding: 10px 16px; margin-bottom: 16px;"
          ),
          tags$span(style = "color: #555;", "Root loader calls:"),
          useLoaderData(
            tags$strong(style = "font-size: 1.4em; color: #1565c0;"),
            selector = "calls"
          ),
          tags$span(
            style = "color: #aaa; font-size: 0.85em;",
            "\u2014 last at"
          ),
          useLoaderData(
            tags$span(style = "color: #aaa; font-size: 0.85em;"),
            selector = "fetchedAt"
          )
        ),
        # Navigation
        tags$nav(
          tags$ul(
            style = "list-style: none; display: flex; gap: 12px; padding: 0; margin: 0 0 12px;",
            tags$li(NavLink(to = "/", "Home")),
            tags$li(NavLink(to = "/posts", "Posts")),
            tags$li(NavLink(to = "/about", "About")),
            tags$li(NavLink(
              to = "/settings",
              style = "color: #b71c1c; font-weight: bold;",
              "Settings \u2192 triggers revalidation"
            ))
          )
        ),
        tags$hr(),
        Outlet()
      ),
      # Index route
      Route(
        index = TRUE,
        element = div(
          tags$h3("Home"),
          tags$p(
            "Click Posts or About \u2014 the counter above stays the same."
          ),
          tags$p("Click Settings \u2014 the counter increments.")
        )
      ),
      Route(
        path = "posts",
        element = div(
          tags$h3("Posts"),
          tags$p(
            style = "color: #2e7d32;",
            "\u2713 Root loader was skipped (shouldRevalidate returned false)."
          )
        )
      ),
      Route(
        path = "about",
        element = div(
          tags$h3("About"),
          tags$p(
            style = "color: #2e7d32;",
            "\u2713 Root loader was skipped (shouldRevalidate returned false)."
          )
        )
      ),
      Route(
        path = "settings",
        element = div(
          tags$h3("Settings"),
          tags$p(
            style = "color: #b71c1c;",
            "\u2191 Root loader RAN (shouldRevalidate returned true for this route)."
          )
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# Minimal dynamic-segment example.
# Demonstrates: createHashRouter() with a `:id` route param, useParams(),
#               NavLink, Outlet. Uses the default `reloadDocument = FALSE`
#               (the recommended setting — see vignette("routers")).
# For a richer dynamic-segment app with loaders, errorElements, charts, and
# data grids, see reactRouterExample("star-wars-explorer").

library(reactRouter)
library(htmltools)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        h3("reactRouter — dynamic segments"),
        tags$nav(
          NavLink(to = "/project/1", "Project 1"), " | ",
          NavLink(to = "/project/2", "Project 2"), " | ",
          NavLink(to = "/project/3", "Project 3")
        ),
        tags$hr(),
        Outlet()
      ),
      Route(
        index = TRUE,
        element = p("Select a project above.")
      ),
      Route(
        path = "project/:id",
        element = div(
          h4("Project ", useParams(tags$strong(), selector = "id")),
          p("The route parameter `:id` is read with useParams().")
        )
      ),
      Route(path = "*", element = div(h4("Not found"), NavLink(to = "/", "Home")))
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# Inline search using useFetcher() — fetch data without navigating.
#
# Key: use Form(navigate = FALSE, fetcherKey = "...") to submit without a
# page-level navigation. useFetcher() with the same fetcherKey reads that
# fetcher's state and data.
#
# useFetcher() exposes (via selector):
#   state       "idle" | "loading" | "submitting"
#   data.count  value returned by the route loader

library(reactRouter)
library(htmltools)
library(jsonlite)

people_json <- jsonlite::toJSON(
  data.frame(name = c("Alice", "Bob", "Charlie", "David", "Eve")),
  dataframe = "rows",
  auto_unbox = TRUE
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        tags$h2("useFetcher Example — Inline Search"),
        tags$p(
          "Type a name and click Search. The URL does not change.",
          style = "color: #555;"
        ),
        # Form with navigate = FALSE submits via a fetcher (no URL change).
        # fetcherKey ties this form to the useFetcher() calls below.
        Form(
          navigate = FALSE,
          fetcherKey = "search",
          method = "get",
          action = "/search",
          style = "display: flex; gap: 8px; margin-bottom: 16px;",
          tags$input(
            type = "search",
            name = "q",
            placeholder = "Search names...",
            style = "padding: 6px; font-size: 14px;"
          ),
          tags$button(
            type = "submit",
            "Search",
            style = "padding: 6px 12px; cursor: pointer;"
          )
        ),
        tags$p(
          tags$strong("Fetcher state: "),
          useFetcher(tags$span(), fetcherKey = "search", selector = "state")
        ),
        tags$p(
          tags$strong("Matches found: "),
          useFetcher(
            tags$span(),
            fetcherKey = "search",
            selector = "data.count"
          )
        ),
        tags$details(
          tags$summary("Raw results (JSON)"),
          tags$pre(useFetcher(
            tags$span(),
            fetcherKey = "search",
            selector = "data.results"
          ))
        ),
        tags$hr(),
        Outlet()
      ),
      Route(
        index = TRUE,
        element = div(
          tags$p("Enter a name above and click Search.")
        )
      ),
      Route(
        path = "search",
        loader = JS(sprintf(
          "async ({ request }) => {
          await new Promise(resolve => setTimeout(resolve, 800));
          const q = new URL(request.url).searchParams.get('q') || '';
          const db = %s;
          const hits = db.filter(r =>
            r.name.toLowerCase().includes(q.toLowerCase())
          );
          return { count: hits.length, results: hits };
        }",
          people_json
        )),
        element = div()
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# useNavigate() returns an imperative navigate function.
# Use it to trigger navigation from arbitrary event handlers —
# outside of <Link> / <NavLink> — e.g. after a button click or
# once some async work has completed.
#
# Because the hook returns a function (not a value), the `render`
# form is the natural fit: `render = JS("nav => <...>")`.

library(reactRouter)
library(htmltools)

# render receives the navigate function:
#   nav("/path")       push a new entry
#   nav("/path", { replace: true })
#   nav(-1)            go back one entry
nav_buttons <- JS(
  "nav => React.createElement('div', null,
    React.createElement('button', { onClick: () => nav('/about') }, 'Go to About'),
    ' ',
    React.createElement('button', { onClick: () => nav('/contact') }, 'Go to Contact'),
    ' ',
    React.createElement('button', { onClick: () => nav(-1) }, 'Back')
  )"
)

Layout <- div(
  style = "max-width: 480px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("useNavigate Example"),
  tags$p(
    style = "color: #555; font-size: 0.9em;",
    "The buttons below call ", tags$code("navigate()"),
    " directly — no ", tags$code("<Link>"), " needed."
  ),
  useNavigate(render = nav_buttons),
  tags$hr(),
  Outlet()
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE, element = tags$h3("Home")),
      Route(path = "about", element = tags$h3("About")),
      Route(path = "contact", element = tags$h3("Contact"))
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# useOutlet() returns the matched child route's element, or NULL if no
# child route is matched. Unlike the <Outlet/> component, this lets you
# branch on whether the user is on the parent route itself and render a
# fallback (e.g. a "pick something" hint) instead of a blank slot.

library(reactRouter)
library(htmltools)

# render receives the matched child element (or null on the index/parent
# route). Falling back to a hint keeps the layout from looking empty.
outlet_or_hint <- JS(
  "o => o || React.createElement(
    'p',
    { style: { color: '#888', fontStyle: 'italic' } },
    'Pick a section from the menu above to see its content here.'
  )"
)

Layout <- div(
  style = "max-width: 520px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("useOutlet Example"),
  tags$p(
    style = "color: #555; font-size: 0.9em;",
    "The area below renders the matched child route, or a fallback when ",
    "you are on the parent route."
  ),
  tags$nav(tags$ul(
    tags$li(NavLink(to = "/", "Home (no child)")),
    tags$li(NavLink(to = "/news", "News")),
    tags$li(NavLink(to = "/about", "About"))
  )),
  tags$hr(),
  tags$section(
    style = "padding: 12px; border: 1px dashed #ccc; border-radius: 6px;",
    useOutlet(render = outlet_or_hint)
  )
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = Layout,
      Route(path = "news", element = tags$h3("Latest news")),
      Route(path = "about", element = tags$h3("About us"))
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

library(reactRouter)
library(bslib)
library(htmltools)
library(shiny)

Layout <- bslib::page_navbar(
  title = "reactRouter with bslib",
  nav_item(
    reactRouter::NavLink(
      "Home",
      to = "/",
      style = JS(
        '({isActive}) => { return isActive ? {color: "red", textDecoration: "none"} : {}; }'
      )
    )
  ),
  nav_item(
    reactRouter::NavLink(
      "Analysis",
      to = "/analysis",
      style = JS(
        '({isActive}) => { return isActive ? {color: "red", textDecoration: "none"} : {}; }'
      )
    )
  ),
  reactRouter::Outlet()
)

ui <- reactRouter::RouterProvider(
  router = createHashRouter(
    reactRouter::Route(
      path = "/",
      element = Layout,
      reactRouter::Route(
        index = TRUE,
        element = div(
          tags$h3("Home page"),
          p("A basic example of reactRouter with bslib.")
        )
      ),
      reactRouter::Route(
        path = "analysis",
        element = "Content analysis"
      ),
      reactRouter::Route(path = "*", element = "Custom error 404")
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# useLoaderData example — MUI Data Grid + Autocomplete filter powered by JS loaders
# Demonstrates: useLoaderData wrapping muiDataGrid::DataGrid() and muiMaterial::Autocomplete()
# Requires: library(muiDataGrid), library(muiMaterial)

library(reactRouter)
library(muiDataGrid) # https://github.com/lgnbhl/muiDataGrid
library(muiMaterial) # https://github.com/lgnbhl/muiMaterial
library(htmltools)

# -- JS loader: fetches Star Wars characters from SWAPI, filters by ?name= param ----
loader_people <- JS(
  "
  async ({ request }) => {
    const filter = new URL(request.url).searchParams.getAll('name');
    const data = await fetch('https://swapi.info/api/people').then(r => r.json());
    return {
      names: data.map(p => p.name).sort(),
      people: filter.length ? data.filter(p => filter.includes(p.name)) : data
    };
  }
"
)

# -- UI --------------------------------------------------------------------
ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      loader = loader_people,
      element = div(
        h2("Star Wars Characters"),
        p(
          "Select characters to filter the table, or clear the selection to show all."
        ),
        useLoaderData(
          into = muiMaterial::Autocomplete(
            onChange = JS(
              "(event, value) => { window.location.hash = value.length ? '/?' + value.map(v => 'name=' + encodeURIComponent(v)).join('&') : '/'; }"
            ),
            renderInput = JS(
              "(params) => React.createElement(window.jsmodule['@mui/material'].TextField, {...params, label: 'Name'})"
            ),
            multiple = TRUE,
            sx = list(width = 300, marginBottom = 2)
          ),
          as = "options",
          selector = "names"
        ),
        useLoaderData(
          into = muiDataGrid::DataGrid(
            columns = list(
              list(field = 'name', headerName = 'Name', flex = 1),
              list(field = 'height', headerName = 'Height', width = 100),
              list(field = 'mass', headerName = 'Mass', width = 100),
              list(field = 'gender', headerName = 'Gender', width = 120),
              list(field = 'birth_year', headerName = 'Birth Year', width = 120)
            ),
            getRowId = JS("function(row) { return(row.name) }"),
            initialState = list(
              pagination = list(paginationModel = list(pageSize = 5))
            ),
            showToolbar = TRUE
          ),
          as = "rows",
          selector = "people"
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
browsable(ui)

# Minimal example showing useRouteError() which exposes useRouteError()
# useRouteError displays error information inside an errorElement

library(reactRouter)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        tags$h2("useRouteError Example"),
        tags$nav(tags$ul(
          tags$li(NavLink(to = "/", "Home")),
          tags$li(NavLink(to = "/broken", "Broken Page"))
        )),
        tags$hr(),
        Outlet()
      ),
      Route(
        index = TRUE,
        element = div(tags$p("Home page - try the broken link!"))
      ),
      Route(
        path = "broken",
        loader = JS(
          "async () => { throw new Error('Something went wrong!'); }"
        ),
        element = div(tags$p("This should not render.")),
        errorElement = div(
          tags$h3(style = "color: red;", "Error!"),
          tags$p(
            "Error message: ",
            useRouteError(tags$span(), selector = "message")
          )
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# useAsyncValue() — reads the resolved value of the closest <Await>.
# Used in "children mode" of Await(): when no `into` / `render` is
# given on Await, its children are rendered as descendants of the
# underlying React Router <Await>, and useAsyncValue() picks up the
# resolved promise value for any node beneath it.

library(reactRouter)
library(htmltools)

Layout <- div(
  style = "max-width: 560px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("useAsyncValue Example"),
  tags$nav(tags$ul(
    tags$li(NavLink(to = "/", "Home")),
    tags$li(NavLink(to = "/report", "Report (deferred)"))
  )),
  tags$hr(),
  Outlet()
)

ui <- RouterProvider(
  router = createMemoryRouter(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE, element = tags$p("Open the Report tab.")),
      Route(
        path = "report",
        loader = JS(
          "async () => ({
             metrics: new Promise((resolve) =>
               setTimeout(() => resolve({
                 revenue: '$1,240,000',
                 users:   '18,430',
                 growth:  '+14.2%'
               }), 1500)
             )
           })"
        ),
        element = div(
          tags$h3("Quarterly Metrics"),
          # Children-mode Await: no into/render, the children are rendered
          # inside <Await> so descendants can call useAsyncValue().
          Await(
            resolveKey = "metrics",
            fallback = tags$em(style = "color:gray;", "loading…"),
            div(
              style = "background:#f8f9fa; padding:12px; border-radius:6px;",
              tags$p(tags$strong("Revenue: "), useAsyncValue(tags$span(), selector = "revenue")),
              tags$p(tags$strong("Users: "),   useAsyncValue(tags$span(), selector = "users")),
              tags$p(tags$strong("Growth: "),  useAsyncValue(tags$span(), selector = "growth")),
              tags$hr(),
              tags$p(
                tags$strong("Whole object: "),
                useAsyncValue(tags$pre(style = "background:#fff;padding:8px;"))
              )
            )
          )
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# useAsyncError() — reads the rejection reason of the closest <Await>.
# Pair it with an `errorElement` rendered via the children-mode Await:
# the deferred promise rejects, the errorElement subtree mounts, and
# useAsyncError() inside it surfaces the error message.

library(reactRouter)
library(htmltools)

ErrorBox <- div(
  style = "background:#fdecea; border:1px solid #f5c2c7; padding:12px; border-radius:6px;",
  tags$strong("Something went wrong: "),
  useAsyncError(tags$span(style = "color:#842029;"), selector = "message")
)

Layout <- div(
  style = "max-width: 560px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("useAsyncError Example"),
  tags$nav(tags$ul(
    tags$li(NavLink(to = "/", "Home")),
    tags$li(NavLink(to = "/flaky", "Flaky endpoint (always fails)"))
  )),
  tags$hr(),
  Outlet()
)

ui <- RouterProvider(
  router = createMemoryRouter(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE, element = tags$p("Open the flaky endpoint.")),
      Route(
        path = "flaky",
        loader = JS(
          "async () => ({
             payload: new Promise((_, reject) =>
               setTimeout(() => reject(new Error('upstream service unavailable')), 800)
             )
           })"
        ),
        element = div(
          tags$h3("Flaky Endpoint"),
          # Children-mode Await with an errorElement subtree.
          Await(
            resolveKey = "payload",
            fallback = tags$em(style = "color:gray;", "fetching…"),
            errorElement = ErrorBox,
            tags$p(useAsyncValue(tags$span()))
          )
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# useSearchParams() — when used with `render`, the JS function receives
# BOTH the current params and a setSearchParams setter as a second
# argument. Use the setter to update the URL's query string
# programmatically (without constructing a new href).

library(reactRouter)
library(htmltools)

# render receives (value, setSearchParams):
#   setSearchParams({ key: value })                 — replace all params
#   setSearchParams(prev => { prev.set(k,v); return prev; })  — update
color_controls <- JS(
  "(value, setSearchParams) => {
     const current = value.color ? value.color.join(', ') : '(none)';
     const setTo = (c) => () => setSearchParams(c ? { color: c } : {});
     return React.createElement('div', null,
       React.createElement('p', null,
         React.createElement('strong', null, 'color: '),
         React.createElement('code', null, current)
       ),
       React.createElement('button', { onClick: setTo('red')   }, 'Set red'),   ' ',
       React.createElement('button', { onClick: setTo('blue')  }, 'Set blue'),  ' ',
       React.createElement('button', { onClick: setTo('green') }, 'Set green'), ' ',
       React.createElement('button', { onClick: setTo(null)    }, 'Clear')
     );
   }"
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        style = "max-width: 520px; margin: 0 auto; padding: 20px; font-family: system-ui;",
        tags$h2("setSearchParams Example"),
        tags$p(
          style = "color: #555; font-size: 0.9em;",
          "Buttons below update the URL query string via ",
          tags$code("setSearchParams"), ". Watch the address bar change."
        ),
        useSearchParams(render = color_controls)
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# Example showing useLoaderData() which exposes useLoaderData()
# Demonstrates:
#   - Accessing individual keys with `selector`
#   - Combining useLoaderData with useParams for dynamic loaders
#   - Using useRouteLoaderData to access a parent route's loader from a child

library(reactRouter)
library(htmltools)
library(dplyr)

# --- Data defined in R -----------------------------------------------------------

# Serialize for use in JS loaders
# Array of objects
people_json <- dplyr::starwars |>
  mutate(id = dplyr::row_number()) |>
  jsonlite::toJSON(dataframe = "rows", auto_unbox = TRUE)

# --- App -------------------------------------------------------------------------

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = div(
        tags$h2("Loader Demo"),
        tags$nav(tags$ul(
          tags$li(NavLink(to = "/", "Home")),
          tags$li(NavLink(to = "/people", "People List")),
          tags$li(NavLink(to = "/people/1", "Person 1")),
          tags$li(NavLink(to = "/people/2", "Person 2")),
          tags$li(NavLink(to = "/people/3", "Person 3"))
        )),
        tags$hr(),
        Outlet()
      ),
      Route(
        index = TRUE,
        element = tags$p("Welcome! Select a page from the nav above.")
      ),
      Route(
        path = "people",
        element = div(
          muiDataGrid::DataGrid(
            rows = dplyr::starwars,
            initialState = list(
              pagination = list(paginationModel = list(pageSize = 5))
            ),
            showToolbar = TRUE
          )
        )
      ),
      Route(
        path = "people/:id",
        loader = JS(
          sprintf(
            "({ params }) => {
            const starwars_db = %s;
            
            // Think of .find() as a subset() that returns only the first match
            const person = starwars_db.find(row => row.id == params.id);
            
            // Error handling (similar to an if/stop statement in R)
            if (!person) {
              throw new Response('Character not found', { status: 404 });
            }
            
            return person;
            }",
            people_json
          )
        ),
        element = div(
          useLoaderData(tags$h3(), selector = "name"),
          tags$table(
            style = "border-collapse: collapse;",
            tags$tr(
              tags$th(style = "text-align: left; padding: 4px 12px;", "Gender"),
              tags$td(
                style = "padding: 4px 12px;",
                useLoaderData(tags$span(), selector = "gender")
              )
            ),
            tags$tr(
              tags$th(
                style = "text-align: left; padding: 4px 12px;",
                "Species"
              ),
              tags$td(
                style = "padding: 4px 12px;",
                useLoaderData(tags$span(), selector = "species")
              )
            )
          )
        ),
        errorElement = div(
          tags$h3(style = "color: red;", "Person not found"),
          tags$p(NavLink(to = "/people", "Back to people list"))
        )
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# Minimal example using createHashRouter() with RouterProvider().
# This uses the data router API instead of the component-based <HashRouter> + <Routes>.
# Demonstrates the `loader` argument with useLoaderData() to fetch and display data.

library(reactRouter)
library(htmltools)

Home <- div(
  h2("Home"),
  p("Welcome to the Home page.")
)

About <- div(
  h2("About"),
  p("This page was loaded with data from a loader:"),
  tags$blockquote(
    useLoaderData(
      tags$span(),
      selector = "message"
    )
  ),
  p(
    tags$small(
      "Loaded at: ",
      useLoaderData(
        tags$span(),
        selector = "timestamp"
      )
    )
  )
)

NoMatch <- div(
  h2("404 - Not Found"),
  p(
    NavLink(to = "/", "Go back home")
  )
)

Layout <- div(
  tags$nav(
    tags$ul(
      tags$li(NavLink(to = "/", "Home")),
      tags$li(NavLink(to = "/about", "About")),
      tags$li(NavLink(to = "/nothing-here", "Nothing Here"))
    )
  ),
  tags$hr(),
  Outlet()
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE, element = Home),
      Route(
        path = "about",
        loader = JS(
          "async () => {
            await new Promise(r => setTimeout(r, 500));
            return {
              message: 'Hello from the About loader!',
              timestamp: new Date().toLocaleTimeString()
            };
          }"
        ),
        element = About
      ),
      Route(path = "*", element = NoMatch)
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# Minimal example using MemoryRouter() as a static HTML page.
# MemoryRouter keeps routing state in memory and always starts at "/",
# making it ideal for static files (file://) where the real URL is not "/".
# BrowserRouter cannot be used here because it reads the real URL path on
# init, which on file:// is the file path (not "/"), causing a 404.
# For Shiny apps, prefer HashRouter().
# View with: htmltools::browsable(ui) or htmltools::save_html(ui, "index.html")

library(reactRouter)
library(htmltools)

Home <- div(
  h2("Home"),
  p("Welcome to the Home page.")
)

About <- div(
  h2("About"),
  p("This is the About page.")
)

NoMatch <- div(
  h2("404 - Not Found"),
  p(NavLink(to = "/", "Go back home"))
)

Layout <- div(
  tags$nav(
    tags$ul(
      tags$li(NavLink(to = "/", end = TRUE, "Home")),
      tags$li(NavLink(to = "/about", "About")),
      tags$li(NavLink(to = "/nothing-here", "Nothing Here"))
    )
  ),
  tags$hr(),
  Outlet()
)

ui <- MemoryRouter(
  Routes(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE, element = Home),
      Route(path = "about", element = About),
      Route(path = "*", element = NoMatch)
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# useViewTransitionState(to) returns TRUE while a View Transitions API
# navigation toward `to` is in progress. Pair it with the `viewTransition`
# prop on <Link> / <NavLink> to drive transition-aware styling — e.g. a
# pulsing label on the link that just got clicked.
#
# Note: requires a browser that supports the View Transitions API
# (Chromium-based browsers and recent Safari).

library(reactRouter)
library(htmltools)

# Tiny helper: NavLink with viewTransition enabled, that bolds + tints itself
# while a transition toward its target is in flight.
animatedLink <- function(to, label) {
  useViewTransitionState(
    to = to,
    render = JS(sprintf(
      "isPending => React.createElement(
         window.jsmodule['react-router-dom'].NavLink,
         { to: '%s', viewTransition: true,
           style: {
             fontWeight: isPending ? 'bold' : 'normal',
             color:      isPending ? '#0a66c2' : 'inherit',
             marginRight: '12px'
           }
         },
         '%s'
       )",
      to, label
    ))
  )
}

Layout <- div(
  style = "max-width: 520px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("useViewTransitionState Example"),
  tags$p(
    style = "color: #555; font-size: 0.9em;",
    "Click a link below — the link to the page being transitioned to is ",
    "briefly bolded and tinted while the transition runs."
  ),
  tags$nav(
    style = "margin-bottom: 16px;",
    animatedLink("/",       "Home"),
    animatedLink("/about",  "About"),
    animatedLink("/contact", "Contact")
  ),
  tags$hr(),
  Outlet()
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE,    element = tags$h3("Home")),
      Route(path = "about",  element = tags$h3("About")),
      Route(path = "contact", element = tags$h3("Contact"))
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

# redirect() returns a JS loader that redirects to another route.
# Use it as a Route's `loader` to build "guard" or "alias" routes that
# always send the user elsewhere.
#
# For conditional redirects inside a custom loader/action, call the
# global `window.jsmodule['@/reactRouter'].helpers.redirect(to)` from your own JS()
# string — see the "conditional" route below.

library(reactRouter)
library(htmltools)

# Conditional redirect written as a custom loader. Flips a coin and
# either redirects to /target or returns some data.
coin_flip_loader <- JS(
  "() => {
     if (Math.random() < 0.5) {
       return window.jsmodule['@/reactRouter'].helpers.redirect('/target');
     }
     return { message: 'You got lucky — no redirect this time.' };
   }"
)

Layout <- div(
  style = "max-width: 540px; margin: 0 auto; padding: 20px; font-family: system-ui;",
  tags$h2("redirect Example"),
  tags$nav(tags$ul(
    tags$li(NavLink(to = "/", "Home")),
    tags$li(NavLink(to = "/old", "/old (always redirects to /target)")),
    tags$li(NavLink(to = "/maybe", "/maybe (50/50 redirect)")),
    tags$li(NavLink(to = "/target", "/target"))
  )),
  tags$hr(),
  Outlet()
)

ui <- RouterProvider(
  router = createHashRouter(
    Route(
      path = "/",
      element = Layout,
      Route(index = TRUE, element = tags$p("Pick a link above.")),
      # Static redirect: /old -> /target
      Route(
        path = "old",
        loader = redirect("/target"),
        element = NULL
      ),
      # Conditional redirect using window.jsmodule['@/reactRouter'].helpers.redirect
      Route(
        path = "maybe",
        loader = coin_flip_loader,
        element = div(
          tags$h3("/maybe"),
          useLoaderData(tags$p(), selector = "message")
        )
      ),
      Route(
        path = "target",
        element = tags$h3("/target — you arrived.")
      )
    )
  )
)

# htmltools::save_html(ui, "index.html")
htmltools::browsable(ui)

library(shiny)

ui <- reactRouter::HashRouter(
  reactRouter::NavLink.shinyInput(
    inputId = "NavLinkHome",
    to = "/",
    "Home"
  ),
  div(),
  reactRouter::NavLink.shinyInput(
    inputId = "NavLinkPage",
    to = "page",
    "Page"
  ),
  hr(),
  reactRouter::Routes(
    reactRouter::Route(
      path = "/",
      element = div(
        uiOutput(outputId = "outputHome")
      )
    ),
    reactRouter::Route(
      path = "page",
      element = div(
        uiOutput(outputId = "outputPage")
      )
    )
  )
)

server <- function(input, output, session) {
  output$outputHome <- renderUI({
    p("home content")
  })
  output$outputPage <- renderUI({
    p("page content")
  })
  # observe({
  #   # print full URL hash
  #   client_data <- reactiveValuesToList(session$clientData)
  #   url_hostname <- client_data$url_hostname
  #   url_port <- client_data$url_port
  #   url_hash <- client_data$url_hash
  #   print(paste0(url_hostname, ":", url_port, "/", url_hash))
  # })
}

shinyApp(ui, server)

library(shiny)

ui <- reactRouter::RouterProvider(
  router = reactRouter::createHashRouter(
    reactRouter::Route(
      path = "/",
      element = div(
        reactRouter::Outlet()
      ),
      reactRouter::Route(
        path = "user/:id",
        element = div(
          div(
            id = "paramId",
            reactRouter::useParams(
              tags$span(),
              selector = "id"
            )
          ),
          div(
            id = "paramAll",
            reactRouter::useParams(
              tags$span()
            )
          )
        )
      )
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

library(shiny)

ui <- reactRouter::MemoryRouter(
  initialEntries = list("/about"),
  reactRouter::Routes(
    reactRouter::Route(
      path = "/",
      element = div(id = "memHome", tags$p("memory home"))
    ),
    reactRouter::Route(
      path = "about",
      element = div(id = "memAbout", tags$p("memory about"))
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

library(shiny)

ui <- reactRouter::RouterProvider(
  router = reactRouter::createHashRouter(
    reactRouter::Route(
      id = "root",
      path = "/",
      loader = reactRouter::JS("async () => { return { loaded: true }; }"),
      element = div(
        div(
          id = "matchesAll",
          reactRouter::useMatches(
            tags$span()
          )
        ),
        reactRouter::Outlet()
      ),
      reactRouter::Route(
        path = "form",
        action = reactRouter::JS(
          "async ({ request }) => {
              const formData = await request.formData();
              return { submitted: formData.get('name') };
            }"
        ),
        element = div(
          div(
            id = "actionData",
            reactRouter::useActionData(
              tags$span()
            )
          ),
          div(
            id = "actionField",
            reactRouter::useActionData(
              tags$span(),
              selector = "submitted"
            )
          ),
          reactRouter::Form(
            method = "post",
            tags$input(type = "hidden", name = "name", value = "Bob"),
            tags$button(id = "submitBtn", type = "submit", "Submit")
          )
        )
      )
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

library(shiny)

ui <- reactRouter::RouterProvider(
  router = reactRouter::createHashRouter(
    reactRouter::Route(
      path = "/",
      element = div(reactRouter::Outlet()),
      reactRouter::Route(
        path = "products/:id",
        element = div(
          div(
            id = "matchFull",
            reactRouter::useMatch(
              tags$span(),
              pattern = "/products/:id"
            )
          ),
          div(
            id = "matchParams",
            reactRouter::useMatch(
              tags$span(),
              pattern = "/products/:id",
              selector = "params"
            )
          ),
          div(
            id = "matchPathname",
            reactRouter::useMatch(
              tags$span(),
              pattern = "/products/:id",
              selector = "pathname"
            )
          ),
          div(
            id = "noMatch",
            reactRouter::useMatch(
              tags$span(),
              pattern = "/other"
            )
          )
        )
      )
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

#' Run reactRouterExample example
#'
#' Launch a Shiny example app or list the available examples.
#' Use `reactRouter::reactRouterExample("basic")` to run a showcase app.
#'
#' @param example The name of the example to run, or `NULL` to print and
#'   invisibly return the list of available examples.
#' @param ... Additional arguments to pass to `shiny::runApp()`.
#' @return When `example` is `NULL`, invisibly returns a character vector of
#'   example names (also printed via `message()`). Otherwise this function
#'   normally does not return; interrupt R to stop the application
#'   (usually by pressing Ctrl+C or Esc).
#'
#' @seealso \code{shiny.blueprint::runExample()}, which this function is
#'   adapted from.
#'
#' @export
reactRouterExample <- function(example = NULL, ...) {
  examples <- system.file("examples", package = utils::packageName(), mustWork = TRUE)
  valid <- sub("\\.R$", "", list.files(examples))
  if (is.null(example)) {
    message(
      "Available reactRouter examples:\n",
      paste0("  - ", valid, collapse = "\n"),
      "\n\nRun one with reactRouterExample(\"<name>\")."
    )
    return(invisible(valid))
  }
  if (!is.character(example) || length(example) != 1 || is.na(example)) {
    stop("reactRouterExample(): `example` must be a single, non-NA character string.", call. = FALSE)
  }
  if (!example %in% valid) {
    stop(sprintf(
      "reactRouterExample(): unknown example '%s'. Available: %s.",
      example, paste(valid, collapse = ", ")
    ), call. = FALSE)
  }
  shiny::runApp(file.path(examples, example), ...)
}

library(shiny)

ui <- reactRouter::RouterProvider(
  router = reactRouter::createHashRouter(
    reactRouter::Route(
      id = "root",
      path = "/",
      loader = reactRouter::JS(
        "async () => { return { title: 'Root Data', version: 1 }; }"
      ),
      element = div(
        reactRouter::Outlet()
      ),
      reactRouter::Route(
        path = "child",
        element = div(
          div(
            id = "rootDataAll",
            reactRouter::useRouteLoaderData(
              tags$span(),
              routeId = "root"
            )
          ),
          div(
            id = "rootDataTitle",
            reactRouter::useRouteLoaderData(
              tags$span(),
              routeId = "root",
              selector = "title"
            )
          )
        )
      )
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

library(shiny)

# Deferred loader: returns a fast value plus a promise that resolves shortly
# after route entry. Await() should first show the fallback then the resolved
# value. Children-mode Await wraps a useAsyncValue() to test that path too.
ui <- reactRouter::RouterProvider(
  router = reactRouter::createHashRouter(
    reactRouter::Route(
      path = "/",
      element = div(reactRouter::Outlet()),
      reactRouter::Route(
        path = "report",
        loader = reactRouter::JS(
          "async () => ({
             title: 'Q1',
             details: new Promise(r => setTimeout(
               () => r({ revenue: '$1.2M', users: 18430 }), 250
             ))
           })"
        ),
        element = div(
          div(
            id = "title",
            reactRouter::useLoaderData(tags$span(), selector = "title")
          ),
          # Target mode: selector pulls one field from the resolved value.
          div(
            id = "revenue",
            reactRouter::Await(
              tags$span(),
              resolveKey = "details",
              selector = "revenue",
              fallback = tags$em(id = "revenueFallback", "loading-revenue")
            )
          ),
          # render = JS(...) target mode: receives the full resolved value.
          div(
            id = "usersRender",
            reactRouter::Await(
              resolveKey = "details",
              render = reactRouter::JS("d => `${d.users} users`"),
              fallback = tags$em(id = "usersFallback", "loading-users")
            )
          )
        )
      )
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

#' JavaScript helpers exposed on \code{window.jsmodule['@/reactRouter'].helpers}
#'
#' For convenience, the package exposes a curated set of React Router v7
#' utility functions under \code{window.jsmodule['@/reactRouter'].helpers}
#' so user-authored \code{\link{JS}} loaders, actions, and render callbacks
#' can call them without reaching into webpack internals. The same names are
#' available as on the JS side of React Router itself.
#'
#' Scoped under the \pkg{shiny.react} \code{jsmodule} registry rather than
#' \code{window} so the package never adds a top-level global to the host
#' page. A short alias is the only ergonomic cost:
#' \preformatted{
#'   loader = JS("async () => {
#'     const { redirect } = window.jsmodule['@/reactRouter'].helpers;
#'     if (!await ok()) return redirect('/login');
#'     return { ok: true };
#'   }")
#' }
#'
#' \strong{Loader/action response helpers}
#' \itemize{
#'   \item \code{redirect(to, init?)} -- client-side navigation redirect.
#'   \item \code{replace(to, init?)} -- redirect that replaces the history entry.
#'   \item \code{redirectDocument(to)} -- full document reload redirect.
#'   \item \code{data(value, init?)} -- attach status/headers to a payload.
#' }
#'
#' \strong{Path / URL utilities}
#' \itemize{
#'   \item \code{generatePath(path, params)} -- build a URL from a pattern.
#'   \item \code{matchPath(pattern, pathname)} -- match a pathname.
#'   \item \code{matchRoutes(routes, location, basename?)} -- match a route
#'     tree against a location.
#'   \item \code{resolvePath(to, fromPathname?)} -- resolve a relative path.
#'   \item \code{parsePath(path)} -- split into \code{pathname/search/hash}.
#'   \item \code{createPath(parts)} -- inverse of \code{parsePath}.
#'   \item \code{createSearchParams(init)} -- build a \code{URLSearchParams}.
#' }
#'
#' These run in the browser only (inside \code{\link{JS}} loader/action code).
#' For R-side construction of a \code{Link()} \code{to} value from data, use
#' base R: \code{paste0("/users/", utils::URLencode(id, reserved = TRUE))}.
#'
#' \strong{Error helpers}
#' \itemize{
#'   \item \code{isRouteErrorResponse(error)} -- type guard intended for use
#'     inside an \code{errorElement} alongside \code{\link{useRouteError}};
#'     returns \code{true} when the error came from a thrown
#'     \code{Response} (e.g. \code{throw new Response(..., { status: 404 })}).
#' }
#'
#' These are the JavaScript implementations from \code{react-router-dom}.
#'
#' @examples
#' \dontrun{
#' # Conditional redirect inside a custom loader.
#' Route(
#'   path = "/admin",
#'   loader = JS("async () => {
#'     const { redirect } = window.jsmodule['@/reactRouter'].helpers;
#'     const ok = await checkAuth();
#'     if (!ok) return redirect('/login');
#'     return { ok: true };
#'   }"),
#'   element = useLoaderData(tags$pre())
#' )
#'
#' # Branch on whether the route error is a Response.
#' Route(
#'   path = "/items/:id",
#'   loader = JS("async ({ params }) => {
#'     const r = await fetch('/api/items/' + params.id);
#'     if (!r.ok) throw new Response('Not found', { status: 404 });
#'     return r.json();
#'   }"),
#'   errorElement = useRouteError(
#'     render = JS(
#'       "e => window.jsmodule['@/reactRouter'].helpers.isRouteErrorResponse(e)" +
#'       " ? <p>HTTP {e.status}</p> : <p>Unknown error</p>"
#'     )
#'   ),
#'   element = useLoaderData(tags$pre())
#' )
#' }
#'
#' @name reactRouterHelpers
#' @keywords internal
NULL

# Internal: JS expression that resolves to the package's helpers namespace.
# All R-side `redirect()`, `replaceResponse()`, etc. emit JS strings that
# look up helpers through this path so the package never adds anything to
# the global `window` object — see ?reactRouterHelpers.
HELPERS_JS <- "window.jsmodule['@/reactRouter'].helpers"

# Internal: produce a JavaScript string literal (including the surrounding
# quotes) for an R string. Delegates to jsonlite for full escaping of control
# characters, unicode line separators, and `</script>`.
jsLiteral <- function(x) {
  as.character(jsonlite::toJSON(x, auto_unbox = TRUE))
}

# Internal: reject URL schemes that can execute code when used as a redirect
# target (javascript:, data:, vbscript:). Allows everything else, including
# absolute http(s) URLs and root-relative paths.
assertSafeRedirectTarget <- function(fn, to) {
  if (!is.character(to) || length(to) != 1 || is.na(to)) {
    stop(
      sprintf(
        "%s(): `to` must be a single, non-NA character string.",
        fn
      ),
      call. = FALSE
    )
  }
  if (grepl("^\\s*(javascript|data|vbscript):", to, ignore.case = TRUE)) {
    stop(
      sprintf(
        "%s(): refusing unsafe URL scheme in `to` = %s. ",
        fn,
        deparse(to)
      ),
      call. = FALSE
    )
  }
  # Protocol-relative URLs (//host/...) inherit the page's scheme and send
  # the user off-origin. Safe by default: reject. Callers wanting a
  # cross-origin absolute URL can spell out the full https:// form.
  if (grepl("^\\s*//", to)) {
    stop(
      sprintf(
        "%s(): refusing protocol-relative URL in `to` = %s. ",
        fn,
        deparse(to)
      ),
      "Use a full https:// URL if you really want a cross-origin redirect.",
      call. = FALSE
    )
  }
}

#' redirect (loader/action helper)
#'
#' \url{https://reactrouter.com/api/utils/redirect}
#'
#' Returns a \code{\link{JS}} loader function that redirects to \code{to}.
#' Pass as the \code{loader} argument of a \code{\link{Route}} to perform
#' an unconditional redirect -- typically used for guard routes that always
#' send the user somewhere else.
#'
#' \strong{Security:} \code{to} must be a trusted, package-author-controlled
#' string. \code{javascript:}, \code{data:}, and \code{vbscript:} URL schemes
#' are rejected. If you build \code{to} from user-supplied input, validate it
#' yourself before passing it in -- never round-trip untrusted strings through
#' \code{redirect()} into a navigation.
#'
#' For conditional redirects inside a custom loader/action, call
#' \code{window.jsmodule['@/reactRouter'].helpers.redirect(to)} from your own
#' \code{JS()} string, e.g.
#' \preformatted{
#'   loader = JS(
#'     "async () => {
#'        const { redirect } = window.jsmodule['@/reactRouter'].helpers;
#'        if (!authed()) return redirect('/login'); ...
#'      }"
#'   )
#' }
#'
#' The \code{data}, \code{replace}, and \code{redirectDocument} helpers are
#' exposed on the same namespace.
#'
#' @param to Character. Destination path.
#' @return A \code{\link{JS}} expression suitable for the \code{loader}
#'   argument of \code{\link{Route}}.
#'
#' @examples
#' \dontrun{
#' Route(path = "/old", loader = redirect("/new"), element = NULL)
#' }
#'
#' @rdname redirect
#' @export
redirect <- function(to) {
  assertSafeRedirectTarget("redirect", to)
  shiny.react::JS(sprintf(
    '() => %s.redirect(%s)',
    HELPERS_JS,
    jsLiteral(to)
  ))
}

#' replaceResponse (loader/action helper)
#'
#' \url{https://reactrouter.com/api/utils/replace}
#'
#' Returns a \code{\link{JS}} loader function that performs a \emph{replace}
#' navigation to \code{to} -- same as \code{\link{redirect}}, but the new
#' entry replaces the current one in the history stack instead of pushing
#' a new one. Use for "alias" routes where the original URL should not
#' remain in the user's back-history.
#'
#' Renamed from \code{replace()} to avoid masking \code{base::replace}.
#' This mirrors the \code{dataResponse()} naming for the same reason.
#'
#' For conditional replacements inside a custom loader/action, call
#' \code{window.jsmodule['@/reactRouter'].helpers.replace(to)} from your own
#' \code{JS()} string.
#'
#' @param to Character. Destination path.
#' @return A \code{\link{JS}} expression suitable for the \code{loader}
#'   argument of \code{\link{Route}}.
#'
#' @examples
#' \dontrun{
#' Route(path = "/legacy", loader = replaceResponse("/new"), element = NULL)
#' }
#'
#' @rdname replaceResponse
#' @export
replaceResponse <- function(to) {
  assertSafeRedirectTarget("replaceResponse", to)
  shiny.react::JS(sprintf(
    '() => %s.replace(%s)',
    HELPERS_JS,
    jsLiteral(to)
  ))
}

#' redirectDocument (loader/action helper)
#'
#' \url{https://reactrouter.com/api/utils/redirectDocument}
#'
#' Returns a \code{\link{JS}} loader function that performs a \emph{document}
#' redirect to \code{to} -- a full page reload, as opposed to the client-side
#' navigation that \code{\link{redirect}} performs. Use when navigating to a
#' URL outside the router's control (e.g. a server-rendered page) so the
#' browser fully unloads the SPA.
#'
#' For conditional document redirects inside a custom loader/action, call
#' \code{window.jsmodule['@/reactRouter'].helpers.redirectDocument(to)} from
#' your own \code{JS()} string.
#'
#' @param to Character. Destination path or absolute URL.
#' @return A \code{\link{JS}} expression suitable for the \code{loader}
#'   argument of \code{\link{Route}}.
#'
#' @examples
#' \dontrun{
#' Route(
#'   path = "/docs",
#'   loader = redirectDocument("/static/docs/index.html"),
#'   element = NULL
#' )
#' }
#'
#' @rdname redirectDocument
#' @export
redirectDocument <- function(to) {
  assertSafeRedirectTarget("redirectDocument", to)
  shiny.react::JS(sprintf(
    '() => %s.redirectDocument(%s)',
    HELPERS_JS,
    jsLiteral(to)
  ))
}

#' dataResponse (loader/action helper)
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.data.html}
#'
#' Returns a \code{\link{JS}} loader function that resolves to a React Router
#' \code{data()} response -- a thin wrapper that lets you attach an HTTP
#' \code{status}, \code{statusText}, and/or \code{headers} alongside the
#' loader/action payload while still exposing \code{value} via
#' \code{\link{useLoaderData}} / \code{\link{useActionData}}.
#'
#' Use the R helper for static loaders that always return the same value plus
#' status. For values computed inside a custom loader/action, call
#' \code{window.jsmodule['@/reactRouter'].helpers.data(value, init)} directly
#' in your \code{JS()} string, e.g.
#' \preformatted{
#'   loader = JS("async () => {
#'     const { data } = window.jsmodule['@/reactRouter'].helpers;
#'     const rows = await fetchRows();
#'     return data(\{ rows \}, \{ status: 200 \});
#'   }")
#' }
#'
#' @param value The payload to expose via \code{useLoaderData()} /
#'   \code{useActionData()}. Either an R object (list, vector, data.frame --
#'   serialized to JSON), or a \code{\link{JS}} expression for a JavaScript
#'   value.
#' @param init Optional. Either a list with \code{status} (integer),
#'   \code{statusText} (character) and/or \code{headers} (named list), or a
#'   \code{\link{JS}} expression evaluating to such an object.
#' @return A \code{\link{JS}} expression suitable for the \code{loader} or
#'   \code{action} argument of \code{\link{Route}}.
#'
#' @examples
#' \dontrun{
#' Route(
#'   path = "/profile",
#'   loader = dataResponse(
#'     list(name = "Ada", role = "Engineer"),
#'     init = list(status = 200)
#'   ),
#'   element = useLoaderData(tags$pre())
#' )
#' }
#'
#' @name dataResponse
#' @export
dataResponse <- function(value, init = NULL) {
  if (missing(value)) {
    stop(
      "dataResponse(): `value` is required -- pass the payload that ",
      "useLoaderData()/useActionData() should expose. For an empty body, ",
      "pass NULL explicitly.",
      call. = FALSE
    )
  }
  serialize <- function(x) {
    if (inherits(x, "JS_EVAL")) {
      return(as.character(x))
    }
    jsonlite::toJSON(x, auto_unbox = TRUE, null = "null", na = "null")
  }
  # Surface common shape mistakes in `init` at call site, rather than as a
  # confusing browser-side error from React Router's data() helper.
  if (!is.null(init) && !inherits(init, "JS_EVAL")) {
    checkmate::assert_list(init, names = "named", .var.name = "init")
    allowed <- c("status", "statusText", "headers")
    bad <- setdiff(names(init), allowed)
    if (length(bad)) {
      stop(
        sprintf(
          "dataResponse(): unknown `init` field(s): %s. Allowed: %s.",
          paste(bad, collapse = ", "),
          paste(allowed, collapse = ", ")
        ),
        call. = FALSE
      )
    }
    if (!is.null(init$status)) {
      checkmate::assert_int(
        init$status,
        lower = 100,
        upper = 599,
        .var.name = "init$status"
      )
    }
    if (!is.null(init$statusText)) {
      checkmate::assert_string(init$statusText, .var.name = "init$statusText")
    }
    if (!is.null(init$headers)) {
      # Headers can be a named list/character vector; both serialize to a
      # plain JS object that the Headers constructor accepts.
      if (
        !(is.list(init$headers) || is.character(init$headers)) ||
          is.null(names(init$headers)) ||
          any(!nzchar(names(init$headers)))
      ) {
        stop(
          "dataResponse(): `init$headers` must be a fully named list or ",
          "character vector (e.g. list(`Content-Type` = \"application/json\")).",
          call. = FALSE
        )
      }
    }
  }
  valueJS <- serialize(value)
  initStr <- if (!is.null(init)) paste0(", ", serialize(init)) else ""
  shiny.react::JS(sprintf(
    "() => %s.data(%s%s)",
    HELPERS_JS,
    valueJS,
    initStr
  ))
}

#' isRouteErrorResponse
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.isRouteErrorResponse.html}
#'
#' Returns a \code{\link{JS}} reference to the \code{isRouteErrorResponse}
#' type guard. Use it inside an \code{errorElement} render callback to branch
#' on whether the error came from a thrown \code{Response}
#' (e.g. \code{throw new Response(..., \{ status: 404 \})}) or from arbitrary
#' code. Pair with \code{\link{useRouteError}}.
#'
#' Calling \code{isRouteErrorResponse()} from R returns a \code{\link{JS}}
#' expression that evaluates, in the browser, to the upstream
#' \code{isRouteErrorResponse} function. Interpolate it inside the
#' \code{render} string of \code{useRouteError()} as shown below.
#'
#' For convenience, the same function is also reachable inside any
#' user-authored \code{\link{JS}} string as
#' \code{window.jsmodule['@/reactRouter'].helpers.isRouteErrorResponse}.
#'
#' @return A \code{\link{JS}} expression evaluating to the
#'   \code{isRouteErrorResponse} function reference.
#'
#' @examples
#' \dontrun{
#' useRouteError(render = JS(paste0(
#'   "e => ", isRouteErrorResponse(),
#'   "(e) ? <p>HTTP {e.status}</p> : <p>Unknown error</p>"
#' )))
#' }
#'
#' @rdname isRouteErrorResponse
#' @export
isRouteErrorResponse <- function() {
  shiny.react::JS(paste0(HELPERS_JS, ".isRouteErrorResponse"))
}

# Pure-R unit tests for the loader/action helpers — no browser needed.

test_that("redirect helpers reject unsafe URL schemes", {
  for (fn in list(redirect, replaceResponse, redirectDocument)) {
    for (bad in c(
      "javascript:alert(1)",
      "JavaScript:alert(1)",       # case-insensitive
      "  javascript:alert(1)",     # leading whitespace tolerated by browsers
      "data:text/html,<script>1</script>",
      "vbscript:msgbox(1)"
    )) {
      expect_error(fn(bad), "refusing unsafe URL scheme", fixed = TRUE)
    }
  }
})

test_that("redirect helpers reject protocol-relative URLs", {
  for (fn in list(redirect, replaceResponse, redirectDocument)) {
    expect_error(fn("//evil.example.com/x"), "protocol-relative", fixed = TRUE)
    expect_error(fn(" //evil.example.com/x"), "protocol-relative", fixed = TRUE)
  }
})

test_that("redirect helpers reject non-character / NA / multi-element `to`", {
  for (fn in list(redirect, replaceResponse, redirectDocument)) {
    expect_error(fn(NA_character_), "must be a single, non-NA")
    expect_error(fn(c("/a", "/b")), "must be a single, non-NA")
    expect_error(fn(42), "must be a single, non-NA")
  }
})

test_that("redirect helpers accept safe targets and emit JS via the helpers namespace", {
  for (info in list(
    list(fn = redirect,         js = "redirect"),
    list(fn = replaceResponse,  js = "replace"),
    list(fn = redirectDocument, js = "redirectDocument")
  )) {
    out <- info$fn("/safe")
    expect_s3_class(out, "JS_EVAL")
    txt <- as.character(out)
    expect_match(txt, "window.jsmodule['@/reactRouter'].helpers", fixed = TRUE)
    expect_match(txt, paste0(".", info$js, "("), fixed = TRUE)
    expect_match(txt, '"/safe"', fixed = TRUE)
  }
})

test_that("dataResponse(value = NULL) round-trips as `null`", {
  out <- dataResponse(NULL)
  txt <- as.character(out)
  expect_s3_class(out, "JS_EVAL")
  # No init -> just `data(null)`, no second arg.
  expect_match(txt, ".data(null)", fixed = TRUE)
})

test_that("dataResponse() requires an explicit `value`", {
  expect_error(dataResponse(), "`value` is required", fixed = TRUE)
})

test_that("dataResponse() serialises R lists and forwards init", {
  out <- dataResponse(list(name = "Ada"), init = list(status = 201))
  txt <- as.character(out)
  expect_match(txt, '{"name":"Ada"}', fixed = TRUE)
  expect_match(txt, '{"status":201}', fixed = TRUE)
})

test_that("dataResponse() preserves a JS() value as raw JS", {
  out <- dataResponse(shiny.react::JS("makeRows()"))
  txt <- as.character(out)
  # The JS() is inlined verbatim, not JSON-stringified.
  expect_match(txt, "makeRows()", fixed = TRUE)
  expect_false(grepl('"makeRows', txt, fixed = TRUE))
})

test_that("isRouteErrorResponse() returns a JS() reference into the helpers namespace", {
  out <- isRouteErrorResponse()
  expect_s3_class(out, "JS_EVAL")
  expect_identical(
    as.character(out),
    "window.jsmodule['@/reactRouter'].helpers.isRouteErrorResponse"
  )
})

#' createBrowserRouter
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.createBrowserRouter.html}
#'
#' Creates a browser router using the data router API.
#' Use with \code{\link{createRoutesFromElements}} and \code{\link{Route}}.
#'
#' @rdname createBrowserRouter
#' @param ... \code{\link{Route}} elements. Pass directly, or (optionally)
#'   wrapped in \code{\link{createRoutesFromElements}} to mirror the
#'   official React Router v7 API.
#' @return A createBrowserRouter component.
#' @export
createBrowserRouter <- function(...) {
  tag <- shiny.react::reactElement(
    module = "@/reactRouter",
    name = "CreateBrowserRouter",
    props = shiny.react::asProps(...),
    deps = reactRouterDependency()
  )
  class(tag) <- c("reactRouter", class(tag))
  tag
}

#' createHashRouter
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.createHashRouter.html}
#'
#' Creates a hash router using the data router API.
#' Use with \code{\link{createRoutesFromElements}} and \code{\link{Route}}.
#'
#' @rdname createHashRouter
#' @param ... \code{\link{Route}} elements. Pass directly, or (optionally)
#'   wrapped in \code{\link{createRoutesFromElements}} to mirror the
#'   official React Router v7 API.
#' @return A createHashRouter component.
#' @export
createHashRouter <- function(...) {
  tag <- shiny.react::reactElement(
    module = "@/reactRouter",
    name = "CreateHashRouter",
    props = shiny.react::asProps(...),
    deps = reactRouterDependency()
  )
  class(tag) <- c("reactRouter", class(tag))
  tag
}

#' createRoutesFromElements
#'
#' \url{https://api.reactrouter.com/v7/variables/react-router.createRoutesFromElements.html}
#'
#' Optional compatibility alias. In R, \code{\link{createHashRouter}},
#' \code{\link{createBrowserRouter}}, and \code{\link{createMemoryRouter}}
#' accept \code{\link{Route}} elements directly, so wrapping them in
#' \code{createRoutesFromElements()} is not required. The function is kept
#' so that examples copied verbatim from the React Router v7 documentation
#' (\code{createHashRouter(createRoutesFromElements(...))}) keep working.
#'
#' The actual JSX-to-route-object conversion always happens on the
#' JavaScript side; this R function simply bundles its arguments into a
#' tag list.
#'
#' @rdname createRoutesFromElements
#' @param ... \code{\link{Route}} elements.
#' @return A tag list of Route elements.
#' @export
createRoutesFromElements <- function(...) {
  tag <- shiny::tagList(...)
  class(tag) <- c("reactRouter", class(tag))
  tag
}

#' createMemoryRouter
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.createMemoryRouter.html}
#'
#' Creates a memory router using the data router API. Routing state is kept
#' in memory and the browser URL is never read or modified, making it suitable
#' for static HTML pages (\code{file://}), Quarto documents, and embedded
#' widgets where the real URL is irrelevant.
#' Use with \code{\link{createRoutesFromElements}} and \code{\link{Route}}.
#'
#' @rdname createMemoryRouter
#' @param ... \code{\link{Route}} elements. Pass directly, or (optionally)
#'   wrapped in \code{\link{createRoutesFromElements}} to mirror the
#'   official React Router v7 API.
#' @return A createMemoryRouter component.
#' @export
createMemoryRouter <- function(...) {
  tag <- shiny.react::reactElement(
    module = "@/reactRouter",
    name = "CreateMemoryRouter",
    props = shiny.react::asProps(...),
    deps = reactRouterDependency()
  )
  class(tag) <- c("reactRouter", class(tag))
  tag
}

#' RouterProvider
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.RouterProvider.html}
#'
#' Renders a data router. Mirrors the React Router v7 composition pattern:
#' pass a router built with \code{\link{createHashRouter}},
#' \code{\link{createBrowserRouter}}, or \code{\link{createMemoryRouter}} to
#' the \code{router} argument.
#'
#' The underlying router is created once on mount and is not rebuilt when
#' \code{Route()} children change at runtime — React Router data routers own
#' their own navigation state and must be stable. To swap the route tree
#' dynamically (e.g. based on Shiny inputs), remount \code{RouterProvider}
#' itself, for instance by toggling its parent via \code{shiny::renderUI}.
#'
#' @rdname RouterProvider
#' @param router A router element produced by \code{\link{createHashRouter}},
#'   \code{\link{createBrowserRouter}}, or \code{\link{createMemoryRouter}}.
#' @param fallbackElement Element shown while the initial route's loader is
#'   resolving.
#' @return A RouterProvider component.
#' @export
RouterProvider <- function(router, fallbackElement = NULL) {
  tag <- shiny.react::reactElement(
    module = "@/reactRouter",
    name = "RouterProvider",
    props = shiny.react::asProps(
      router = router,
      fallbackElement = fallbackElement
    ),
    deps = reactRouterDependency()
  )
  class(tag) <- c("reactRouter", class(tag))
  tag
}

library(shiny)

ui <- reactRouter::RouterProvider(
  router = reactRouter::createHashRouter(
    reactRouter::Route(
      path = "/",
      element = div(reactRouter::Outlet()),
      reactRouter::Route(
        path = "products/:id",
        element = div(
          div(
            id = "hrefRelative",
            reactRouter::useHref(
              tags$span(),
              to = "../settings"
            )
          ),
          div(
            id = "hrefAbsolute",
            reactRouter::useHref(
              tags$span(),
              to = "/home"
            )
          ),
          div(
            id = "resolvedAll",
            reactRouter::useResolvedPath(
              tags$span(),
              to = "../settings"
            )
          ),
          div(
            id = "resolvedPathname",
            reactRouter::useResolvedPath(
              tags$span(),
              to = "../settings",
              selector = "pathname"
            )
          )
        )
      )
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

library(shiny)

ui <- reactRouter::RouterProvider(
  router = reactRouter::createHashRouter(
    reactRouter::Route(
      path = "/",
      element = div(
        div(
          id = "navState",
          reactRouter::useNavigation(
            tags$span(),
            selector = "state"
          )
        ),
        div(
          id = "navAll",
          reactRouter::useNavigation(
            tags$span()
          )
        ),
        reactRouter::Outlet()
      ),
      reactRouter::Route(
        index = TRUE,
        element = div(tags$p("home"))
      )
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

library(shiny)

# A fetcher-powered search: submit the Form to load filtered data
# without a full navigation. useFetcher() exposes state and data.
ui <- reactRouter::RouterProvider(
  router = reactRouter::createHashRouter(
    reactRouter::Route(
      path = "/",
      element = div(
        div(
          id = "fetcherState",
          reactRouter::useFetcher(
            tags$span(),
            selector = "state"
          )
        ),
        div(
          id = "fetcherData",
          reactRouter::useFetcher(
            tags$span(),
            selector = "data.result"
          )
        ),
        reactRouter::Form(
          id = "fetchForm",
          method = "get",
          action = "/search",
          tags$input(type = "hidden", name = "q", value = "hello"),
          tags$button(type = "submit", "Fetch")
        ),
        reactRouter::Outlet()
      ),
      reactRouter::Route(
        path = "search",
        loader = reactRouter::JS(
          "({ request }) => {
            const q = new URL(request.url).searchParams.get('q') || '';
            return { result: 'found:' + q };
          }"
        ),
        element = div()
      )
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

library(shiny)

# useBlocker renders "unblocked" by default (no navigation attempted).
# With shouldBlock = FALSE, navigation is never blocked.
ui <- reactRouter::RouterProvider(
  router = reactRouter::createHashRouter(
    reactRouter::Route(
      path = "/",
      element = div(
        div(
          id = "blockerState",
          reactRouter::useBlocker(
            tags$span(),
            selector = "state",
            shouldBlock = reactRouter::JS(
              "({ currentLocation, nextLocation }) =>
                currentLocation.pathname !== nextLocation.pathname"
            )
          )
        ),
        div(
          id = "blockerStateRender",
          reactRouter::useBlocker(
            render = reactRouter::JS("b => b.state"),
            shouldBlock = reactRouter::JS(
              "({ currentLocation, nextLocation }) =>
                currentLocation.pathname !== nextLocation.pathname"
            )
          )
        ),
        reactRouter::Outlet()
      ),
      reactRouter::Route(
        index = TRUE,
        element = div(tags$p("home"))
      )
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

library(shiny)

ui <- reactRouter::HashRouter(
  reactRouter::Routes(
    reactRouter::Route(
      path = "/",
      element = div(id = "homePage", tags$p("home"))
    ),
    reactRouter::Route(
      path = "old",
      element = reactRouter::Navigate(to = "/new", replace = TRUE)
    ),
    reactRouter::Route(
      path = "new",
      element = div(id = "newPage", tags$p("new page"))
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

library(shiny)

ui <- reactRouter::RouterProvider(
  router = reactRouter::createHashRouter(
    reactRouter::Route(
      path = "/",
      element = div(reactRouter::Outlet()),
      reactRouter::Route(
        path = "data",
        loader = reactRouter::JS(
          "async () => { return { name: 'Alice', age: 30 }; }"
        ),
        element = div(
          div(
            id = "loaderAll",
            reactRouter::useLoaderData(
              tags$span()
            )
          ),
          div(
            id = "loaderName",
            reactRouter::useLoaderData(
              tags$span(),
              selector = "name"
            )
          ),
          div(
            id = "loaderAge",
            reactRouter::useLoaderData(
              tags$span(),
              selector = "age"
            )
          ),
          div(
            id = "loaderNameRender",
            reactRouter::useLoaderData(
              render = reactRouter::JS("d => `${d.name}, age ${d.age}`")
            )
          )
        )
      )
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

library(shiny)

ui <- reactRouter::BrowserRouter(
  div(id = "browserRouterContent", tags$p("BrowserRouter active")),
  reactRouter::Routes(
    reactRouter::Route(
      path = "/*",
      element = div(id = "browserHome", tags$p("browser home"))
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

library(shiny)

# Parent passes context = list(user = "Alice") to Outlet.
# Child reads it with useOutletContext().
ui <- reactRouter::RouterProvider(
  router = reactRouter::createHashRouter(
    reactRouter::Route(
      path = "/",
      element = div(
        reactRouter::Outlet(context = list(user = "Alice", role = "admin"))
      ),
      reactRouter::Route(
        index = TRUE,
        element = div(
          div(
            id = "contextUser",
            reactRouter::useOutletContext(
              tags$span(),
              selector = "user"
            )
          ),
          div(
            id = "contextRole",
            reactRouter::useOutletContext(
              tags$span(),
              selector = "role"
            )
          ),
          div(
            id = "contextAll",
            reactRouter::useOutletContext(
              tags$span()
            )
          ),
          div(
            id = "contextRender",
            reactRouter::useOutletContext(
              render = reactRouter::JS("c => `${c.user} (${c.role})`")
            )
          )
        )
      )
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

library(shiny)

ui <- reactRouter::HashRouter(
  div(
    id = "allParams",
    reactRouter::useSearchParams(
      tags$span()
    )
  ),
  div(
    id = "colorParam",
    reactRouter::useSearchParams(
      tags$span(),
      param = "color"
    )
  ),
  reactRouter::Routes(
    reactRouter::Route(
      path = "/",
      element = div(tags$p("home"))
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

library(shiny)

ui <- reactRouter::RouterProvider(
  router = reactRouter::createHashRouter(
    reactRouter::Route(
      path = "/",
      element = div(reactRouter::Outlet()),
      reactRouter::Route(
        path = "broken",
        loader = reactRouter::JS(
          "async () => { throw new Error('test error message'); }"
        ),
        element = div(tags$p("should not render")),
        errorElement = div(
          div(
            id = "errorMsg",
            reactRouter::useRouteError(
              tags$span(),
              selector = "message"
            )
          )
        )
      )
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

#' Documentation template for components
#'
#' @param ... Props to pass to the component.
#' The allowed props are listed below in the \bold{Details} section.
#'
#' @return
#' Object with `shiny.tag` class suitable for use in the UI of a Shiny app.
#'
#' @keywords internal
#' @name component
NULL

component <- function(name, module = 'react-router-dom') {
  function(...) {
    shiny.react::reactElement(
      module = module,
      name = name,
      props = shiny.react::asProps(...),
      deps = reactRouterDependency()
    )
  }
}

#' HashRouter
#' @rdname HashRouter
#' @description \url{https://api.reactrouter.com/v7/functions/react-router.HashRouter.html}
#' @param ... Props to pass to element.
#' @return A HashRouter component.
#' @export
HashRouter <- function(...) {
  tag <- shiny.react::reactElement(
    module = 'react-router-dom',
    name = "HashRouter",
    props = shiny.react::asProps(...),
    deps = reactRouterDependency()
  )
  class(tag) <- c("reactRouter", class(tag))
  tag
}

#' BrowserRouter
#' @rdname BrowserRouter
#' @description \url{https://api.reactrouter.com/v7/functions/react-router.BrowserRouter.html}
#' @param ... Props to pass to element.
#' @return A BrowserRouter component.
#' @export
BrowserRouter <- function(...) {
  tag <- shiny.react::reactElement(
    module = 'react-router-dom',
    name = "BrowserRouter",
    props = shiny.react::asProps(...),
    deps = reactRouterDependency()
  )
  class(tag) <- c("reactRouter", class(tag))
  tag
}

#' MemoryRouter
#' @rdname MemoryRouter
#' @description \url{https://api.reactrouter.com/v7/functions/react-router.MemoryRouter.html}
#' @param ... Props to pass to element.
#' @return A MemoryRouter component.
#' @export
MemoryRouter <- function(...) {
  tag <- shiny.react::reactElement(
    module = 'react-router-dom',
    name = "MemoryRouter",
    props = shiny.react::asProps(...),
    deps = reactRouterDependency()
  )
  class(tag) <- c("reactRouter", class(tag))
  tag
}

#' Route
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.Route.html}
#'
#' Internally the `element` is wrapped in a `shiny::div()`
#' with a UUID key so, in case R shiny is used, shiny can differentiate
#' each element.
#'
#' Additional React Router \code{Route} props can be passed through \code{...}:
#' \itemize{
#'   \item \code{path} (Character): path pattern, supports \code{:param},
#'     optional \code{:param?}, and splat \code{*}.
#'   \item \code{index} (Boolean): mark this as the index route of its parent.
#'   \item \code{caseSensitive} (Boolean): match the path case-sensitively.
#'   \item \code{id} (Character): stable route id, required for use with
#'     \code{\link{useRouteLoaderData}}.
#'   \item \code{handle} (Any): arbitrary value exposed via
#'     \code{\link{useMatches}} for breadcrumbs and similar use cases.
#'   \item \code{shouldRevalidate} (\code{\link{JS}}): function controlling
#'     whether the loader re-runs on a given navigation.
#'   \item \code{lazy} (\code{\link{JS}}): code-splitting hook returning a
#'     \code{Promise} resolving to a route module.
#'   \item \code{hasErrorBoundary} (Boolean): explicit error-boundary flag
#'     (rarely needed when \code{errorElement} is provided).
#' }
#'
#' @rdname Route
#' @param ... Additional Route props (see Details).
#' @param element The element to render when the route matches. Wrapped on
#'   the JS side in a no-DOM \code{Keyed} component so React remounts the
#'   subtree on every route change -- this is required for Shiny output
#'   bindings (e.g. \code{textOutput()}) to reinitialise correctly when two
#'   routes render the same component shape with different namespaces.
#'   Unlike the previous \code{<div>} wrapper, \code{Keyed} adds no DOM node,
#'   so layouts like MUI \code{Grid} that require typed direct children keep
#'   working.
#' @param key Stable React key for the route's \code{element}. Defaults to
#'   a random alphanumeric string per \code{Route()} call, so each route's
#'   element has a distinct identity from its siblings -- this is what
#'   causes React to unmount the previous route's subtree (and reinitialise
#'   its Shiny output bindings) when navigating between routes that render
#'   the same component shape. You almost never need to pass this
#'   explicitly. \strong{Note:} this key only affects identity of the
#'   route's \code{element}; it does \emph{not} rebuild the
#'   \code{\link{RouterProvider}} data router. The route tree is created
#'   once on mount and subsequent \code{Route()} edits are ignored; to
#'   apply a new route tree at runtime, give \code{RouterProvider} itself
#'   a changing \code{key} (e.g. via \code{shiny::renderUI}).
#' @param loader Optional. A \code{\link{JS}} expression evaluating to a
#'   loader function, e.g. \code{JS("({ params }) => fetch(...)")}. For a
#'   plain unconditional redirect, use \code{\link{redirect}}. To embed
#'   static R data, serialize it first with \code{jsonlite::toJSON()} and
#'   wrap the result in \code{JS()}.
#' @param action Optional. A \code{\link{JS}} expression evaluating to an
#'   action function called by \code{\link{Form}} submissions and
#'   \code{\link{useSubmit}} / \code{\link{useFetcher}} submits.
#' @param errorElement Optional. Element rendered when the route's
#'   \code{loader}, \code{action}, or rendering throws.
#' @return A Route component.
#' @export
Route <- function(
  ...,
  element,
  loader = NULL,
  action = NULL,
  errorElement = NULL,
  key = randomKey()
) {
  # Mirrors the diagnostic quality of the hook wrappers: surface the most
  # common mistake (passing an R function instead of a JS() expression) at
  # call time, rather than as a confusing browser-side error later.
  if (!is.null(loader) && !inherits(loader, "JS_EVAL")) {
    stop(
      "Route(): `loader` must be a JS() expression, e.g. ",
      "loader = JS(\"({ params }) => fetch(...)\"). ",
      "For static R data, use jsonlite::toJSON() and wrap with JS(), or use redirect()/dataResponse().",
      call. = FALSE
    )
  }
  if (!is.null(action) && !inherits(action, "JS_EVAL")) {
    stop(
      "Route(): `action` must be a JS() expression, e.g. ",
      "action = JS(\"async ({ request }) => { ... }\").",
      call. = FALSE
    )
  }
  shiny.react::reactElement(
    module = "react-router-dom",
    name = "Route",
    props = shiny.react::asProps(
      ...,
      loader = loader,
      action = action,
      errorElement = errorElement,
      element = shiny.react::reactElement(
        module = "@/reactRouter",
        name = "Keyed",
        props = shiny.react::asProps(key = key, element),
        deps = reactRouterDependency()
      )
    ),
    deps = reactRouterDependency()
  )
}

#' Link
#'
#' \url{https://api.reactrouter.com/v7/variables/react-router.Link.html}
#'
#' The `reloadDocument` prop controls whether clicking the link uses React
#' Router's client-side navigation (`FALSE`, the default) or skips it and lets
#' the browser handle the click natively (`TRUE`). The default is correct for
#' almost every use, including Shiny apps with server-rendered output
#' (`uiOutput`, `renderUI`, `plotOutput`, htmlwidgets) — Shiny output bindings
#' re-attach automatically when React Router mounts the new route's element.
#' See `vignette("routers", package = "reactRouter")` for details.
#'
#' \strong{Two flavors.} Pick \code{Link()} for a plain navigation link
#' (the common case, mirroring React Router's API one-to-one). Pick
#' \code{Link.shinyInput()} only when you also need the click to fire a
#' Shiny input on the server — it adds an \code{inputId} that updates with
#' the link's \code{to} every time it is clicked, while still navigating.
#' If in doubt, use \code{Link()}.
#'
#' @rdname Link
#' @param ... Props to pass to element.
#' @param reloadDocument Boolean. Default `FALSE`. When `TRUE`, the click is
#'   handled natively by the browser instead of by React Router's client-side
#'   navigation. Rarely needed — leave at the default in most cases.
#' @return A Link component.
#' @export
Link <- function(..., reloadDocument = FALSE) {
  # Only forward reloadDocument when the caller actually supplied it, so
  # we don't override any future change to React Router's own default.
  props <- if (missing(reloadDocument)) {
    shiny.react::asProps(...)
  } else {
    shiny.react::asProps(..., reloadDocument = reloadDocument)
  }
  shiny.react::reactElement(
    module = "react-router-dom",
    name = "Link",
    props = props,
    deps = reactRouterDependency()
  )
}

#' Navigate
#' @rdname Navigate
#' @description \url{https://api.reactrouter.com/v7/functions/react-router.Navigate.html}
#' @param ... Props to pass to element.
#' @return A Navigate component.
#' @export
Navigate <- component('Navigate')

#' NavLink
#'
#' \url{https://api.reactrouter.com/v7/variables/react-router.NavLink.html}
#'
#' The `reloadDocument` prop controls whether clicking the link uses React
#' Router's client-side navigation (`FALSE`, the default) or skips it and lets
#' the browser handle the click natively (`TRUE`). The default is correct for
#' almost every use, including Shiny apps with server-rendered output
#' (`uiOutput`, `renderUI`, `plotOutput`, htmlwidgets) — Shiny output bindings
#' re-attach automatically when React Router mounts the new route's element.
#' See `vignette("routers", package = "reactRouter")` for details.
#'
#' @rdname NavLink
#' @param ... Props to pass to element.
#' @param reloadDocument Boolean. Default `FALSE`. When `TRUE`, the click is
#'   handled natively by the browser instead of by React Router's client-side
#'   navigation. Rarely needed — leave at the default in most cases.
#' @return A NavLink component.
#' @export
NavLink <- function(..., reloadDocument = FALSE) {
  props <- if (missing(reloadDocument)) {
    shiny.react::asProps(...)
  } else {
    shiny.react::asProps(..., reloadDocument = reloadDocument)
  }
  shiny.react::reactElement(
    module = "react-router-dom",
    name = "NavLink",
    props = props,
    deps = reactRouterDependency()
  )
}


#' Outlet
#' @rdname Outlet
#' @description \url{https://api.reactrouter.com/v7/functions/react-router.Outlet.html}
#' @param ... Props to pass to element.
#' @return A Outlet component.
#' @export
Outlet <- component('Outlet')

#' Routes
#' @rdname Routes
#' @description \url{https://api.reactrouter.com/v7/functions/react-router.Routes.html}
#' @param ... Props to pass to element.
#' @return A Routes component.
#' @export
Routes <- component('Routes')

#' Form
#' @rdname Form
#' @description \url{https://api.reactrouter.com/v7/variables/react-router.Form.html}
#' @param ... Props to pass to element.
#' @return A Form component.
#' @export
Form <- component('Form')

#' ScrollRestoration
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.ScrollRestoration.html}
#'
#' Emulates the browser's scroll restoration on location changes after loaders
#' have completed. Place once inside the root layout of a data router app.
#' Requires a data router (\code{\link{createBrowserRouter}},
#' \code{\link{createHashRouter}}, etc.).
#'
#' @rdname ScrollRestoration
#' @param ... Props to pass to element. Notable props: \code{getKey} (a
#'   \code{\link{JS}} function to compute the scroll key from the location)
#'   and \code{storageKey} (Character, custom \code{sessionStorage} key).
#' @return A ScrollRestoration component.
#' @export
ScrollRestoration <- component('ScrollRestoration')

# Sys.setenv(
#   CHROMOTE_CHROME = "/Program Files (x86)/Microsoft/Edge/Application/msedge.exe"
# )

test_that("routes with HashRouter() work", {
  # Don't run these tests on the CRAN build servers
  skip_on_cran()

  # set up new app driver object
  app <- shinytest2::AppDriver$new(app_dir = testthat::test_path("test-apps", "HashRouter"))
  app$wait_for_idle()

  app$get_js("window.location.hash = '#/'")
  app$wait_for_idle()
  values_home <- app$get_values()

  app$get_js("window.location.hash = '#/page'")
  app$wait_for_idle()
  values_page <- app$get_values()

  expect_identical(names(values_home$input), c("NavLinkHome", "NavLinkPage"))
  expect_true("outputHome" %in% names(values_home$output))
  expect_identical(as.character(values_home$output$outputHome$html), as.character(tags$p("home content")))
  expect_identical(names(values_page$input), c("NavLinkHome", "NavLinkPage"))
  expect_true("outputPage" %in% names(values_page$output))
  expect_identical(as.character(values_page$output$outputPage$html), as.character(tags$p("page content")))
})

library(shiny)

ui <- reactRouter::HashRouter(
  div(
    id = "locAll",
    reactRouter::useLocation(
      tags$span()
    )
  ),
  div(
    id = "locPathname",
    reactRouter::useLocation(
      tags$span(),
      selector = "pathname"
    )
  ),
  div(
    id = "locSearch",
    reactRouter::useLocation(
      tags$span(),
      selector = "search"
    )
  ),
  div(
    id = "locHash",
    reactRouter::useLocation(
      tags$span(),
      selector = "hash"
    )
  ),
  reactRouter::Routes(
    reactRouter::Route(
      path = "/*",
      element = div(tags$p("content"))
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

#' Link
#' @rdname Link
#' @description \url{https://api.reactrouter.com/v7/variables/react-router.Link.html}
#'
#' \strong{Repeat clicks.} The Shiny input value is the link's \code{to}
#' string. Clicking the same link twice publishes the same value, and Shiny
#' suppresses identical-value updates by default — your
#' \code{observeEvent(input$myLink, ...)} will fire only on the first click.
#' If you need to react to every click (e.g. logging, refreshing a panel),
#' bind to a counter or use \code{shiny::observeEvent(..., ignoreNULL = FALSE,
#' priority = "event")} alongside an explicit click counter, or wrap the
#' click target in a regular \code{shiny::actionButton()} that triggers the
#' navigation programmatically.
#'
#' @param ... Props to pass to element.
#' @param inputId ID of the component.
#' @param reloadDocument Boolean. Default FALSE. Let browser handle the transition normally
#' @param session For \code{updateLink.shinyInput()} / \code{updateNavLink.shinyInput()}
#'   only: the Shiny session object. Defaults to the current reactive domain.
#'   Not used by \code{Link.shinyInput()} / \code{NavLink.shinyInput()} themselves.
#' @export
Link.shinyInput <- function(inputId, ..., reloadDocument = FALSE) {
  checkmate::assert_string(inputId)
  checkmate::assert_flag(reloadDocument)

  shiny.react::reactElement(
    module = "@/reactRouter",
    name = "Link",
    props = shiny.react::asProps(
      inputId = inputId,
      ...,
      reloadDocument = reloadDocument
    ),
    deps = reactRouterDependency()
  )
}

#' @rdname Link
#' @export
updateLink.shinyInput <- shiny.react::updateReactInput

#' NavLink
#' @rdname NavLink
#' @description \url{https://api.reactrouter.com/v7/variables/react-router.NavLink.html}
#'
#' \strong{Repeat clicks.} The Shiny input value is the link's \code{to}
#' string; clicking the same link twice publishes the same value and Shiny
#' suppresses identical-value updates by default. See
#' \code{\link{Link.shinyInput}} for the workaround.
#'
#' @param ... Props to pass to element.
#' @param inputId ID of the component.
#' @param reloadDocument Boolean. Default FALSE Let browser handle the transition normally
#' @param session For \code{updateLink.shinyInput()} / \code{updateNavLink.shinyInput()}
#'   only: the Shiny session object. Defaults to the current reactive domain.
#'   Not used by \code{Link.shinyInput()} / \code{NavLink.shinyInput()} themselves.
#' @export
NavLink.shinyInput <- function(inputId, ..., reloadDocument = FALSE) {
  checkmate::assert_string(inputId)
  checkmate::assert_flag(reloadDocument)

  shiny.react::reactElement(
    module = "@/reactRouter",
    name = "NavLink",
    props = shiny.react::asProps(
      inputId = inputId,
      ...,
      reloadDocument = reloadDocument
    ),
    deps = reactRouterDependency()
  )
}

#' @rdname NavLink
#' @export
updateNavLink.shinyInput <- shiny.react::updateReactInput

# Internal: 16-char random alphanumeric string used as a React key on the
# Keyed wrapper of Route(element=). React keys only need to be unique among
# siblings, not RFC 4122 UUIDs — this avoids a dependency on the `uuid` pkg.
# The user's RNG state is preserved so reproducible tests aren't disturbed.
randomKey <- function() {
  if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
    old <- get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
    on.exit(assign(".Random.seed", old, envir = .GlobalEnv))
  } else {
    on.exit(suppressWarnings(rm(".Random.seed", envir = .GlobalEnv)))
  }
  paste0(sample(c(0:9, letters), 16, replace = TRUE), collapse = "")
}

#' react-router-dom JS dependency
#'
#' @return HTML dependency object.
#'
#' @export
reactRouterDependency <- function() {
  htmltools::htmlDependency(
    name = "reactRouter",
    version = getNamespaceVersion("reactRouter"),
    package = "reactRouter",
    src = c(file = "reactRouter"),
    script = "react-router-dom.js"
  )
}

library(shiny)

ui <- reactRouter::HashRouter(
  reactRouter::Link(to = "/about", id = "linkAbout", "Go to About"),
  reactRouter::Routes(
    reactRouter::Route(
      path = "/",
      element = div(id = "homePage", tags$p("home"))
    ),
    reactRouter::Route(
      path = "about",
      element = div(id = "aboutPage", tags$p("about content"))
    )
  )
)

server <- function(input, output, session) {}

shinyApp(ui, server)

#' @param into A component (HTML tag or \pkg{shiny.react}-based element)
#'   that will receive the hook data as the specified prop.
#' @param as Character. The name of the component's prop to inject the hook
#'   data into (by default \code{"children"} for text display, \code{"rows"} for
#'   a data grid, \code{"value"} for an input).
#' @param selector Character. Optional key to extract from the hook data object.
#'   If \code{NULL} (the default), the entire data is passed. Dotted paths
#'   like \code{"summary.title"} navigate nested objects.
#' @param render Optional \code{\link{JS}} function \code{(value) => ReactNode}
#'   used in place of \code{into}/\code{as}. Mirrors the native React Router
#'   pattern for cases where a single prop is not expressive enough (e.g.
#'   \code{JS("v => `${v.first} ${v.last}`")}).
#' @param ... Additional props to pass to the component.
#' @name hook-wrapper
#' @keywords internal
NULL

#' Documentation template for hook wrappers whose value has no internal
#' structure to walk (scalars, booleans, or functions). Identical to
#' \code{hook-wrapper} except that \code{selector} is intentionally
#' omitted -- it would have nothing to walk.
#'
#' @param into A component (HTML tag or \pkg{shiny.react}-based element)
#'   that will receive the hook value as the specified prop.
#' @param as Character. The name of the component's prop to inject the hook
#'   value into. Defaults to \code{"children"}.
#' @param render Optional \code{\link{JS}} function \code{(value) => ReactNode}
#'   used in place of \code{into}/\code{as}.
#' @param ... Additional props to pass to the component.
#' @name hook-wrapper-noselector
#' @keywords internal
NULL

# For hooks whose value is a function (useNavigate, useSubmit,
# useLinkClickHandler): rendering a function as React `children` triggers
# "functions are not valid as a React child". Catch the common mistake at
# call time with a message that points the user at the right escape hatch.
#
# `onClickSafe` controls whether `as = "onClick"` is suggested. It is only
# safe for `useLinkClickHandler`, whose returned function takes a single
# MouseEvent argument and is therefore a drop-in onClick handler. For
# `useNavigate(to, options?)` and `useSubmit(target, options?)` the function
# expects positional args other than a MouseEvent — wiring the click event
# straight to it would call `navigate(MouseEvent)` and try to navigate to
# the event object. For those hooks we recommend `render = JS(...)` only.
validateFunctionTarget <- function(label, into, render, as, onClickSafe = FALSE) {
  validateTarget(label, into, render)
  if (!is.null(into) && identical(as, "children")) {
    if (onClickSafe) {
      stop(
        sprintf(
          '%s(): the hook returns a function, so it cannot be injected as `children`. ',
          label
        ),
        'Either pass `render = JS("fn => <a onClick={fn}>...</a>")`, ',
        'or set `as = "onClick"` (and pass a click target via `into`).',
        call. = FALSE
      )
    }
    stop(
      sprintf(
        '%s(): the hook returns a function (signature unlike a click handler), ',
        label
      ),
      'so it cannot be injected as `children` and must not be wired directly to ',
      '`as = "onClick"` (the click event would be passed as the function\'s ',
      'first argument). Use `render = JS("fn => <button onClick={() => fn(...)}>...</button>")` ',
      'so you can call the function with the right arguments.',
      call. = FALSE
    )
  }
  # Even when `as` is set explicitly, "onClick" is unsafe for these hooks:
  # the click event would be passed as the function's first positional arg.
  if (!onClickSafe && !is.null(into) && identical(as, "onClick")) {
    stop(
      sprintf(
        '%s(): wiring the hook function directly to `as = "onClick"` is unsafe ',
        label
      ),
      'because the click event would be passed as the first argument (the hook ',
      'expects e.g. `(to, options)`, not a MouseEvent). Use ',
      '`render = JS("fn => <button onClick={() => fn(...)}>...</button>")` instead.',
      call. = FALSE
    )
  }
}

validateTarget <- function(label, into, render) {
  if (!is.null(render) && !inherits(render, "JS_EVAL")) {
    stop(
      sprintf(
        '%s(): `render` must be a JS() function, e.g. render = JS("v => v.name"). Got value of class "%s" -- did you forget to wrap it in JS()?',
        label,
        class(render)[1]
      ),
      call. = FALSE
    )
  }
  if (!is.null(render) && !is.null(into)) {
    stop(
      sprintf(
        '%s(): `render` and `into` are mutually exclusive -- provide one or the other, not both.',
        label
      ),
      call. = FALSE
    )
  }
  if (is.null(render) && is.null(into)) {
    stop(
      sprintf(
        '%s(): provide either `into` (a component to inject the value into) or `render` (a JS() function that returns a React element).',
        label
      ),
      call. = FALSE
    )
  }
}

# Internal: build a reactElement from the package's "@/reactRouter" JSX
# module and tag it with the `reactRouter` S3 class so `print.reactRouter`
# picks it up. Used by every hook wrapper that doesn't go through the
# generic `UseHook` dispatcher.
customHookElement <- function(name, ...) {
  tag <- shiny.react::reactElement(
    module = "@/reactRouter",
    name = name,
    props = shiny.react::asProps(...),
    deps = reactRouterDependency()
  )
  class(tag) <- c("reactRouter", class(tag))
  tag
}

#' Await
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.Await.html}
#'
#' Renders \code{into} when a deferred loader promise resolves, injecting the
#' resolved value (or a \code{selector} from it) \code{as} a prop.
#' Use inside a \code{\link{Route}} whose \code{loader} returns an object
#' containing a promise (written via \code{\link{JS}}). In React Router v7,
#' simply return the object directly -- no \code{defer()} wrapper is needed.
#'
#' @inheritParams hook-wrapper
#' @param resolveKey Character. The key in the loader's return value that holds
#'   the promise (e.g. if the loader returns \code{\{ data: promise \}},
#'   set \code{resolveKey = "data"}).
#' @param errorElement Element to render if the promise rejects.
#' @param fallback Element shown while the promise is pending. Defaults to a
#'   plain \code{"Loading\u2026"} span.
#'
#' @rdname Await
#' @export
Await <- function(
  into = NULL,
  as = "children",
  resolveKey,
  selector = NULL,
  render = NULL,
  errorElement = NULL,
  fallback = NULL,
  ...
) {
  if (missing(resolveKey)) {
    stop('Await(): `resolveKey` is required (the key in the loader\'s return value that holds the promise).', call. = FALSE)
  }
  # Await has two shapes:
  #
  #   1. Target mode  — `into` or `render` is given. The resolved value is
  #      injected like any other hook (delegates to validateTarget).
  #   2. Children mode — neither `into` nor `render`, but children are
  #      passed via `...`. Those children are rendered directly inside
  #      <Await> so descendants can call useAsyncValue() / useAsyncError()
  #      against the same promise (matches React Router's native pattern).
  #
  # Only target mode runs the validator — children mode legitimately omits
  # both `into` and `render`.
  dots <- list(...)
  # Children are unnamed dots; named dots are extra props (forwarded but
  # not treated as children). asProps follows the same convention.
  dotNames <- names(dots)
  hasChildren <- if (is.null(dotNames)) length(dots) > 0 else any(!nzchar(dotNames))
  if (hasChildren && (!is.null(into) || !is.null(render))) {
    stop(
      "Await(): pass either `into`/`render` (target mode) or children ",
      "(children mode for descendant useAsyncValue()/useAsyncError()), ",
      "not both -- with `into`/`render` set, the children would be ignored.",
      call. = FALSE
    )
  }
  if (!(is.null(into) && is.null(render) && hasChildren)) {
    validateTarget("Await", into, render)
  }
  customHookElement(
    "Await",
    as = as,
    into = into,
    resolveKey = resolveKey,
    selector = selector,
    render = render,
    errorElement = errorElement,
    fallback = fallback,
    ...
  )
}

# Internal: build a React element that dispatches through the generic
# `UseHook` JSX component. Every simple hook wrapper delegates here.
useHookElement <- function(
  hook,
  into = NULL,
  render = NULL,
  ...,
  mapArray = FALSE,
  nullIfFalsy = FALSE
) {
  validateTarget(hook, into, render)
  customHookElement(
    "UseHook",
    hook = hook,
    mapArray = mapArray,
    nullIfFalsy = nullIfFalsy,
    into = into,
    render = render,
    ...
  )
}

#' useLoaderData
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useLoaderData.html}
#'
#' Calls the \code{useLoaderData()} hook and injects the result (or a
#' \code{selector} from it) \code{as} a prop of the \code{into} component.
#' Use inside a \code{\link{Route}} that has a \code{loader}.
#'
#' @inheritParams hook-wrapper
#'
#' @examples
#' \dontrun{
#' # Display a selector as text
#' useLoaderData(tags$h3(), selector = "name")
#'
#' # Pass an array to a data grid
#' useLoaderData(
#'   muiDataGrid::DataGrid(columns = JS("[
#'     { field: 'name', headerName: 'Name', flex: 1 }
#'   ]")),
#'   as = "rows",
#'   selector = "people"
#' )
#' }
#'
#' @rdname useLoaderData
#' @export
useLoaderData <- function(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  ...
) {
  useHookElement(
    hook = "useLoaderData",
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useActionData
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useActionData.html}
#'
#' Calls the \code{useActionData()} hook and injects the result (or a
#' \code{selector} from it) \code{as} a prop of the \code{into} component.
#' Use inside a \code{\link{Route}} that has an \code{action}.
#'
#' @inheritParams hook-wrapper
#'
#' @rdname useActionData
#' @export
useActionData <- function(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  ...
) {
  useHookElement(
    hook = "useActionData",
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useLocation
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useLocation.html}
#'
#' Calls the \code{useLocation()} hook and injects the result (or a
#' \code{selector} from it) \code{as} a prop of the \code{into} component.
#' Available selectors: \code{pathname}, \code{search}, \code{hash},
#' \code{state}, \code{key}.
#'
#' @inheritParams hook-wrapper
#'
#' @rdname useLocation
#' @export
useLocation <- function(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  ...
) {
  useHookElement(
    hook = "useLocation",
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useParams
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useParams.html}
#'
#' Calls the \code{useParams()} hook and injects the result (or a
#' \code{selector} from it) \code{as} a prop of the \code{into} component.
#' Returns the dynamic parameters from the current URL matched by the
#' \code{\link{Route}} path pattern.
#'
#' @inheritParams hook-wrapper
#'
#' @rdname useParams
#' @export
useParams <- function(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  ...
) {
  useHookElement(
    hook = "useParams",
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useNavigation
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useNavigation.html}
#'
#' Calls the \code{useNavigation()} hook and injects the result (or a
#' \code{selector} from it) \code{as} a prop of the \code{into} component.
#' Returns the current navigation state: \code{"idle"}, \code{"loading"},
#' or \code{"submitting"}. Only works inside a data router.
#'
#' @inheritParams hook-wrapper
#'
#' @rdname useNavigation
#' @export
useNavigation <- function(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  ...
) {
  useHookElement(
    hook = "useNavigation",
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useRouteLoaderData
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useRouteLoaderData.html}
#'
#' Calls the \code{useRouteLoaderData()} hook and injects the result (or a
#' \code{selector} from it) \code{as} a prop of the \code{into} component.
#' Accesses loader data from any route by its \code{routeId}.
#' Only works inside a data router.
#'
#' @inheritParams hook-wrapper
#' @param routeId Character. The route ID to fetch loader data from.
#'
#' @rdname useRouteLoaderData
#' @export
useRouteLoaderData <- function(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  routeId,
  ...
) {
  if (missing(routeId)) {
    stop('useRouteLoaderData(): `routeId` is required (the `id` of the Route to read loader data from).', call. = FALSE)
  }
  useHookElement(
    hook = "useRouteLoaderData",
    hookArg = routeId,
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useRouteError
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useRouteError.html}
#'
#' Calls the \code{useRouteError()} hook and injects the result (or a
#' \code{selector} from it) \code{as} a prop of the \code{into} component.
#' Use inside the \code{errorElement} of a \code{\link{Route}}.
#'
#' @inheritParams hook-wrapper
#'
#' @rdname useRouteError
#' @export
useRouteError <- function(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  ...
) {
  useHookElement(
    hook = "useRouteError",
    nullIfFalsy = TRUE,
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useNavigationType
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useNavigationType.html}
#'
#' Calls the \code{useNavigationType()} hook and injects the result
#' \code{as} a prop of the \code{into} component.
#' Returns one of \code{"POP"}, \code{"PUSH"}, or \code{"REPLACE"}.
#'
#' @inheritParams hook-wrapper-noselector
#'
#' @rdname useNavigationType
#' @export
useNavigationType <- function(
  into = NULL,
  as = "children",
  # No `selector` arg: the upstream hook returns a scalar string, so there
  # is nothing to walk. If `selector` is ever added, also drop `mapArray =
  # FALSE` semantics from injectValue's path for this hook.
  render = NULL,
  ...
) {
  useHookElement(
    hook = "useNavigationType",
    into = into,
    as = as,
    render = render,
    ...
  )
}

#' useMatch
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useMatch.html}
#'
#' Calls the \code{useMatch()} hook and injects the result (or a
#' \code{selector} from it) \code{as} a prop of the \code{into} component.
#' Returns \code{NULL} if no match.
#'
#' @inheritParams hook-wrapper
#' @param pattern Character. The path pattern to match against
#'   (e.g. \code{"/products/:id"}).
#'
#' @rdname useMatch
#' @export
useMatch <- function(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  pattern,
  ...
) {
  if (missing(pattern)) {
    stop('useMatch(): `pattern` is required (a path pattern such as "/products/:id").', call. = FALSE)
  }
  useHookElement(
    hook = "useMatch",
    hookArg = pattern,
    nullIfFalsy = TRUE,
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useMatches
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useMatches.html}
#'
#' Calls the \code{useMatches()} hook and injects the result (or a
#' \code{selector} extracted from each match) \code{as} a prop of the
#' \code{into} component. Only works inside a data router.
#'
#' @inheritParams hook-wrapper
#'
#' @rdname useMatches
#' @export
useMatches <- function(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  ...
) {
  useHookElement(
    hook = "useMatches",
    mapArray = TRUE,
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useSearchParams
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useSearchParams.html}
#'
#' Calls the \code{useSearchParams()} hook and injects the result
#' \code{as} a prop of the \code{into} component. Use the \code{param}
#' argument to extract a query parameter by name.
#'
#' Values are always returned as character vectors so that repeated keys
#' (e.g. \code{"?tag=a&tag=b"}) are preserved. When injected as
#' \code{"children"}, vectors are joined with \code{", "}; for custom
#' formatting, use \code{render}.
#'
#' \strong{Reading vs. writing.} The upstream JS hook returns a tuple
#' \code{[searchParams, setSearchParams]}. This wrapper splits the two paths:
#' \itemize{
#'   \item \code{into} / \code{as} — \emph{read-only}. Receives the parsed
#'     params (or one \code{param}) and ignores the setter.
#'   \item \code{render} — receives both as \code{(params, setSearchParams)},
#'     so use this form when you need to update the URL programmatically:
#'     \preformatted{
#'   useSearchParams(render = JS(
#'     "(p, set) => <button onClick={() => set({tag:'b'})}>Filter</button>"
#'   ))
#'     }
#' }
#'
#' @inheritParams hook-wrapper
#' @param param Character. Name of a single query parameter to extract.
#'   Returns a character vector of all values for that key (length 0 if
#'   absent, length 1+ otherwise). When \code{NULL}, returns a named list
#'   mapping each key to its vector of values.
#'
#'   \strong{Empty vs. missing.} A missing key produces a length-0 vector,
#'   which renders as an empty string when injected with the default
#'   \code{as = "children"}. If you need to distinguish "absent" from
#'   "present-but-empty" (e.g. show a placeholder), use the \code{render}
#'   form and branch on \code{Array.isArray(v) && v.length === 0}, e.g.
#'   \code{render = JS("v => v.length ? v.join(', ') : <em>none</em>")}.
#'
#' @rdname useSearchParams
#' @export
useSearchParams <- function(
  into = NULL,
  as = "children",
  param = NULL,
  render = NULL,
  ...
) {
  validateTarget("useSearchParams", into, render)
  customHookElement(
    "useSearchParams",
    as = as,
    into = into,
    param = param,
    render = render,
    ...
  )
}

#' useHref
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useHref.html}
#'
#' Calls the \code{useHref()} hook and injects the resolved href string
#' \code{as} a prop of the \code{into} component.
#'
#' @inheritParams hook-wrapper-noselector
#' @param to Character. The path to resolve.
#'
#' @rdname useHref
#' @export
useHref <- function(into = NULL, as = "children", to, render = NULL, ...) {
  if (missing(to)) {
    stop('useHref(): `to` is required (the path to resolve).', call. = FALSE)
  }
  useHookElement(
    hook = "useHref",
    hookArg = to,
    into = into,
    as = as,
    render = render,
    ...
  )
}

#' useResolvedPath
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useResolvedPath.html}
#'
#' Calls the \code{useResolvedPath()} hook and injects the result (or a
#' \code{selector} from it) \code{as} a prop of the \code{into} component.
#' Returns \code{pathname}, \code{search}, and \code{hash}.
#'
#' @inheritParams hook-wrapper
#' @param to Character. The path to resolve.
#'
#' @rdname useResolvedPath
#' @export
useResolvedPath <- function(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  to,
  ...
) {
  if (missing(to)) {
    stop('useResolvedPath(): `to` is required (the path to resolve).', call. = FALSE)
  }
  useHookElement(
    hook = "useResolvedPath",
    hookArg = to,
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useFetcher
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useFetcher.html}
#'
#' Calls the \code{useFetcher()} hook and injects the result (or a
#' \code{selector} from it) \code{as} a prop of the \code{into} component.
#' Use to fetch data or submit forms without causing a navigation.
#' The fetcher object has \code{state} (\code{"idle"}/\code{"loading"}/\code{"submitting"}) and
#' \code{data} (the response from a loader or action).
#'
#' @inheritParams hook-wrapper
#' @param fetcherKey Character. Optional key to share a fetcher across
#'   components (e.g. \code{"my-fetcher"}).
#'
#' @details
#' \code{selector} defaults to \code{"state"} so the default \code{into}/
#' \code{children} display shows a readable string (\code{"idle"} /
#' \code{"loading"} / \code{"submitting"}). The full fetcher object contains
#' methods (\code{submit}, \code{load}, \code{Form}) that would be silently
#' dropped by JSON serialization if the whole object were rendered as
#' children. To call those methods, use the \code{render = JS(...)} form,
#' which receives the full fetcher: \code{render = JS("f => <button onClick={() => f.load('/data')}>Reload</button>")}.
#'
#' @rdname useFetcher
#' @export
useFetcher <- function(
  into = NULL,
  as = "children",
  selector = "state",
  render = NULL,
  fetcherKey = NULL,
  ...
) {
  validateTarget("useFetcher", into, render)
  # Routed through `customHookElement` (the @/reactRouter JSX shim) rather
  # than the generic `useHookElement`/UseHook dispatcher because upstream
  # `useFetcher` takes an options object `{ key }`, not a positional arg,
  # and `{ key: undefined }` is not equivalent to passing nothing -- the
  # generic single-positional `hookArg` cannot express that.
  customHookElement(
    "useFetcher",
    as = as,
    into = into,
    selector = selector,
    render = render,
    fetcherKey = fetcherKey,
    ...
  )
}


#' useFetchers
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useFetchers.html}
#'
#' Calls the \code{useFetchers()} hook and injects the result (or a
#' \code{selector} mapped over each fetcher) \code{as} a prop of the
#' \code{into} component. Returns an array of all active fetchers.
#' Useful for showing a global loading indicator.
#'
#' @inheritParams hook-wrapper
#'
#' @rdname useFetchers
#' @export
useFetchers <- function(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  ...
) {
  useHookElement(
    hook = "useFetchers",
    mapArray = TRUE,
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useRevalidator
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useRevalidator.html}
#'
#' Calls the \code{useRevalidator()} hook and injects the result (or a
#' \code{selector} from it) \code{as} a prop of the \code{into} component.
#' Returns the revalidation \code{state} (\code{"idle"} or \code{"loading"}).
#' Useful for showing loading feedback during manual or polling revalidation.
#'
#' @inheritParams hook-wrapper
#'
#' @rdname useRevalidator
#' @export
useRevalidator <- function(
  into = NULL,
  as = "children",
  selector = "state",
  render = NULL,
  ...
) {
  useHookElement(
    hook = "useRevalidator",
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useBlocker
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useBlocker.html}
#'
#' Calls the \code{useBlocker()} hook and injects the blocker's \code{state}
#' (or another \code{selector} field) \code{as} a prop of the \code{into}
#' component. Use to intercept navigation -- e.g. warn the user about unsaved
#' changes before they leave a route.
#'
#' The blocker \code{state} is one of \code{"unblocked"} (default),
#' \code{"blocked"} (navigation intercepted), or \code{"proceeding"}
#' (user confirmed, navigation in progress).
#'
#' @inheritParams hook-wrapper
#' @param shouldBlock Either \code{FALSE} (the default, disables blocking)
#'   or a \code{\link{JS}} function receiving
#'   \code{\{ currentLocation, nextLocation, historyAction \}} and returning
#'   \code{true} to block navigation or \code{false} to allow it.
#'   \strong{Must be a JS() expression}, not an R function -- R functions
#'   cannot be invoked from inside React Router's blocker callback.
#'
#' @rdname useBlocker
#' @export
useBlocker <- function(
  into = NULL,
  as = "children",
  selector = "state",
  render = NULL,
  shouldBlock = FALSE,
  ...
) {
  useHookElement(
    hook = "useBlocker",
    hookArg = shouldBlock,
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useNavigate
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useNavigate.html}
#'
#' Calls the \code{useNavigate()} hook and passes the navigate function
#' to \code{render} (or injects it \code{as} a prop of \code{into}).
#' The navigate function has signature \code{navigate(to, options?)}, e.g.
#' \code{navigate("/about")} or \code{navigate(-1)} to go back.
#'
#' Because the hook returns a function (not a value), the \code{render}
#' form is the natural way to use it:
#' \preformatted{
#'   useNavigate(render = JS(
#'     "nav => <button onClick={() => nav('/about')}>Go</button>"
#'   ))
#' }
#'
#' @inheritParams hook-wrapper-noselector
#'
#' @rdname useNavigate
#' @export
useNavigate <- function(
  into = NULL,
  as = "children",
  render = NULL,
  ...
) {
  validateFunctionTarget("useNavigate", into, render, as)
  useHookElement(
    hook = "useNavigate",
    into = into,
    as = as,
    render = render,
    ...
  )
}

#' useSubmit
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useSubmit.html}
#'
#' Calls the \code{useSubmit()} hook and passes the submit function to
#' \code{render} (or injects it \code{as} a prop of \code{into}).
#' The submit function has signature \code{submit(target, options?)} and
#' triggers a form submission (including calling the route's \code{action})
#' without requiring a \code{\link{Form}} element.
#' Only works inside a data router.
#'
#' Because the hook returns a function (not a value), the \code{into} form
#' is rarely useful here -- prefer \code{render = JS(...)} so you can call
#' the submit function from inside the rendered element.
#'
#' @inheritParams hook-wrapper-noselector
#'
#' @examples
#' \dontrun{
#' useSubmit(render = JS(
#'   "submit => <button onClick={() =>
#'      submit({ intent: 'delete' }, { method: 'post' })
#'    }>Delete</button>"
#' ))
#' }
#'
#' @rdname useSubmit
#' @export
useSubmit <- function(
  into = NULL,
  as = "children",
  render = NULL,
  ...
) {
  validateFunctionTarget("useSubmit", into, render, as)
  useHookElement(
    hook = "useSubmit",
    into = into,
    as = as,
    render = render,
    ...
  )
}

#' useAsyncValue
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useAsyncValue.html}
#'
#' Calls the \code{useAsyncValue()} hook and injects the resolved value (or a
#' \code{selector} from it) \code{as} a prop of the \code{into} component.
#' Must be rendered inside an \code{\link{Await}} that has been called in
#' \emph{children mode} (no \code{into} / \code{render} on \code{Await}) -- the
#' hook reads the value resolved by the closest \code{<Await>} ancestor.
#'
#' @inheritParams hook-wrapper
#'
#' @rdname useAsyncValue
#' @export
useAsyncValue <- function(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  ...
) {
  useHookElement(
    hook = "useAsyncValue",
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useAsyncError
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useAsyncError.html}
#'
#' Calls the \code{useAsyncError()} hook and injects the rejection reason
#' (or a \code{selector} from it) \code{as} a prop of the \code{into}
#' component. Must be rendered inside the \code{errorElement} of an
#' \code{\link{Await}} so the hook can pick up the rejected promise's error.
#'
#' @inheritParams hook-wrapper
#'
#' @rdname useAsyncError
#' @export
useAsyncError <- function(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  ...
) {
  useHookElement(
    hook = "useAsyncError",
    nullIfFalsy = TRUE,
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

#' useRoutes
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useRoutes.html}
#'
#' Builds a route tree from \code{\link{Route}} children (or a plain object
#' \code{routes} array) and renders the matched route. The hook-based
#' equivalent of \code{\link{Routes}} / \code{createRoutesFromElements} for
#' code that prefers a configuration-as-data style. Must be called inside a
#' router (\code{\link{RouterProvider}}, \code{\link{HashRouter}}, etc.).
#'
#' @param ... \code{\link{Route}} elements describing the route tree.
#'   Ignored if \code{routes} is supplied.
#' @param routes Optional. A \code{\link{JS}} expression evaluating to a plain
#'   JavaScript array of route objects (e.g. \code{JS("[\{ path: '/', element: ... \}]")}),
#'   used in place of \code{Route()} children.
#'
#' @rdname useRoutes
#' @export
useRoutes <- function(..., routes = NULL) {
  customHookElement("UseRoutes", ..., routes = routes)
}

#' useInRouterContext
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useInRouterContext.html}
#'
#' Calls the \code{useInRouterContext()} hook and injects the boolean result
#' \code{as} a prop of the \code{into} component. Useful inside reusable
#' components that may be rendered with or without a surrounding router --
#' guard router-only logic with this check before calling other hooks.
#'
#' @inheritParams hook-wrapper-noselector
#'
#' @rdname useInRouterContext
#' @export
useInRouterContext <- function(
  into = NULL,
  as = "children",
  render = NULL,
  ...
) {
  useHookElement(
    hook = "useInRouterContext",
    into = into,
    as = as,
    render = render,
    ...
  )
}

#' useOutlet
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useOutlet.html}
#'
#' Calls the \code{useOutlet()} hook and injects the matched child route's
#' element \code{as} a prop of the \code{into} component (or passes it to
#' \code{render}). Returns \code{NULL} when no child route matches -- useful for
#' rendering a fallback inside a layout when the user is on the parent route.
#'
#' Differs from the \code{\link{Outlet}} component in that it returns the
#' element as a value, so you can branch on whether a child route is matched.
#'
#' @inheritParams hook-wrapper-noselector
#' @param context Optional value to expose to descendants via
#'   \code{\link{useOutletContext}}.
#'
#' @examples
#' \dontrun{
#' # In a layout route: render the matched child, or a fallback if on the
#' # parent route itself.
#' useOutlet(
#'   render = JS("o => o || <p>Pick a section from the menu.</p>")
#' )
#'
#' # Inject the matched outlet element as the body of a wrapping <section>.
#' useOutlet(into = shiny::tags$section(class = "page"))
#' }
#'
#' @rdname useOutlet
#' @export
useOutlet <- function(
  into = NULL,
  as = "children",
  render = NULL,
  context = NULL,
  ...
) {
  useHookElement(
    hook = "useOutlet",
    hookArg = context,
    nullIfFalsy = TRUE,
    into = into,
    as = as,
    render = render,
    ...
  )
}

#' useViewTransitionState
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useViewTransitionState.html}
#'
#' Calls the \code{useViewTransitionState()} hook and injects the boolean
#' result \code{as} a prop of the \code{into} component. Returns \code{TRUE}
#' while a View Transitions API navigation toward \code{to} is in progress.
#' Pair with the \code{viewTransition} prop on \code{\link{Link}}/\code{\link{NavLink}}
#' to drive transition-aware styling.
#'
#' @inheritParams hook-wrapper-noselector
#' @param to Character. The destination path being transitioned to.
#' @param relative Optional character. Either \code{"route"} (default) or
#'   \code{"path"}.
#'
#' @rdname useViewTransitionState
#' @export
useViewTransitionState <- function(
  into = NULL,
  as = "children",
  render = NULL,
  to,
  relative = NULL,
  ...
) {
  if (missing(to)) {
    stop('useViewTransitionState(): `to` is required (the destination path being transitioned to).', call. = FALSE)
  }
  validateTarget("useViewTransitionState", into, render)
  customHookElement(
    "useViewTransitionState",
    as = as,
    into = into,
    render = render,
    to = to,
    relative = relative,
    ...
  )
}

#' useLinkClickHandler
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useLinkClickHandler.html}
#'
#' Calls the \code{useLinkClickHandler()} hook and exposes the returned click
#' handler function via \code{render} (or injects it \code{as} a prop of
#' \code{into}, e.g. \code{as = "onClick"}). Lets you build link-like
#' components that drive client-side navigation without using
#' \code{\link{Link}}.
#'
#' Because the hook returns a function, the \code{render} form is the natural
#' fit:
#' \preformatted{
#'   useLinkClickHandler(
#'     to = "/about",
#'     render = JS("h => <span onClick={h} role='link'>About</span>")
#'   )
#' }
#'
#' @inheritParams hook-wrapper-noselector
#' @param to Character. Destination path.
#' @param replace Optional boolean. Replace the current entry in the history
#'   stack instead of pushing a new one.
#' @param state Optional. State value to attach to the new location.
#' @param target Optional character. Anchor target (e.g. \code{"_blank"}).
#' @param preventScrollReset Optional boolean. If \code{TRUE}, do not reset
#'   scroll position on navigation.
#' @param relative Optional character. Either \code{"route"} (default) or
#'   \code{"path"}.
#'
#' @rdname useLinkClickHandler
#' @export
useLinkClickHandler <- function(
  into = NULL,
  as = "children",
  render = NULL,
  to,
  replace = NULL,
  state = NULL,
  target = NULL,
  preventScrollReset = NULL,
  relative = NULL,
  ...
) {
  if (missing(to)) {
    stop('useLinkClickHandler(): `to` is required (the destination path).', call. = FALSE)
  }
  validateFunctionTarget("useLinkClickHandler", into, render, as, onClickSafe = TRUE)
  customHookElement(
    "useLinkClickHandler",
    as = as,
    into = into,
    render = render,
    to = to,
    replace = replace,
    state = state,
    target = target,
    preventScrollReset = preventScrollReset,
    relative = relative,
    ...
  )
}

#' useOutletContext
#'
#' \url{https://api.reactrouter.com/v7/functions/react-router.useOutletContext.html}
#'
#' Calls the \code{useOutletContext()} hook and injects the context value
#' (or a \code{selector} from it) \code{as} a prop of the \code{into}
#' component. The context is whatever was passed to the parent route's
#' \code{Outlet(context = ...)} call.
#'
#' @inheritParams hook-wrapper
#'
#' @rdname useOutletContext
#' @export
useOutletContext <- function(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  ...
) {
  useHookElement(
    hook = "useOutletContext",
    into = into,
    as = as,
    selector = selector,
    render = render,
    ...
  )
}

test_that("useRouteError() renders error message", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseRouteError")
  )
  app$wait_for_idle()

  app$get_js("window.location.hash = '#/broken'")
  app$wait_for_idle()

  error_text <- app$get_text("#errorMsg")
  expect_equal(error_text, "test error message")
})

test_that("useLocation() renders location properties", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseLocation")
  )
  app$wait_for_idle()

  app$get_js("window.location.hash = '#/about?q=test'")
  app$wait_for_idle()

  pathname_text <- app$get_text("#locPathname")
  expect_equal(pathname_text, "/about")

  search_text <- app$get_text("#locSearch")
  expect_equal(search_text, "?q=test")

  hash_text <- app$get_text("#locHash")
  expect_equal(hash_text, "")

  all_text <- app$get_text("#locAll")
  expect_true(grepl('"pathname"', all_text))
  expect_true(grepl('"/about"', all_text))
})

test_that("useNavigation() renders navigation state", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseNavigation")
  )
  app$wait_for_idle()

  nav_state <- app$get_text("#navState")
  expect_equal(nav_state, "idle")

  nav_all <- app$get_text("#navAll")
  expect_true(grepl('"state"', nav_all))
  expect_true(grepl('"idle"', nav_all))
})

test_that("useSearchParams() renders query parameters", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseSearchParams")
  )
  app$wait_for_idle()

  app$get_js("window.location.hash = '#/?color=red'")
  app$wait_for_idle()

  color_text <- app$get_text("#colorParam")
  expect_equal(color_text, "red")

  all_text <- app$get_text("#allParams")
  expect_true(grepl('"color"', all_text))
  expect_true(grepl('"red"', all_text))
})

test_that("Await resolves deferred loader data (selector and render paths)", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "Await")
  )
  app$wait_for_idle()

  app$get_js("window.location.hash = '#/report'")
  app$wait_for_idle()

  # Fast (non-deferred) loader value should already be visible.
  expect_equal(app$get_text("#title"), "Q1")

  # The deferred promise resolves after ~250 ms; wait_for_idle should cover
  # that, but give Suspense a moment to commit the resolved tree.
  app$wait_for_idle(duration = 500)

  expect_equal(app$get_text("#revenue"), "$1.2M")
  expect_equal(app$get_text("#usersRender"), "18430 users")
})

test_that("Await(): combining into/render with children is rejected", {
  # Name every formal arg so the trailing positional lands in `...` (children),
  # not in `as`/`selector`/etc.
  expect_error(
    reactRouter::Await(
      into = htmltools::tags$span(),
      as = "children",
      resolveKey = "x",
      selector = NULL,
      render = NULL,
      errorElement = NULL,
      fallback = NULL,
      htmltools::tags$div("child")
    ),
    "not both",
    fixed = TRUE
  )
})

test_that("Await(): missing resolveKey is reported clearly", {
  expect_error(
    reactRouter::Await(htmltools::tags$span()),
    "`resolveKey` is required",
    fixed = TRUE
  )
})

test_that("useMatches() renders route matches array", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseMatchesActionData")
  )
  app$wait_for_idle()

  matches_text <- app$get_text("#matchesAll")
  expect_true(grepl("root", matches_text))
  expect_true(grepl('"pathname"', matches_text))
})

test_that("useActionData() is null before form submission", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseMatchesActionData")
  )
  app$wait_for_idle()

  app$get_js("window.location.hash = '#/form'")
  app$wait_for_idle()

  action_text <- app$get_text("#actionData")
  expect_equal(action_text, "")
})

test_that("Form submission populates useActionData()", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseMatchesActionData")
  )
  app$wait_for_idle()

  app$get_js("window.location.hash = '#/form'")
  app$wait_for_idle()

  app$click(selector = "#submitBtn")
  app$wait_for_idle()

  action_text <- app$get_text("#actionData")
  expect_true(grepl('"submitted"', action_text))
  expect_true(grepl('"Bob"', action_text))

  action_field <- app$get_text("#actionField")
  expect_equal(action_field, "Bob")
})

test_that("useLoaderData() renders loader data", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseLoaderData")
  )
  app$wait_for_idle()

  app$get_js("window.location.hash = '#/data'")
  app$wait_for_idle()

  loader_all <- app$get_text("#loaderAll")
  expect_true(grepl('"name"', loader_all))
  expect_true(grepl('"Alice"', loader_all))

  loader_name <- app$get_text("#loaderName")
  expect_equal(loader_name, "Alice")

  loader_age <- app$get_text("#loaderAge")
  expect_equal(loader_age, "30")

  # render = JS(...) path: receives full loader object, composes a string
  loader_name_render <- app$get_text("#loaderNameRender")
  expect_equal(loader_name_render, "Alice, age 30")
})

test_that("useOutletContext() reads context passed through Outlet", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseOutletContext")
  )
  app$wait_for_idle()

  context_user <- app$get_text("#contextUser")
  expect_equal(context_user, "Alice")

  context_role <- app$get_text("#contextRole")
  expect_equal(context_role, "admin")

  context_all <- app$get_text("#contextAll")
  expect_true(grepl('"user"', context_all))
  expect_true(grepl('"Alice"', context_all))

  # render = JS(...) path: receives full context object, composes a string
  context_render <- app$get_text("#contextRender")
  expect_equal(context_render, "Alice (admin)")
})

test_that("useFetcher() renders fetcher state", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseFetcher")
  )
  app$wait_for_idle()

  fetcher_state <- app$get_text("#fetcherState")
  expect_equal(fetcher_state, "idle")
})

test_that("useParams() renders route parameters", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseParams")
  )
  app$wait_for_idle()

  app$get_js("window.location.hash = '#/user/42'")
  app$wait_for_idle()

  param_text <- app$get_text("#paramId")
  expect_equal(param_text, "42")

  param_json <- app$get_text("#paramAll")
  expect_true(grepl('"id"', param_json))
  expect_true(grepl('"42"', param_json))
})

# Pure-R unit tests for the call-time validation introduced for the
# function-returning hooks and the missing()-arg guards.

test_that("function-returning hooks reject `into` with as = 'children'", {
  for (info in list(
    list(fn = useNavigate,         label = "useNavigate"),
    list(fn = useSubmit,           label = "useSubmit")
  )) {
    expect_error(
      info$fn(into = htmltools::tags$button("Go")),
      "cannot be injected as `children`",
      fixed = TRUE
    )
  }

  expect_error(
    useLinkClickHandler(to = "/x", into = htmltools::tags$span()),
    "cannot be injected as `children`",
    fixed = TRUE
  )
})

test_that("function-returning hooks accept render = JS(...)", {
  expect_no_error(useNavigate(render = JS("nav => nav")))
  expect_no_error(useSubmit(render = JS("submit => submit")))
  expect_no_error(useLinkClickHandler(to = "/x", render = JS("h => h")))
})

test_that("useNavigate/useSubmit reject as = 'onClick' (unsafe signature)", {
  expect_error(
    useNavigate(into = htmltools::tags$button("Go"), as = "onClick"),
    'unsafe',
    fixed = TRUE
  )
  expect_error(
    useSubmit(into = htmltools::tags$button("Save"), as = "onClick"),
    'unsafe',
    fixed = TRUE
  )
})

test_that("useLinkClickHandler accepts as = 'onClick' (MouseEvent signature)", {
  expect_no_error(
    useLinkClickHandler(
      to = "/x",
      into = htmltools::tags$span("link"),
      as = "onClick"
    )
  )
})

test_that("function-returning hooks accept other explicit non-children as", {
  expect_no_error(
    useNavigate(into = htmltools::tags$button("Go"), as = "onMouseEnter")
  )
  expect_no_error(
    useSubmit(into = htmltools::tags$button("Save"), as = "onMouseEnter")
  )
})

test_that("missing() guards on required positional args fire", {
  expect_error(useHref(), "`to` is required", fixed = TRUE)
  expect_error(useResolvedPath(), "`to` is required", fixed = TRUE)
  expect_error(useViewTransitionState(), "`to` is required", fixed = TRUE)
  expect_error(useLinkClickHandler(), "`to` is required", fixed = TRUE)
  expect_error(useMatch(), "`pattern` is required", fixed = TRUE)
  expect_error(useRouteLoaderData(), "`routeId` is required", fixed = TRUE)
})

test_that("useHref() and useResolvedPath() resolve paths", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseHrefResolvedPath")
  )
  app$wait_for_idle()

  app$get_js("window.location.hash = '#/products/42'")
  app$wait_for_idle()

  href_relative <- app$get_text("#hrefRelative")
  expect_equal(href_relative, "#/settings")

  href_absolute <- app$get_text("#hrefAbsolute")
  expect_equal(href_absolute, "#/home")

  resolved_all <- app$get_text("#resolvedAll")
  expect_true(grepl('"pathname"', resolved_all))

  resolved_pathname <- app$get_text("#resolvedPathname")
  expect_equal(resolved_pathname, "/settings")
})

test_that("useNavigationType() renders navigation type", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseNavigationType")
  )
  app$wait_for_idle()

  nav_type_text <- app$get_text("#navType")
  expect_equal(nav_type_text, "POP")
})

test_that("useMatch() renders match object or null", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseMatch")
  )
  app$wait_for_idle()

  app$get_js("window.location.hash = '#/products/7'")
  app$wait_for_idle()

  match_full <- app$get_text("#matchFull")
  expect_true(grepl('"params"', match_full))
  expect_true(grepl('"pathname"', match_full))

  match_params <- app$get_text("#matchParams")
  expect_true(grepl('"id"', match_params))
  expect_true(grepl('"7"', match_params))

  match_pathname <- app$get_text("#matchPathname")
  expect_equal(match_pathname, "/products/7")

  no_match <- app$get_text("#noMatch")
  expect_equal(no_match, "")
})

test_that("useBlocker() with default shouldBlock = FALSE constructs without error", {
  expect_no_error(
    useBlocker(into = shiny::span(), as = "children")
  )
})

test_that("useBlocker() renders blocker state", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseBlocker")
  )
  app$wait_for_idle()

  # into/as/selector path: default selector = "state"
  blocker_state <- app$get_text("#blockerState")
  expect_equal(blocker_state, "unblocked")

  # render = JS(...) path: receives full blocker object, returns b.state
  blocker_state_render <- app$get_text("#blockerStateRender")
  expect_equal(blocker_state_render, "unblocked")
})

test_that("useRevalidator() renders revalidation state", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseRevalidator")
  )
  app$wait_for_idle()

  revalidator_state <- app$get_text("#revalidatorState")
  expect_equal(revalidator_state, "idle")
})

test_that("useRouteLoaderData() accesses parent route loader data", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseRouteLoaderData")
  )
  app$wait_for_idle()

  app$get_js("window.location.hash = '#/child'")
  app$wait_for_idle()

  root_all <- app$get_text("#rootDataAll")
  expect_true(grepl('"title"', root_all))
  expect_true(grepl('"Root Data"', root_all))

  root_title <- app$get_text("#rootDataTitle")
  expect_equal(root_title, "Root Data")
})

test_that("useFetchers() renders when no fetchers are active", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "UseFetchers")
  )
  app$wait_for_idle()

  # When no fetchers are active, useFetchers() returns an empty array.
  # On slower CI runners the React span may not have populated yet on first
  # idle, so accept either the empty initial render or "[]".
  fetchers_all <- app$get_text("#fetchersAll")
  expect_true(fetchers_all %in% c("", "[]"))
})

#' Print reactRouter components
#'
#' When called interactively, renders the component in the IDE viewer panel.
#' Otherwise, falls back to standard shiny.tag printing (raw HTML text).
#'
#' Only the router-root constructors carry the \code{"reactRouter"} S3 class
#' and therefore dispatch to this method:
#' \code{\link{RouterProvider}}, \code{\link{createHashRouter}},
#' \code{\link{createBrowserRouter}}, \code{\link{createMemoryRouter}},
#' \code{\link{HashRouter}}, \code{\link{BrowserRouter}}, and
#' \code{\link{MemoryRouter}}. Inner pieces (\code{\link{Route}},
#' \code{\link{Link}}, \code{\link{Outlet}}, hooks, ...) are plain
#' \code{shiny.tag} elements -- printing one of those on its own is rarely
#' useful (it has no router context to render against), so they intentionally
#' fall through to the default \code{shiny.tag} print method.
#'
#' @param x A reactRouter object (also inherits shiny.tag).
#' @param browse Whether to render in viewer. Defaults to TRUE in interactive sessions.
#' @param ... Additional arguments passed to print.
#' @return Invisibly returns x.
#'
#' @export
print.reactRouter <- function(x, browse = interactive(), ...) {
  rendered <- FALSE
  if (browse) {
    # Fall back to plain shiny.tag printing if the htmltools render path
    # fails -- e.g. a partial install where the bundled JS dependency is
    # missing. Better an HTML dump than an opaque htmltools error from
    # what is, after all, just a print() call.
    rendered <- tryCatch({
      htmltools::html_print(htmltools::browsable(x))
      TRUE
    }, error = function(e) FALSE)
  }
  if (!rendered) {
    NextMethod("print")
  }
  invisible(x)
}

test_that("MemoryRouter renders with initialEntries", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "MemoryRouter")
  )
  app$wait_for_idle()

  about_text <- app$get_text("#memAbout")
  expect_true(grepl("memory about", about_text))
})

test_that("BrowserRouter renders at root URL", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "BrowserRouter")
  )
  app$wait_for_idle()

  content <- app$get_text("#browserRouterContent")
  expect_true(grepl("BrowserRouter active", content))

  home_text <- app$get_text("#browserHome")
  expect_true(grepl("browser home", home_text))
})

test_that("Navigate redirects to the target route", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "Navigate")
  )
  app$wait_for_idle()

  app$get_js("window.location.hash = '#/old'")
  app$wait_for_idle()

  new_text <- app$get_text("#newPage")
  expect_true(grepl("new page", new_text))
})

test_that("Link renders and routes work", {
  skip_on_cran()

  app <- shinytest2::AppDriver$new(
    app_dir = testthat::test_path("test-apps", "Link")
  )
  app$wait_for_idle()

  link_text <- app$get_text("#linkAbout")
  expect_equal(link_text, "Go to About")

  home_text <- app$get_text("#homePage")
  expect_true(grepl("home", home_text))

  app$get_js("window.location.hash = '#/about'")
  app$wait_for_idle()

  about_text <- app$get_text("#aboutPage")
  expect_true(grepl("about content", about_text))
})

---
title: "Introduction to reactRouter"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to reactRouter}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

### Install

``` r
#remotes::install_github("lgnbhl/reactRouter") # development version

install.packages("reactRouter")
```

## Minimal example

```{r setup}
library(reactRouter)
```

You can add URL pages in Quarto document or R shiny like so: 

```{r basic}
library(reactRouter)

HashRouter(
  NavLink(to = "/", "Main"),
  NavLink(to = "/analysis", "Analysis"),
  Routes(
    Route(path = "/", element = "Main content"),
    Route(path = "/analysis", element = "Analysis content")
  )
)
```

### Usage with shiny

A minimal example using [shiny](https://shiny.posit.co/).

```r
library(shiny)
library(reactRouter)

ui <- HashRouter(
  NavLink(to = "/", "Main"), 
  shiny::br(),
  NavLink(to = "/other", "Other"),
  Routes(
    Route(
      path = "/", 
      element = uiOutput(outputId = "uiMain")
    ),
    Route(
      path = "/other", 
      element = uiOutput(outputId = "uiOther")
    )
  )
)

server <- function(input, output, session) {
  output$uiMain <- renderUI( { p("Content home") } )
  output$uiOther <- renderUI( { p("Other content") })
}

shinyApp(ui = ui, server = server)
```

### Usage with bslib

A minimal example using [bslib](https://rstudio.github.io/bslib/).

```r
library(reactRouter)
library(bslib)
library(htmltools)

reactRouter::HashRouter(
  bslib::page_navbar(
    title = "reactRouter with bslib",
    nav_item(
      reactRouter::NavLink(
        "Home", 
        to = "/"
      )
    ),
    nav_item(
      reactRouter::NavLink(
        "Analysis", 
        to = "/analysis"
      )
    ),
    reactRouter::Routes(
      reactRouter::Route(
        path = "/",
        element = div(
          tags$h3("Home page"),
          p("A basic example of reactRouter with bslib.")
        )
      ),
      reactRouter::Route(
        path = "/analysis",
        element = "Content analysis"
      ),
      reactRouter::Route(path = "*", element = "Custom error 404")
    )
  )
)
```

<img style="border:1px solid black;" src="https://raw.githubusercontent.com/lgnbhl/reactRouter/master/man/figures/reactRouter-with-bslib.png" align="center" />

### Usage with shinyMaterialUI

A minimal example using [shinyMaterialUI](https://felixluginbuhl.com/shinyMaterialUI/).

```r
# remotes::install_github("lgnbhl/shinyMaterialUI")
library(shinyMaterialUI)

HashRouter(
  Box(
    sx = list(flexGrow = 1),
    AppBar(
      position = "static",
      Toolbar(
        Typography(
          variant = "h6",
          component = "div",
          sx = list(mr = 1),
          "shinyMaterialUI"
        ),
        NavLink(
          to = "/",
          Button(
            color = "inherit",
            "Home"
          )
        ),
        NavLink(
          to = "analysis",
          Button(
            color = "inherit",
            "Analysis"
          )
        )
      )
    ),
      Box(
        reactRouter::Routes(
          reactRouter::Route(
            path = "/",
            element = Box("Home page", sx = list(p = 1))
          ),
          reactRouter::Route(
            path = "/analysis",
            element = Box("Content analysis", sx = list(p = 1))
          ),
          reactRouter::Route(path = "*", element = "Error 404")
        )
      )
  )
)
```

<img style="border:1px solid black;" src="https://raw.githubusercontent.com/lgnbhl/reactRouter/master/man/figures/reactRouter-with-shinyMaterialUI.png" align="center" />

Find more examples with **shinyMaterialUI** [here](https://felixluginbuhl.com/shinyMaterialUI/).

### Usage with Shiny modules

```r
# adapted from example of shiny.router
# https://github.com/Appsilon/shiny.router/tree/main/examples/shiny_modules
library(shiny)
library(reactRouter)

# This creates UI for each page.
page <- function(title, content, id) {
  ns <- NS(id)
  div(
    titlePanel(title),
    p(content),
    textOutput(ns("click_me"))
  )
}

# Both sample pages.
root_page <- page("Home page", "Home page clicks", "root")
second_page <- page("Other page", "Other page clicks", "second")

server_module <- function(id, clicks, power = 1) {
  moduleServer(id, function(input, output, session) {
    output$click_me <- renderText({
      as.numeric(clicks())^power
    })
  })
}

# Create output for our router in main UI of Shiny app.
ui <- reactRouter::HashRouter(
  NavLink(to = "/", "Main"), br(),
  NavLink(to = "/other", "Other"),
  actionButton("clicks", "Click me!"),
  Routes(
    Route(
      path = "/", 
      element = div(
        root_page
      )
    ),
    Route(
      path = "/other", 
      element = div(
        second_page
      )
    )
  )
)

# Plug router into Shiny server.
server <- function(input, output, session) {
  clicks <- reactive({
    input$clicks
  })
  server_module("root", clicks = clicks, power = 1)
  server_module("second", clicks = clicks, power = 2)
}

# Run server in a standard way.
shinyApp(ui, server)
```

### Example with Quarto

As React Router provides client routing, you can easily create multiple routes in a Quarto or R markdown documents:

``` {r}
# code to run in a Quarto document
# example adapted from: https://github.com/remix-run/react-router/tree/dev/examples/basic
library(reactRouter)
library(htmltools)

Layout <- div(
  # A "layout route" is a good place to put markup you want to
  # share across all the pages on your site, like navigation.
  tags$nav(
    tags$ul(
      tags$li(
        reactRouter::Link(to = "/", "Home")
      ),
      tags$li(
        reactRouter::Link(to = "/dashboard", "Dashboard")
      ),
      tags$li(
        reactRouter::Link(to = "/nothing-here", "Nothing Here")
      )
    )
  ),
  tags$hr(),
  # An <Outlet> renders whatever child route is currently active,
  # so you can think about this <Outlet> as a placeholder for
  # the child routes we defined above.
  reactRouter::Outlet()
)

reactRouter::HashRouter(
  div(
    style = "border:1px solid black;", # add border just for the example
    h1("Basic Example"),
    tags$p(
      paste0('This example demonstrates some of the core features of React Router
          including nested reactRouter::Route(), reactRouter::Outlet(), 
          reactRouter::Link(), and using a "*" route (aka "splat route") 
          to render a "not found" page when someone visits an unrecognized URL.'
      )
    ),
    reactRouter::Routes(
      Route(
        path = "/",
        element = Layout,
        Route(
          index = TRUE,
          element = div(
            tags$h2("Home"),
            tags$p("Home content")
          )
        ),
        Route(
          path = "dashboard",
          element = div(
            tags$h2("Dashboard"),
            tags$p("Dashboard here")
          )
        ),
        # Using path="*"" means "match anything", so this route
        # acts like a catch-all for URLs that we don't have explicit
        # routes for.
        Route(
          path = "*",
          element = div(
            tags$h2("Nothing to see here!"),
            tags$p(
              Link(to = "/", "Go to the home page")
            )
          )
        )
      )
    )
  )
)
```

### Dynamic segments

A minimal example using dynamic segments, i.e. using `Route(to = ":id/*")`.

```r
library(shiny)
library(reactRouter)
library(bslib)

ui <- HashRouter(
  bslib::page(
    Link(
      to = "/", 
      h3("reactRouter with dynamic routes", class = "m-3"),
      style = "text-decoration: none; color: black"
    ),
    Routes(
      Route(
        path = "/",
        element = div(
          # tags$a() necessary to observe `url_hash` in session
          NavLink(
            to = "project/1/overview",
            "Project 1"
          ),
          tags$br(),
          NavLink(
            to = "project/2/overview",
            "Project 2"
          )
        )
      ),
      Route(
        path = "project/:id/*", 
        element = div(
          NavLink(
            to = "overview",
            "Overview"
          ),
          tags$br(),
          NavLink(
            to = "analysis",
            "Analysis"
          ),
          Outlet()
        ),
        children = list(
          reactRouter::Route(
            path = "overview",
            element = uiOutput("uiOverview")
          ),
          reactRouter::Route(
            path = "analysis",
            element = uiOutput("uiAnalysis")
          )
        )
      )
    )
  )
)

server <- function(input, output, session) {
  
  url_hash <- shiny::reactiveVal(value = NA)

  # update reactive values based on url hash
  observeEvent(session$clientData$url_hash, {
    current_url_hash <- session$clientData$url_hash
    print(current_url_hash)
    url_hash(current_url_hash)
  })
  
  output$uiOverview <- renderUI({
    url_hash()
  })
  output$uiAnalysis <- renderUI({
    url_hash()
  })
}

shinyApp(ui, server)
```

Run a more advanced example of dynamic routes with:

```r
reactRouterExample("dynamic-segments")
```

### Alternatives

- [shiny.router](https://appsilon.github.io/shiny.router/) implements a custom hash routing for shiny.
- [brochure](https://github.com/ColinFay/brochure) provide a mechanism for creating natively multi-page shiny applications (but is still WIP).

### More information

**reactRouter** implements React Router [v.6.30.0](https://reactrouter.com/6.30.0).

More info about how to use React Router can be found in the [official website](https://reactrouter.com/6.30.0).

---
title: "Tutorial: Using reactRouter with rhino and shiny.fluent"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Tutorial: Using reactRouter with rhino and shiny.fluent}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

This tutorial demonstrates how to build a dynamic Shiny application using `reactRouter` for routing and `shiny.fluent` for modern UI components, all within the `rhino` framework. We'll use [Dota 2](https://www.opendota.com/) API data as an example of routing multiple pages.

Initially, ensure you have the necessary rhino package installed. You can do this by running the following command in your R console:

```r
# Install rhino if not yet installed
install.packages("rhino")
```

Next, you will need to create a new rhino project. If you haven't already set up a rhino project, you can do so by running the following command in your R console:

```r
# Initialize a new rhino project (will create project scaffolding)
rhino::init()
```

This will create a basic structure for your application. Add the following libraries to your `dependencies.R` file:

```r
# dependencies.R

library(rhino) # App structure
library(httr) # API requests
library(shiny.fluent) # UI components
library(reactRouter) # Client-side routing
library(echarts4r) # Charting
library(stringdist) # String matching
library(treesitter) # Optional: Syntax parsing
library(treesitter.r) # Optional: R syntax support
```

and then

```r
renv::snapshot()
```

Now we are ready to go.

## Building the Application

In this example, we will create a simple application that displays information about Dota 2 heroes. The application will have multiple routes, allowing users to navigate between different pages.

The components of the application will be structured as follows:

- home: the main page of the application, which will display a list of heroes
- menu: a navigation menu for the application, allowing users to navigate between different pages
- header: a header component that will be displayed on every page
- details: a page containing detailed information about a specific hero
- benchmark: a page that will display benchmark statistics for heroes
- ranks: a page that will display the ranks of heroes based on their performance

The final strucutre of the `app` will look like this:

```yml
├── app
│   ├── js
│   │   └── index.js
│   ├── logic
│   │   ├── data.R
│   │   └── utils.R
│   ├── main.R
│   ├── static
│   │   ├── css
│   │   │   └── app.min.css
│   │   ├── favicon.ico
│   │   └── js
│   │       └── app.min.js
│   ├── styles
│   │   └── main.scss
│   └── view
│       ├── benchmark.R
│       ├── details.R
│       ├── header.R
│       ├── home.R
│       ├── menu.R
│       └── rank.R
├── app.R
├── config.yml
├── dependencies.R
├── renv.lock
├── rhino.yml
└── run_dev.R
```

The main part of the application to address the routing and the UI components is in the `app.R` file. 

```r
# app / main.R

box::use(
  app / view / home,
  app / view / menu,
  app / view / details,
  app / view / benchmark,
  app / view / rank
)


# Define UI with namespaced modules
ui <- function(id) {
  ns <- shiny::NS(id) # Namespace for module isolation
  shiny.fluent::fluentPage(
    reactRouter::HashRouter(
      reactRouter::Routes(
        # Home page route
        reactRouter::Route(path = "/", element = home$ui(ns("home"))),

        # Project-based nested routes
        reactRouter::Route(
          path = "/:projectId/*",
          element = menu$ui(ns("menu")), # Common layout/menu
          children = list(
            reactRouter::Route(
              path = "details",
              element = details$ui(ns("details"))
            ),
            reactRouter::Route(
              path = "benchmark",
              element = benchmark$ui(ns("benchmark"))
            ),
            reactRouter::Route(
              path = "rank",
              element = rank$ui(ns("rank"))
            )
          )
        ),

        # Fallback for undefined routes
        reactRouter::Route(path = "*", element = "Custom error 404")
      )
    )
  )
}

#' @export
server <- function(id) {
  shiny::moduleServer(id, function(input, output, session) {
    hero_selected <- home$server("home")

    shiny::observe({
      shiny::req(hero_selected())

      print(paste0("hero_id selected: ", hero_selected()))
    })

    menu$server("menu", hero_selected = hero_selected)
    details$server("details", hero_selected = hero_selected)
    benchmark$server("benchmark", hero_selected = hero_selected)
    rank$server("rank", hero_selected = hero_selected)
  })
}
```

This function defines the overall layout and routing of the application using reactRouter. It contains four key parts:

1. Top-level routing via `HashRouter()` and `Routes()`.

2. Root Route /: Displays the home page.

3. Nested Route /:projectId/*:

  1. Displays a layout (menu) and child routes (details, benchmark, rank).

  2. Each sub-route renders a different module UI (e.g., details$ui, benchmark$ui).

4. Fallback * Route: Catches any undefined paths and shows a custom 404 message.

## Running the Application

You can now run your app locally with the following script:

```r
# run_dev.R

rhino::build_js()
rhino::build_sass()
shiny::runApp(port = 4929, launch.browser = FALSE)
```

## Recap

In this tutorial, dynamic Shiny application using:

- **`rhino`** for structured project setup.
- **`shiny.fluent`** for modern and interactive UI.
- **`reactRouter`** for dynamic and nested client-side routing.

This setup provides flexibility, scalability, and maintainability for more complex Shiny applications.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.R
\name{hook-wrapper-noselector}
\alias{hook-wrapper-noselector}
\title{Documentation template for hook wrappers whose value has no internal
structure to walk (scalars, booleans, or functions). Identical to
\code{hook-wrapper} except that \code{selector} is intentionally
omitted -- it would have nothing to walk.}
\arguments{
\item{into}{A component (HTML tag or \pkg{shiny.react}-based element)
that will receive the hook value as the specified prop.}

\item{as}{Character. The name of the component's prop to inject the hook
value into. Defaults to \code{"children"}.}

\item{render}{Optional \code{\link{JS}} function \code{(value) => ReactNode}
used in place of \code{into}/\code{as}.}

\item{...}{Additional props to pass to the component.}
}
\description{
Documentation template for hook wrappers whose value has no internal
structure to walk (scalars, booleans, or functions). Identical to
\code{hook-wrapper} except that \code{selector} is intentionally
omitted -- it would have nothing to walk.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/components.R
\name{Outlet}
\alias{Outlet}
\title{Outlet}
\usage{
Outlet(...)
}
\arguments{
\item{...}{Props to pass to element.}
}
\value{
A Outlet component.
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.Outlet.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.R
\name{useRoutes}
\alias{useRoutes}
\title{useRoutes}
\usage{
useRoutes(..., routes = NULL)
}
\arguments{
\item{...}{\code{\link{Route}} elements describing the route tree.
Ignored if \code{routes} is supplied.}

\item{routes}{Optional. A \code{\link{JS}} expression evaluating to a plain
JavaScript array of route objects (e.g. \code{JS("[\{ path: '/', element: ... \}]")}),
used in place of \code{Route()} children.}
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.useRoutes.html}
}
\details{
Builds a route tree from \code{\link{Route}} children (or a plain object
\code{routes} array) and renders the matched route. The hook-based
equivalent of \code{\link{Routes}} / \code{createRoutesFromElements} for
code that prefers a configuration-as-data style. Must be called inside a
router (\code{\link{RouterProvider}}, \code{\link{HashRouter}}, etc.).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{redirect}
\alias{redirect}
\title{redirect (loader/action helper)}
\usage{
redirect(to)
}
\arguments{
\item{to}{Character. Destination path.}
}
\value{
A \code{\link{JS}} expression suitable for the \code{loader}
  argument of \code{\link{Route}}.
}
\description{
\url{https://reactrouter.com/api/utils/redirect}
}
\details{
Returns a \code{\link{JS}} loader function that redirects to \code{to}.
Pass as the \code{loader} argument of a \code{\link{Route}} to perform
an unconditional redirect -- typically used for guard routes that always
send the user somewhere else.

\strong{Security:} \code{to} must be a trusted, package-author-controlled
string. \code{javascript:}, \code{data:}, and \code{vbscript:} URL schemes
are rejected. If you build \code{to} from user-supplied input, validate it
yourself before passing it in -- never round-trip untrusted strings through
\code{redirect()} into a navigation.

For conditional redirects inside a custom loader/action, call
\code{window.jsmodule['@/reactRouter'].helpers.redirect(to)} from your own
\code{JS()} string, e.g.
\preformatted{
  loader = JS(
    "async () => {
       const { redirect } = window.jsmodule['@/reactRouter'].helpers;
       if (!authed()) return redirect('/login'); ...
     }"
  )
}

The \code{data}, \code{replace}, and \code{redirectDocument} helpers are
exposed on the same namespace.
}
\examples{
\dontrun{
Route(path = "/old", loader = redirect("/new"), element = NULL)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.R
\name{useSearchParams}
\alias{useSearchParams}
\title{useSearchParams}
\usage{
useSearchParams(into = NULL, as = "children", param = NULL, render = NULL, ...)
}
\arguments{
\item{into}{A component (HTML tag or \pkg{shiny.react}-based element)
that will receive the hook data as the specified prop.}

\item{as}{Character. The name of the component's prop to inject the hook
data into (by default \code{"children"} for text display, \code{"rows"} for
a data grid, \code{"value"} for an input).}

\item{param}{Character. Name of a single query parameter to extract.
  Returns a character vector of all values for that key (length 0 if
  absent, length 1+ otherwise). When \code{NULL}, returns a named list
  mapping each key to its vector of values.

  \strong{Empty vs. missing.} A missing key produces a length-0 vector,
  which renders as an empty string when injected with the default
  \code{as = "children"}. If you need to distinguish "absent" from
  "present-but-empty" (e.g. show a placeholder), use the \code{render}
  form and branch on \code{Array.isArray(v) && v.length === 0}, e.g.
  \code{render = JS("v => v.length ? v.join(', ') : <em>none</em>")}.}

\item{render}{Optional \code{\link{JS}} function \code{(value) => ReactNode}
used in place of \code{into}/\code{as}. Mirrors the native React Router
pattern for cases where a single prop is not expressive enough (e.g.
\code{JS("v => `${v.first} ${v.last}`")}).}

\item{...}{Additional props to pass to the component.}
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.useSearchParams.html}
}
\details{
Calls the \code{useSearchParams()} hook and injects the result
\code{as} a prop of the \code{into} component. Use the \code{param}
argument to extract a query parameter by name.

Values are always returned as character vectors so that repeated keys
(e.g. \code{"?tag=a&tag=b"}) are preserved. When injected as
\code{"children"}, vectors are joined with \code{", "}; for custom
formatting, use \code{render}.

\strong{Reading vs. writing.} The upstream JS hook returns a tuple
\code{[searchParams, setSearchParams]}. This wrapper splits the two paths:
\itemize{
  \item \code{into} / \code{as} — \emph{read-only}. Receives the parsed
    params (or one \code{param}) and ignores the setter.
  \item \code{render} — receives both as \code{(params, setSearchParams)},
    so use this form when you need to update the URL programmatically:
    \preformatted{
  useSearchParams(render = JS(
    "(p, set) => <button onClick={() => set({tag:'b'})}>Filter</button>"
  ))
    }
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.R
\name{useNavigate}
\alias{useNavigate}
\title{useNavigate}
\usage{
useNavigate(into = NULL, as = "children", render = NULL, ...)
}
\arguments{
\item{into}{A component (HTML tag or \pkg{shiny.react}-based element)
that will receive the hook value as the specified prop.}

\item{as}{Character. The name of the component's prop to inject the hook
value into. Defaults to \code{"children"}.}

\item{render}{Optional \code{\link{JS}} function \code{(value) => ReactNode}
used in place of \code{into}/\code{as}.}

\item{...}{Additional props to pass to the component.}
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.useNavigate.html}
}
\details{
Calls the \code{useNavigate()} hook and passes the navigate function
to \code{render} (or injects it \code{as} a prop of \code{into}).
The navigate function has signature \code{navigate(to, options?)}, e.g.
\code{navigate("/about")} or \code{navigate(-1)} to go back.

Because the hook returns a function (not a value), the \code{render}
form is the natural way to use it:
\preformatted{
  useNavigate(render = JS(
    "nav => <button onClick={() => nav('/about')}>Go</button>"
  ))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.R
\name{useFetcher}
\alias{useFetcher}
\title{useFetcher}
\usage{
useFetcher(
  into = NULL,
  as = "children",
  selector = "state",
  render = NULL,
  fetcherKey = NULL,
  ...
)
}
\arguments{
\item{into}{A component (HTML tag or \pkg{shiny.react}-based element)
that will receive the hook data as the specified prop.}

\item{as}{Character. The name of the component's prop to inject the hook
data into (by default \code{"children"} for text display, \code{"rows"} for
a data grid, \code{"value"} for an input).}

\item{selector}{Character. Optional key to extract from the hook data object.
If \code{NULL} (the default), the entire data is passed. Dotted paths
like \code{"summary.title"} navigate nested objects.}

\item{render}{Optional \code{\link{JS}} function \code{(value) => ReactNode}
used in place of \code{into}/\code{as}. Mirrors the native React Router
pattern for cases where a single prop is not expressive enough (e.g.
\code{JS("v => `${v.first} ${v.last}`")}).}

\item{fetcherKey}{Character. Optional key to share a fetcher across
components (e.g. \code{"my-fetcher"}).}

\item{...}{Additional props to pass to the component.}
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.useFetcher.html}
}
\details{
Calls the \code{useFetcher()} hook and injects the result (or a
\code{selector} from it) \code{as} a prop of the \code{into} component.
Use to fetch data or submit forms without causing a navigation.
The fetcher object has \code{state} (\code{"idle"}/\code{"loading"}/\code{"submitting"}) and
\code{data} (the response from a loader or action).


\code{selector} defaults to \code{"state"} so the default \code{into}/
\code{children} display shows a readable string (\code{"idle"} /
\code{"loading"} / \code{"submitting"}). The full fetcher object contains
methods (\code{submit}, \code{load}, \code{Form}) that would be silently
dropped by JSON serialization if the whole object were rendered as
children. To call those methods, use the \code{render = JS(...)} form,
which receives the full fetcher: \code{render = JS("f => <button onClick={() => f.load('/data')}>Reload</button>")}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{redirectDocument}
\alias{redirectDocument}
\title{redirectDocument (loader/action helper)}
\usage{
redirectDocument(to)
}
\arguments{
\item{to}{Character. Destination path or absolute URL.}
}
\value{
A \code{\link{JS}} expression suitable for the \code{loader}
  argument of \code{\link{Route}}.
}
\description{
\url{https://reactrouter.com/api/utils/redirectDocument}
}
\details{
Returns a \code{\link{JS}} loader function that performs a \emph{document}
redirect to \code{to} -- a full page reload, as opposed to the client-side
navigation that \code{\link{redirect}} performs. Use when navigating to a
URL outside the router's control (e.g. a server-rendered page) so the
browser fully unloads the SPA.

For conditional document redirects inside a custom loader/action, call
\code{window.jsmodule['@/reactRouter'].helpers.redirectDocument(to)} from
your own \code{JS()} string.
}
\examples{
\dontrun{
Route(
  path = "/docs",
  loader = redirectDocument("/static/docs/index.html"),
  element = NULL
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.R
\name{Await}
\alias{Await}
\title{Await}
\usage{
Await(
  into = NULL,
  as = "children",
  resolveKey,
  selector = NULL,
  render = NULL,
  errorElement = NULL,
  fallback = NULL,
  ...
)
}
\arguments{
\item{into}{A component (HTML tag or \pkg{shiny.react}-based element)
that will receive the hook data as the specified prop.}

\item{as}{Character. The name of the component's prop to inject the hook
data into (by default \code{"children"} for text display, \code{"rows"} for
a data grid, \code{"value"} for an input).}

\item{resolveKey}{Character. The key in the loader's return value that holds
the promise (e.g. if the loader returns \code{\{ data: promise \}},
set \code{resolveKey = "data"}).}

\item{selector}{Character. Optional key to extract from the hook data object.
If \code{NULL} (the default), the entire data is passed. Dotted paths
like \code{"summary.title"} navigate nested objects.}

\item{render}{Optional \code{\link{JS}} function \code{(value) => ReactNode}
used in place of \code{into}/\code{as}. Mirrors the native React Router
pattern for cases where a single prop is not expressive enough (e.g.
\code{JS("v => `${v.first} ${v.last}`")}).}

\item{errorElement}{Element to render if the promise rejects.}

\item{fallback}{Element shown while the promise is pending. Defaults to a
plain \code{"Loading\u2026"} span.}

\item{...}{Additional props to pass to the component.}
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.Await.html}
}
\details{
Renders \code{into} when a deferred loader promise resolves, injecting the
resolved value (or a \code{selector} from it) \code{as} a prop.
Use inside a \code{\link{Route}} whose \code{loader} returns an object
containing a promise (written via \code{\link{JS}}). In React Router v7,
simply return the object directly -- no \code{defer()} wrapper is needed.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reactRouterExample.R
\name{reactRouterExample}
\alias{reactRouterExample}
\title{Run reactRouterExample example}
\usage{
reactRouterExample(example = NULL, ...)
}
\arguments{
\item{example}{The name of the example to run, or `NULL` to print and
invisibly return the list of available examples.}

\item{...}{Additional arguments to pass to `shiny::runApp()`.}
}
\value{
When `example` is `NULL`, invisibly returns a character vector of
  example names (also printed via `message()`). Otherwise this function
  normally does not return; interrupt R to stop the application
  (usually by pressing Ctrl+C or Esc).
}
\description{
Launch a Shiny example app or list the available examples.
Use `reactRouter::reactRouterExample("basic")` to run a showcase app.
}
\seealso{
\code{shiny.blueprint::runExample()}, which this function is
  adapted from.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.R
\name{useViewTransitionState}
\alias{useViewTransitionState}
\title{useViewTransitionState}
\usage{
useViewTransitionState(
  into = NULL,
  as = "children",
  render = NULL,
  to,
  relative = NULL,
  ...
)
}
\arguments{
\item{into}{A component (HTML tag or \pkg{shiny.react}-based element)
that will receive the hook value as the specified prop.}

\item{as}{Character. The name of the component's prop to inject the hook
value into. Defaults to \code{"children"}.}

\item{render}{Optional \code{\link{JS}} function \code{(value) => ReactNode}
used in place of \code{into}/\code{as}.}

\item{to}{Character. The destination path being transitioned to.}

\item{relative}{Optional character. Either \code{"route"} (default) or
\code{"path"}.}

\item{...}{Additional props to pass to the component.}
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.useViewTransitionState.html}
}
\details{
Calls the \code{useViewTransitionState()} hook and injects the boolean
result \code{as} a prop of the \code{into} component. Returns \code{TRUE}
while a View Transitions API navigation toward \code{to} is in progress.
Pair with the \code{viewTransition} prop on \code{\link{Link}}/\code{\link{NavLink}}
to drive transition-aware styling.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/components.R
\name{component}
\alias{component}
\title{Documentation template for components}
\arguments{
\item{...}{Props to pass to the component.
The allowed props are listed below in the \bold{Details} section.}
}
\value{
Object with `shiny.tag` class suitable for use in the UI of a Shiny app.
}
\description{
Documentation template for components
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.R
\name{useLinkClickHandler}
\alias{useLinkClickHandler}
\title{useLinkClickHandler}
\usage{
useLinkClickHandler(
  into = NULL,
  as = "children",
  render = NULL,
  to,
  replace = NULL,
  state = NULL,
  target = NULL,
  preventScrollReset = NULL,
  relative = NULL,
  ...
)
}
\arguments{
\item{into}{A component (HTML tag or \pkg{shiny.react}-based element)
that will receive the hook value as the specified prop.}

\item{as}{Character. The name of the component's prop to inject the hook
value into. Defaults to \code{"children"}.}

\item{render}{Optional \code{\link{JS}} function \code{(value) => ReactNode}
used in place of \code{into}/\code{as}.}

\item{to}{Character. Destination path.}

\item{replace}{Optional boolean. Replace the current entry in the history
stack instead of pushing a new one.}

\item{state}{Optional. State value to attach to the new location.}

\item{target}{Optional character. Anchor target (e.g. \code{"_blank"}).}

\item{preventScrollReset}{Optional boolean. If \code{TRUE}, do not reset
scroll position on navigation.}

\item{relative}{Optional character. Either \code{"route"} (default) or
\code{"path"}.}

\item{...}{Additional props to pass to the component.}
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.useLinkClickHandler.html}
}
\details{
Calls the \code{useLinkClickHandler()} hook and exposes the returned click
handler function via \code{render} (or injects it \code{as} a prop of
\code{into}, e.g. \code{as = "onClick"}). Lets you build link-like
components that drive client-side navigation without using
\code{\link{Link}}.

Because the hook returns a function, the \code{render} form is the natural
fit:
\preformatted{
  useLinkClickHandler(
    to = "/about",
    render = JS("h => <span onClick={h} role='link'>About</span>")
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RouterProvider.R
\name{createRoutesFromElements}
\alias{createRoutesFromElements}
\title{createRoutesFromElements}
\usage{
createRoutesFromElements(...)
}
\arguments{
\item{...}{\code{\link{Route}} elements.}
}
\value{
A tag list of Route elements.
}
\description{
\url{https://api.reactrouter.com/v7/variables/react-router.createRoutesFromElements.html}
}
\details{
Optional compatibility alias. In R, \code{\link{createHashRouter}},
\code{\link{createBrowserRouter}}, and \code{\link{createMemoryRouter}}
accept \code{\link{Route}} elements directly, so wrapping them in
\code{createRoutesFromElements()} is not required. The function is kept
so that examples copied verbatim from the React Router v7 documentation
(\code{createHashRouter(createRoutesFromElements(...))}) keep working.

The actual JSX-to-route-object conversion always happens on the
JavaScript side; this R function simply bundles its arguments into a
tag list.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/components.R, R/inputs.R
\name{Link}
\alias{Link}
\alias{Link.shinyInput}
\alias{updateLink.shinyInput}
\title{Link}
\usage{
Link(..., reloadDocument = FALSE)

Link.shinyInput(inputId, ..., reloadDocument = FALSE)

updateLink.shinyInput(
  session = shiny::getDefaultReactiveDomain(),
  inputId,
  ...
)
}
\arguments{
\item{...}{Props to pass to element.}

\item{reloadDocument}{Boolean. Default FALSE. Let browser handle the transition normally}

\item{inputId}{ID of the component.}

\item{session}{For \code{updateLink.shinyInput()} / \code{updateNavLink.shinyInput()}
only: the Shiny session object. Defaults to the current reactive domain.
Not used by \code{Link.shinyInput()} / \code{NavLink.shinyInput()} themselves.}
}
\value{
A Link component.
}
\description{
\url{https://api.reactrouter.com/v7/variables/react-router.Link.html}

\url{https://api.reactrouter.com/v7/variables/react-router.Link.html}

\strong{Repeat clicks.} The Shiny input value is the link's \code{to}
string. Clicking the same link twice publishes the same value, and Shiny
suppresses identical-value updates by default — your
\code{observeEvent(input$myLink, ...)} will fire only on the first click.
If you need to react to every click (e.g. logging, refreshing a panel),
bind to a counter or use \code{shiny::observeEvent(..., ignoreNULL = FALSE,
priority = "event")} alongside an explicit click counter, or wrap the
click target in a regular \code{shiny::actionButton()} that triggers the
navigation programmatically.
}
\details{
The `reloadDocument` prop controls whether clicking the link uses React
Router's client-side navigation (`FALSE`, the default) or skips it and lets
the browser handle the click natively (`TRUE`). The default is correct for
almost every use, including Shiny apps with server-rendered output
(`uiOutput`, `renderUI`, `plotOutput`, htmlwidgets) — Shiny output bindings
re-attach automatically when React Router mounts the new route's element.
See `vignette("routers", package = "reactRouter")` for details.

\strong{Two flavors.} Pick \code{Link()} for a plain navigation link
(the common case, mirroring React Router's API one-to-one). Pick
\code{Link.shinyInput()} only when you also need the click to fire a
Shiny input on the server — it adds an \code{inputId} that updates with
the link's \code{to} every time it is clicked, while still navigating.
If in doubt, use \code{Link()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RouterProvider.R
\name{RouterProvider}
\alias{RouterProvider}
\title{RouterProvider}
\usage{
RouterProvider(router, fallbackElement = NULL)
}
\arguments{
\item{router}{A router element produced by \code{\link{createHashRouter}},
\code{\link{createBrowserRouter}}, or \code{\link{createMemoryRouter}}.}

\item{fallbackElement}{Element shown while the initial route's loader is
resolving.}
}
\value{
A RouterProvider component.
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.RouterProvider.html}
}
\details{
Renders a data router. Mirrors the React Router v7 composition pattern:
pass a router built with \code{\link{createHashRouter}},
\code{\link{createBrowserRouter}}, or \code{\link{createMemoryRouter}} to
the \code{router} argument.

The underlying router is created once on mount and is not rebuilt when
\code{Route()} children change at runtime — React Router data routers own
their own navigation state and must be stable. To swap the route tree
dynamically (e.g. based on Shiny inputs), remount \code{RouterProvider}
itself, for instance by toggling its parent via \code{shiny::renderUI}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/components.R
\name{Routes}
\alias{Routes}
\title{Routes}
\usage{
Routes(...)
}
\arguments{
\item{...}{Props to pass to element.}
}
\value{
A Routes component.
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.Routes.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/components.R
\name{Route}
\alias{Route}
\title{Route}
\usage{
Route(
  ...,
  element,
  loader = NULL,
  action = NULL,
  errorElement = NULL,
  key = randomKey()
)
}
\arguments{
\item{...}{Additional Route props (see Details).}

\item{element}{The element to render when the route matches. Wrapped on
the JS side in a no-DOM \code{Keyed} component so React remounts the
subtree on every route change -- this is required for Shiny output
bindings (e.g. \code{textOutput()}) to reinitialise correctly when two
routes render the same component shape with different namespaces.
Unlike the previous \code{<div>} wrapper, \code{Keyed} adds no DOM node,
so layouts like MUI \code{Grid} that require typed direct children keep
working.}

\item{loader}{Optional. A \code{\link{JS}} expression evaluating to a
loader function, e.g. \code{JS("({ params }) => fetch(...)")}. For a
plain unconditional redirect, use \code{\link{redirect}}. To embed
static R data, serialize it first with \code{jsonlite::toJSON()} and
wrap the result in \code{JS()}.}

\item{action}{Optional. A \code{\link{JS}} expression evaluating to an
action function called by \code{\link{Form}} submissions and
\code{\link{useSubmit}} / \code{\link{useFetcher}} submits.}

\item{errorElement}{Optional. Element rendered when the route's
\code{loader}, \code{action}, or rendering throws.}

\item{key}{Stable React key for the route's \code{element}. Defaults to
a random alphanumeric string per \code{Route()} call, so each route's
element has a distinct identity from its siblings -- this is what
causes React to unmount the previous route's subtree (and reinitialise
its Shiny output bindings) when navigating between routes that render
the same component shape. You almost never need to pass this
explicitly. \strong{Note:} this key only affects identity of the
route's \code{element}; it does \emph{not} rebuild the
\code{\link{RouterProvider}} data router. The route tree is created
once on mount and subsequent \code{Route()} edits are ignored; to
apply a new route tree at runtime, give \code{RouterProvider} itself
a changing \code{key} (e.g. via \code{shiny::renderUI}).}
}
\value{
A Route component.
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.Route.html}
}
\details{
Internally the `element` is wrapped in a `shiny::div()`
with a UUID key so, in case R shiny is used, shiny can differentiate
each element.

Additional React Router \code{Route} props can be passed through \code{...}:
\itemize{
  \item \code{path} (Character): path pattern, supports \code{:param},
    optional \code{:param?}, and splat \code{*}.
  \item \code{index} (Boolean): mark this as the index route of its parent.
  \item \code{caseSensitive} (Boolean): match the path case-sensitively.
  \item \code{id} (Character): stable route id, required for use with
    \code{\link{useRouteLoaderData}}.
  \item \code{handle} (Any): arbitrary value exposed via
    \code{\link{useMatches}} for breadcrumbs and similar use cases.
  \item \code{shouldRevalidate} (\code{\link{JS}}): function controlling
    whether the loader re-runs on a given navigation.
  \item \code{lazy} (\code{\link{JS}}): code-splitting hook returning a
    \code{Promise} resolving to a route module.
  \item \code{hasErrorBoundary} (Boolean): explicit error-boundary flag
    (rarely needed when \code{errorElement} is provided).
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{isRouteErrorResponse}
\alias{isRouteErrorResponse}
\title{isRouteErrorResponse}
\usage{
isRouteErrorResponse()
}
\value{
A \code{\link{JS}} expression evaluating to the
  \code{isRouteErrorResponse} function reference.
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.isRouteErrorResponse.html}
}
\details{
Returns a \code{\link{JS}} reference to the \code{isRouteErrorResponse}
type guard. Use it inside an \code{errorElement} render callback to branch
on whether the error came from a thrown \code{Response}
(e.g. \code{throw new Response(..., \{ status: 404 \})}) or from arbitrary
code. Pair with \code{\link{useRouteError}}.

Calling \code{isRouteErrorResponse()} from R returns a \code{\link{JS}}
expression that evaluates, in the browser, to the upstream
\code{isRouteErrorResponse} function. Interpolate it inside the
\code{render} string of \code{useRouteError()} as shown below.

For convenience, the same function is also reachable inside any
user-authored \code{\link{JS}} string as
\code{window.jsmodule['@/reactRouter'].helpers.isRouteErrorResponse}.
}
\examples{
\dontrun{
useRouteError(render = JS(paste0(
  "e => ", isRouteErrorResponse(),
  "(e) ? <p>HTTP {e.status}</p> : <p>Unknown error</p>"
)))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/components.R
\name{ScrollRestoration}
\alias{ScrollRestoration}
\title{ScrollRestoration}
\usage{
ScrollRestoration(...)
}
\arguments{
\item{...}{Props to pass to element. Notable props: \code{getKey} (a
\code{\link{JS}} function to compute the scroll key from the location)
and \code{storageKey} (Character, custom \code{sessionStorage} key).}
}
\value{
A ScrollRestoration component.
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.ScrollRestoration.html}
}
\details{
Emulates the browser's scroll restoration on location changes after loaders
have completed. Place once inside the root layout of a data router app.
Requires a data router (\code{\link{createBrowserRouter}},
\code{\link{createHashRouter}}, etc.).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/browser.R
\name{print.reactRouter}
\alias{print.reactRouter}
\title{Print reactRouter components}
\usage{
\method{print}{reactRouter}(x, browse = interactive(), ...)
}
\arguments{
\item{x}{A reactRouter object (also inherits shiny.tag).}

\item{browse}{Whether to render in viewer. Defaults to TRUE in interactive sessions.}

\item{...}{Additional arguments passed to print.}
}
\value{
Invisibly returns x.
}
\description{
When called interactively, renders the component in the IDE viewer panel.
Otherwise, falls back to standard shiny.tag printing (raw HTML text).
}
\details{
Only the router-root constructors carry the \code{"reactRouter"} S3 class
and therefore dispatch to this method:
\code{\link{RouterProvider}}, \code{\link{createHashRouter}},
\code{\link{createBrowserRouter}}, \code{\link{createMemoryRouter}},
\code{\link{HashRouter}}, \code{\link{BrowserRouter}}, and
\code{\link{MemoryRouter}}. Inner pieces (\code{\link{Route}},
\code{\link{Link}}, \code{\link{Outlet}}, hooks, ...) are plain
\code{shiny.tag} elements -- printing one of those on its own is rarely
useful (it has no router context to render against), so they intentionally
fall through to the default \code{shiny.tag} print method.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{dataResponse}
\alias{dataResponse}
\title{dataResponse (loader/action helper)}
\usage{
dataResponse(value, init = NULL)
}
\arguments{
\item{value}{The payload to expose via \code{useLoaderData()} /
\code{useActionData()}. Either an R object (list, vector, data.frame --
serialized to JSON), or a \code{\link{JS}} expression for a JavaScript
value.}

\item{init}{Optional. Either a list with \code{status} (integer),
\code{statusText} (character) and/or \code{headers} (named list), or a
\code{\link{JS}} expression evaluating to such an object.}
}
\value{
A \code{\link{JS}} expression suitable for the \code{loader} or
  \code{action} argument of \code{\link{Route}}.
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.data.html}
}
\details{
Returns a \code{\link{JS}} loader function that resolves to a React Router
\code{data()} response -- a thin wrapper that lets you attach an HTTP
\code{status}, \code{statusText}, and/or \code{headers} alongside the
loader/action payload while still exposing \code{value} via
\code{\link{useLoaderData}} / \code{\link{useActionData}}.

Use the R helper for static loaders that always return the same value plus
status. For values computed inside a custom loader/action, call
\code{window.jsmodule['@/reactRouter'].helpers.data(value, init)} directly
in your \code{JS()} string, e.g.
\preformatted{
  loader = JS("async () => {
    const { data } = window.jsmodule['@/reactRouter'].helpers;
    const rows = await fetchRows();
    return data(\{ rows \}, \{ status: 200 \});
  }")
}
}
\examples{
\dontrun{
Route(
  path = "/profile",
  loader = dataResponse(
    list(name = "Ada", role = "Engineer"),
    init = list(status = 200)
  ),
  element = useLoaderData(tags$pre())
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.R
\name{useAsyncError}
\alias{useAsyncError}
\title{useAsyncError}
\usage{
useAsyncError(
  into = NULL,
  as = "children",
  selector = NULL,
  render = NULL,
  ...
)
}
\arguments{
\item{into}{A component (HTML tag or \pkg{shiny.react}-based element)
that will receive the hook data as the specified prop.}

\item{as}{Character. The name of the component's prop to inject the hook
data into (by default \code{"children"} for text display, \code{"rows"} for
a data grid, \code{"value"} for an input).}

\item{selector}{Character. Optional key to extract from the hook data object.
If \code{NULL} (the default), the entire data is passed. Dotted paths
like \code{"summary.title"} navigate nested objects.}

\item{render}{Optional \code{\link{JS}} function \code{(value) => ReactNode}
used in place of \code{into}/\code{as}. Mirrors the native React Router
pattern for cases where a single prop is not expressive enough (e.g.
\code{JS("v => `${v.first} ${v.last}`")}).}

\item{...}{Additional props to pass to the component.}
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.useAsyncError.html}
}
\details{
Calls the \code{useAsyncError()} hook and injects the rejection reason
(or a \code{selector} from it) \code{as} a prop of the \code{into}
component. Must be rendered inside the \code{errorElement} of an
\code{\link{Await}} so the hook can pick up the rejected promise's error.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RouterProvider.R
\name{createBrowserRouter}
\alias{createBrowserRouter}
\title{createBrowserRouter}
\usage{
createBrowserRouter(...)
}
\arguments{
\item{...}{\code{\link{Route}} elements. Pass directly, or (optionally)
wrapped in \code{\link{createRoutesFromElements}} to mirror the
official React Router v7 API.}
}
\value{
A createBrowserRouter component.
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.createBrowserRouter.html}
}
\details{
Creates a browser router using the data router API.
Use with \code{\link{createRoutesFromElements}} and \code{\link{Route}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RouterProvider.R
\name{createMemoryRouter}
\alias{createMemoryRouter}
\title{createMemoryRouter}
\usage{
createMemoryRouter(...)
}
\arguments{
\item{...}{\code{\link{Route}} elements. Pass directly, or (optionally)
wrapped in \code{\link{createRoutesFromElements}} to mirror the
official React Router v7 API.}
}
\value{
A createMemoryRouter component.
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.createMemoryRouter.html}
}
\details{
Creates a memory router using the data router API. Routing state is kept
in memory and the browser URL is never read or modified, making it suitable
for static HTML pages (\code{file://}), Quarto documents, and embedded
widgets where the real URL is irrelevant.
Use with \code{\link{createRoutesFromElements}} and \code{\link{Route}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{replaceResponse}
\alias{replaceResponse}
\title{replaceResponse (loader/action helper)}
\usage{
replaceResponse(to)
}
\arguments{
\item{to}{Character. Destination path.}
}
\value{
A \code{\link{JS}} expression suitable for the \code{loader}
  argument of \code{\link{Route}}.
}
\description{
\url{https://reactrouter.com/api/utils/replace}
}
\details{
Returns a \code{\link{JS}} loader function that performs a \emph{replace}
navigation to \code{to} -- same as \code{\link{redirect}}, but the new
entry replaces the current one in the history stack instead of pushing
a new one. Use for "alias" routes where the original URL should not
remain in the user's back-history.

Renamed from \code{replace()} to avoid masking \code{base::replace}.
This mirrors the \code{dataResponse()} naming for the same reason.

For conditional replacements inside a custom loader/action, call
\code{window.jsmodule['@/reactRouter'].helpers.replace(to)} from your own
\code{JS()} string.
}
\examples{
\dontrun{
Route(path = "/legacy", loader = replaceResponse("/new"), element = NULL)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.R
\name{useNavigationType}
\alias{useNavigationType}
\title{useNavigationType}
\usage{
useNavigationType(into = NULL, as = "children", render = NULL, ...)
}
\arguments{
\item{into}{A component (HTML tag or \pkg{shiny.react}-based element)
that will receive the hook value as the specified prop.}

\item{as}{Character. The name of the component's prop to inject the hook
value into. Defaults to \code{"children"}.}

\item{render}{Optional \code{\link{JS}} function \code{(value) => ReactNode}
used in place of \code{into}/\code{as}.}

\item{...}{Additional props to pass to the component.}
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.useNavigationType.html}
}
\details{
Calls the \code{useNavigationType()} hook and injects the result
\code{as} a prop of the \code{into} component.
Returns one of \code{"POP"}, \code{"PUSH"}, or \code{"REPLACE"}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependencies.R
\name{reactRouterDependency}
\alias{reactRouterDependency}
\title{react-router-dom JS dependency}
\usage{
reactRouterDependency()
}
\value{
HTML dependency object.
}
\description{
react-router-dom JS dependency
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{reactRouterHelpers}
\alias{reactRouterHelpers}
\title{JavaScript helpers exposed on \code{window.jsmodule['@/reactRouter'].helpers}}
\description{
For convenience, the package exposes a curated set of React Router v7
utility functions under \code{window.jsmodule['@/reactRouter'].helpers}
so user-authored \code{\link{JS}} loaders, actions, and render callbacks
can call them without reaching into webpack internals. The same names are
available as on the JS side of React Router itself.
}
\details{
Scoped under the \pkg{shiny.react} \code{jsmodule} registry rather than
\code{window} so the package never adds a top-level global to the host
page. A short alias is the only ergonomic cost:
\preformatted{
  loader = JS("async () => {
    const { redirect } = window.jsmodule['@/reactRouter'].helpers;
    if (!await ok()) return redirect('/login');
    return { ok: true };
  }")
}

\strong{Loader/action response helpers}
\itemize{
  \item \code{redirect(to, init?)} -- client-side navigation redirect.
  \item \code{replace(to, init?)} -- redirect that replaces the history entry.
  \item \code{redirectDocument(to)} -- full document reload redirect.
  \item \code{data(value, init?)} -- attach status/headers to a payload.
}

\strong{Path / URL utilities}
\itemize{
  \item \code{generatePath(path, params)} -- build a URL from a pattern.
  \item \code{matchPath(pattern, pathname)} -- match a pathname.
  \item \code{matchRoutes(routes, location, basename?)} -- match a route
    tree against a location.
  \item \code{resolvePath(to, fromPathname?)} -- resolve a relative path.
  \item \code{parsePath(path)} -- split into \code{pathname/search/hash}.
  \item \code{createPath(parts)} -- inverse of \code{parsePath}.
  \item \code{createSearchParams(init)} -- build a \code{URLSearchParams}.
}

These run in the browser only (inside \code{\link{JS}} loader/action code).
For R-side construction of a \code{Link()} \code{to} value from data, use
base R: \code{paste0("/users/", utils::URLencode(id, reserved = TRUE))}.

\strong{Error helpers}
\itemize{
  \item \code{isRouteErrorResponse(error)} -- type guard intended for use
    inside an \code{errorElement} alongside \code{\link{useRouteError}};
    returns \code{true} when the error came from a thrown
    \code{Response} (e.g. \code{throw new Response(..., { status: 404 })}).
}

These are the JavaScript implementations from \code{react-router-dom}.
}
\examples{
\dontrun{
# Conditional redirect inside a custom loader.
Route(
  path = "/admin",
  loader = JS("async () => {
    const { redirect } = window.jsmodule['@/reactRouter'].helpers;
    const ok = await checkAuth();
    if (!ok) return redirect('/login');
    return { ok: true };
  }"),
  element = useLoaderData(tags$pre())
)

# Branch on whether the route error is a Response.
Route(
  path = "/items/:id",
  loader = JS("async ({ params }) => {
    const r = await fetch('/api/items/' + params.id);
    if (!r.ok) throw new Response('Not found', { status: 404 });
    return r.json();
  }"),
  errorElement = useRouteError(
    render = JS(
      "e => window.jsmodule['@/reactRouter'].helpers.isRouteErrorResponse(e)" +
      " ? <p>HTTP {e.status}</p> : <p>Unknown error</p>"
    )
  ),
  element = useLoaderData(tags$pre())
)
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.R
\name{useSubmit}
\alias{useSubmit}
\title{useSubmit}
\usage{
useSubmit(into = NULL, as = "children", render = NULL, ...)
}
\arguments{
\item{into}{A component (HTML tag or \pkg{shiny.react}-based element)
that will receive the hook value as the specified prop.}

\item{as}{Character. The name of the component's prop to inject the hook
value into. Defaults to \code{"children"}.}

\item{render}{Optional \code{\link{JS}} function \code{(value) => ReactNode}
used in place of \code{into}/\code{as}.}

\item{...}{Additional props to pass to the component.}
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.useSubmit.html}
}
\details{
Calls the \code{useSubmit()} hook and passes the submit function to
\code{render} (or injects it \code{as} a prop of \code{into}).
The submit function has signature \code{submit(target, options?)} and
triggers a form submission (including calling the route's \code{action})
without requiring a \code{\link{Form}} element.
Only works inside a data router.

Because the hook returns a function (not a value), the \code{into} form
is rarely useful here -- prefer \code{render = JS(...)} so you can call
the submit function from inside the rendered element.
}
\examples{
\dontrun{
useSubmit(render = JS(
  "submit => <button onClick={() =>
     submit({ intent: 'delete' }, { method: 'post' })
   }>Delete</button>"
))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.R
\name{useOutlet}
\alias{useOutlet}
\title{useOutlet}
\usage{
useOutlet(into = NULL, as = "children", render = NULL, context = NULL, ...)
}
\arguments{
\item{into}{A component (HTML tag or \pkg{shiny.react}-based element)
that will receive the hook value as the specified prop.}

\item{as}{Character. The name of the component's prop to inject the hook
value into. Defaults to \code{"children"}.}

\item{render}{Optional \code{\link{JS}} function \code{(value) => ReactNode}
used in place of \code{into}/\code{as}.}

\item{context}{Optional value to expose to descendants via
\code{\link{useOutletContext}}.}

\item{...}{Additional props to pass to the component.}
}
\description{
\url{https://api.reactrouter.com/v7/functions/react-router.useOutlet.html}
}
\details{
Calls the \code{useOutlet()} hook and injects the matched child route's
element \code{as} a prop of the \code{into} component (or passes it to
\code{render}). Returns \code{NULL} when no child route matches -- useful for
rendering a fallback inside a layout when the user is on the parent route.

Differs from the \code{\link{Outlet}} component in that it returns the
element as a value, so you can branch on whether a child route is matched.
}
\examples{
\dontrun{
# In a layout route: render the matched child, or a fallback if on the
# parent route itself.
useOutlet(
  render = JS("o => o || <p>Pick a section from the menu.</p>")
)

# Inject the matched outlet element as the body of a wrapping <section>.
useOutlet(into = shiny::tags$section(class = "page"))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/components.R, R/inputs.R
\name{NavLink}
\alias{NavLink}
\alias{NavLink.shinyInput}
\alias{updateNavLink.shinyInput}
\title{NavLink}
\usage{
NavLink(..., reloadDocument = FALSE)

NavLink.shinyInput(inputId, ..., reloadDocument = FALSE)

updateNavLink.shinyInput(
  session = shiny::getDefaultReactiveDomain(),
  inputId,
  ...
)
}
\arguments{
\item{...}{Props to pass to element.}

\item{reloadDocument}{Boolean. Default FALSE Let browser handle the transition normally}

\item{inputId}{ID of the component.}

\item{session}{For \code{updateLink.shinyInput()} / \code{updateNavLink.shinyInput()}
only: the Shiny session object. Defaults to the current reactive domain.
Not used by \code{Link.shinyInput()} / \code{NavLink.shinyInput()} themselves.}
}
\value{
A NavLink component.
}
\description{
\url{https://api.reactrouter.com/v7/variables/react-router.NavLink.html}

\url{https://api.reactrouter.com/v7/variables/react-router.NavLink.html}

\strong{Repeat clicks.} The Shiny input value is the link's \code{to}
string; clicking the same link twice publishes the same value and Shiny
suppresses identical-value updates by default. See
\code{\link{Link.shinyInput}} for the workaround.
}
\details{
The `reloadDocument` prop controls whether clicking the link uses React
Router's client-side navigation (`FALSE`, the default) or skips it and lets
the browser handle the click natively (`TRUE`). The default is correct for
almost every use, including Shiny apps with server-rendered output
(`uiOutput`, `renderUI`, `plotOutput`, htmlwidgets) — Shiny output bindings
re-attach automatically when React Router mounts the new route's element.
See `vignette("routers", package = "reactRouter")` for details.
}
